<?php
/**
 * @brief		Admin CP Member Form
 * @author		<a href='https://www.invisioncommunity.com'>Invision Power Services, Inc.</a>
 * @copyright	(c) Invision Power Services, Inc.
 * @license		https://www.invisioncommunity.com/legal/standards/
 * @package		Invision Community
 * @subpackage	Members Shop
 * @since		26 Nov 2017
 */

namespace IPS\membersshop\extensions\core\MemberForm;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * Admin CP Member Form
 */
class _points
{
	/**
	 * Action Buttons
	 *
	 * @param	\IPS\Member	$member	The Member
	 * @return	array
	 */
	public function actionButtons( $member )
	{
		return array(
			'id'	=> array(
				'title'		=> 'title',
				'icon'		=> 'cog',
				'link'		=> \IPS\Http\Url::internal( "app=membersshop&module=&controller=&do=&id={$member->member_id}" ),
				'class'		=> ''
			)
		);
	}

	/**
	 * Process Form
	 *
	 * @param	\IPS\Helpers\Form		$form	The form
	 * @param	\IPS\Member				$member	Existing Member
	 * @return	void
	 */
	public function process( &$form, $member )
	{
		$pointsName = \IPS\Settings::i()->ms_points_name;
		$form->add( new \IPS\Helpers\Form\Number( 'ms_points_hand_settings', $member->shopPoints(), FALSE, array(), NULL, NULL, $pointsName ) );
		
		try
		{
			$bank = \IPS\membersshop\Bank::load( $member->member_id, 'member_id' )->points;
			$form->add( new \IPS\Helpers\Form\Number( 'ms_points_bank_settings', $bank, FALSE, array(), NULL, NULL, $pointsName ) );
		}
		catch( \Exception $e )
		{}
	}
	
	/**
	 * Save
	 *
	 * @param	array				$values	Values from form
	 * @param	\IPS\Member			$member	The member
	 * @return	void
	 */
	public function save( $values, &$member )
	{
		$loggedIn = \IPS\Member::loggedIn();
		$logsACP = sprintf( \IPS\Member::loggedIn()->language()->get('ms_acp_log_manual_edited'), $loggedIn->name );
		$member->editPoints( $values['ms_points_hand_settings'], $logsACP );
		
		try
		{
			$bank = \IPS\membersshop\Bank::load( $member->member_id, 'member_id' );
			$bank->points = $values['ms_points_bank_settings'];
			$bank->save();
		}
		catch( \Exception $e )
		{}
	}
}