<?php
/**
 * @brief		Member Sync
 * @author		<a href='https://www.invisioncommunity.com'>Invision Power Services, Inc.</a>
 * @copyright	(c) Invision Power Services, Inc.
 * @license		https://www.invisioncommunity.com/legal/standards/
 * @package		Invision Community
 * @subpackage	Members Shop
 * @since		04 Dec 2017
 */

namespace IPS\membersshop\extensions\core\MemberSync;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * Member Sync
 */
class _membersshop
{
	/**
	 * Member account has been created
	 *
	 * @param	$member	\IPS\Member	New member account
	 * @return	void
	 */
	public function onCreateAccount( $member )
	{
		if ( \IPS\Settings::i()->ms_activate_register == 1 )
		{
			$logsACP = \IPS\Member::loggedIn()->language()->get("ms_acp_log_signed_up");
			$points = \IPS\Settings::i()->ms_register_points;
			$member->addPoints( $points, $logsACP );
		}
	}
	
	/**
	 * Member is merged with another member
	 *
	 * @param	\IPS\Member	$member		Member being kept
	 * @param	\IPS\Member	$member2	Member being removed
	 * @return	void
	 */
	public function onMerge( $member, $member2 )
	{
		\IPS\Db::i()->update( 'membersshop_featured_files', array( 'member_id' => $member->member_id ), array( 'member_id=?', $member2->member_id ) );
		\IPS\Db::i()->update( 'membersshop_pinned_topics', array( 'member_id' => $member->member_id ), array( 'member_id=?', $member2->member_id ) );
		\IPS\Db::i()->update( 'membersshop_purchases', array( 'member_id' => $member->member_id ), array( 'member_id=?', $member2->member_id ) );
		\IPS\Db::i()->update( 'membersshop_rewards', array( 'member_id' => $member->member_id ), array( 'member_id=?', $member2->member_id ) );
		\IPS\Db::i()->update( 'membersshop_sent_items', array( 'sender_id' => $member->member_id ), array( 'sender_id=?', $member2->member_id ) );
		\IPS\Db::i()->update( 'membersshop_upgrades', array( 'member_id' => $member->member_id ), array( 'member_id=?', $member2->member_id ) );
		\IPS\Db::i()->update( 'membersshop_donations', array( 'donator_id' => $member->member_id ), array( 'donator_id=?', $member2->member_id ) );
		\IPS\Db::i()->update( 'membersshop_donations', array( 'receiver_id' => $member->member_id ), array( 'receiver_id=?', $member2->member_id ) );
		\IPS\Db::i()->update( 'membersshop_custompurchases', array( 'member_id' => $member->member_id ), array( 'member_id=?', $member2->member_id ) );
		\IPS\Db::i()->update( 'membersshop_bank_transactions', array( 'member_id' => $member->member_id ), array( 'member_id=?', $member2->member_id ) );
		\IPS\Db::i()->update( 'membersshop_bank', array( 'member_id' => $member->member_id ), array( 'member_id=?', $member2->member_id ) );
	}
}