<?php
/**
 * @brief		Notification Options
 * @author		<a href='https://www.invisioncommunity.com'>Invision Power Services, Inc.</a>
 * @copyright	(c) Invision Power Services, Inc.
 * @license		https://www.invisioncommunity.com/legal/standards/
 * @package		Invision Community
 * @subpackage	Members Shop
 * @since		26 Nov 2017
 */

namespace IPS\membersshop\extensions\core\Notifications;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * Notification Options
 */
class _membersshop
{
	public function getConfiguration( $member )
	{		
		$return = array(
			'stolen_rep'	=> array( 'default' => array( 'inline' ), 'disabled' => array( 'email' ) ),
			'received_item'	=> array( 'default' => array( 'inline' ), 'disabled' => array( 'email' ) ),
			'rewards_reward' => array( 'default' => array( 'inline' ), 'disabled' => array( 'email' ) ),
			'donate_donations' => array( 'default' => array( 'inline' ), 'disabled' => array( 'email' ) ),
		);
		
		if ( \IPS\Member::loggedIn()->member_id == \IPS\Settings::i()->ms_cp_notifi_members )
		{
			$return['custom_purchase_mod'] = array( 'default' => array( 'inline' ), 'disabled' => array( 'email' ) );
		}
		
		return $return;
	}
	
	public function parse_stolen_rep( \IPS\Notification\Inline $notification )
	{
		try
		{
			$member_id = $notification->item->member_id;
			$member = \IPS\Member::load( $member_id );
			$mid = $notification->item->id;
			$amount = $notification->item->reward;
			
			return array(
			    'title'         => \IPS\Member::loggedIn()->language()->addToStack('ms_notif_stolen_rep', FALSE, array( 'sprintf' => array( $member->name, $amount ) ) ),
			    'url'			=> \IPS\Http\Url::internal( 'app=membersshop&module=shop&controller=main' ),
			    'author'		=> $member,
			);
		}
		catch( \Exception $ex )
		{
			$member = \IPS\Member::loggedIn();
			return array(
			    'title'		    => 'Members Shop Expired Or Deleted Item',
			    'url'			=> '',
			    'author'		=> $member,
			);
		}
	}
	
	public function parse_received_item( \IPS\Notification\Inline $notification )
	{
		try
		{
			$item = \IPS\membersshop\Received::load( $notification->item->id );
			$member_id = $item->sender_id;
			$member = \IPS\Member::load( $member_id );
		
			return array(
			    'title'         => \IPS\Member::loggedIn()->language()->addToStack('ms_notif_sent_item', FALSE, array( 'sprintf' => array( $member->name ) ) ),
			    'url'			=> \IPS\Http\Url::internal( 'app=membersshop&module=shop&controller=myitems', 'front', 'myitems' ),
			    'author'		=> $member,
			);
		}
		catch( \Exception $ex )
		{
			$member = \IPS\Member::loggedIn();
			return array(
			    'title'		    => 'Members Shop Expired Or Deleted Item',
			    'url'			=> '',
			    'author'		=> $member,
			);
		}
	}
	
	public function parse_rewards_reward( \IPS\Notification\Inline $notification )
	{
		try
		{
			$member_id = $notification->item->member_id;
			$member = \IPS\Member::load( $member_id );
			$reward = $notification->item->reward;
			$name = $notification->item->name;
		    if ( $notification->item->item_id == 11 )
			{
				$pinned = \IPS\Db::i()->select( '*', 'membersshop_pinned_topics', array( 'member_id=? AND  expired=0', $notification->item->member_id ) )->first();
				$expire = \IPS\DateTime::create()->setTimestamp( $pinned['expire_time_date'] );
				$topic = \IPS\forums\Topic::load( $notification->item->reward );
				$title = 'You have pinned the topic '.$topic->title.' until '.$expire.'';
			}
			elseif ( $notification->item->item_id == 12 )
			{
				$file = \IPS\downloads\File::load( $notification->item->reward );
				$featured = \IPS\Db::i()->select( '*', 'membersshop_featured_files', array( 'member_id=? AND  expired=0', $notification->item->member_id ) )->first();
				$expire = \IPS\DateTime::create()->setTimestamp( $featured['expire_time_date'] );
				$title = 'You have featured the file '.$file->name.' until '.$expire.'';
			}
			elseif ( $notification->item->item_id == 13 or $notification->item->item_id == 14 or $notification->item->item_id == 15 )
			{
				$upgrade = \IPS\Db::i()->select( '*', 'membersshop_upgrades', array( 'member_id=? AND  expired=0', $notification->item->member_id ) )->first();
				$group = \IPS\Member\Group::load( $upgrade['reward_group'] );
				$expire = \IPS\DateTime::create()->setTimestamp( $upgrade['expire_time_date'] );
				$title = 'You have been upgraded to '.$group->name.' until '.$expire.'';
			}
			return array(
			    'title'		    => $title,
			    'url'			=> \IPS\Http\Url::internal( 'app=membersshop&module=shop&controller=myitems' ),
			    'author'		=> $member,
			);
		}
		catch( \Exception $ex )
		{
			$member = \IPS\Member::loggedIn();
			return array(
			    'title'		    => 'Members Shop Expired Or Deleted Item',
			    'url'			=> '',
			    'author'		=> $member,
			);
		}
	}
	
	public function parse_custom_purchase_mod( \IPS\Notification\Inline $notification )
	{
		try
		{
			$name = $notification->item->name;
			$buyer = \IPS\Member::load( $notification->item->member_id );
			
			return array(
			    'title'		    => \IPS\Member::loggedIn()->language()->addToStack('ms_notif_custom_purchase_mod', FALSE, array( 'sprintf' => array( $buyer->name, $name ) ) ),
			    'url'			=> \IPS\Http\Url::internal( 'app=membersshop&module=shop&controller=myitems' ),
			    'author'		=> $buyer,
			);
		}
		catch( \Exception $ex )
		{
			$member = \IPS\Member::loggedIn();
			return array(
			    'title'		    => 'Members Shop Expired Or Deleted Item',
			    'url'			=> '',
			    'author'		=> $member,
			);
		}
	}
	
	public function parse_donate_donations( \IPS\Notification\Inline $notification )
	{
		try
		{
			$amount = $notification->item->amount;
			$donator = \IPS\Member::load( $notification->item->donator_id );
			$pointsName = \IPS\Settings::i()->ms_points_name;
			return array(
			    'title'		    => \IPS\Member::loggedIn()->language()->addToStack('ms_notif_donation', FALSE, array( 'sprintf' => array( $donator->name, number_format( $amount ), $pointsName ) ) ),
			    'url'			=> \IPS\Http\Url::internal( 'app=membersshop&module=shop&controller=myitems' ),
			    'author'		=> $donator,
			);
		}
		catch( \Exception $ex )
		{
			$member = \IPS\Member::loggedIn();
			return array(
			    'title'		    => 'Members Shop Expired Or Deleted Item',
			    'url'			=> '',
			    'author'		=> $member,
			);
		}
	}
}