<?php
/**
 * @brief		Uninstall callback
 * @author		<a href='https://www.invisioncommunity.com'>Invision Power Services, Inc.</a>
 * @copyright	(c) Invision Power Services, Inc.
 * @license		https://www.invisioncommunity.com/legal/standards/
 * @package		Invision Community
 * @subpackage	Members Shop
 * @since		26 Nov 2017
 */

namespace IPS\membersshop\extensions\core\Uninstall;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * Uninstall callback
 */
class _uninstall
{
	public function preUninstall( $application )
	{
		\IPS\Db::i()->delete( 'core_notifications', array( 'notification_app=?', 'membersshop' ) );
	}

	public function postUninstall( $application )
	{
		try
		{
			$groupsSQL = array( 'ms_bank_interest', 'ms_daily_points', 'ms_weekly_points', 'ms_monthly_points', 'ms_can_bank', 'ms_deposit_cost' );
   		    foreach( $groupsSQL as $groups )
	        {
	            if( \IPS\Db::i()->checkForColumn( 'core_groups', $groups ) )
	            {
	                \IPS\Db::i()->dropColumn( 'core_groups', array( $groups ) );    
	            }            
	        }
        }
		catch( \Exception $ex ){}
		
		try
		{
			$forumsSQL = array( 'ms_points_starter', 'ms_points_reply', 'ms_points_topic' );
   		    foreach( $forumsSQL as $forum )
	        {
	            if( \IPS\Db::i()->checkForColumn( 'forums_forums', $forum ) )
	            {
	                \IPS\Db::i()->dropColumn( 'forums_forums', array( $forum ) );    
	            }            
	        }
        }
		catch( \Exception $ex ){}
		
		try
		{
			$downloadsSQL = array( 'ms_points_downloads', 'ms_points_upload' );
			foreach( $downloadsSQL as $download )
	        {
	            if( \IPS\Db::i()->checkForColumn( 'downloads_categories', $download ) )
	            {
	                \IPS\Db::i()->dropColumn( 'downloads_categories', array( $download ) );    
	            }            
	        }
		}
		catch( \Exception $ex ){}
		
		try
		{
			$membersSQL = array( 'shop_points' );
			foreach( $membersSQL as $member )
	        {
	            if( \IPS\Db::i()->checkForColumn( 'core_members', $member ) )
	            {
	                \IPS\Db::i()->dropColumn( 'core_members', array( $member ) );    
	            }            
	        }
		}
		catch( \Exception $ex ){}
		
		try
		{
			$reactionsSQL = array( 'reaction_shop_points' );
			foreach( $reactionsSQL as $reaction )
	        {
	            if( \IPS\Db::i()->checkForColumn( 'core_reactions', $reaction ) )
	            {
	                \IPS\Db::i()->dropColumn( 'core_reactions', array( $reaction ) );    
	            }            
	        }
		}
		catch( \Exception $ex ){}
	}
}