<?php


namespace IPS\membersshop\modules\admin\points;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * forums
 */
class _forums extends \IPS\Dispatcher\Controller
{
	/**
	 * Execute
	 *
	 * @return	void
	 */
	public function execute()
	{
		parent::execute();
	}

	/**
	 * ...
	 *
	 * @return	void
	 */
	protected function manage()
	{
		\IPS\Output::i()->title = \IPS\Member::loggedIn()->language()->addToStack('module__membersshop_points_forums');
		
		\IPS\Output::i()->sidebar['actions']['misc'] = array(
			'icon'		=> 'superpowers',
			'title'		=> 'ms_misc',
			'link'		=> \IPS\Http\Url::internal( 'app=membersshop&module=points&controller=misc' ),
			'data'		=> array( )
		);
		
		if ( \IPS\Settings::i()->clubs AND !\IPS\Settings::i()->ms_disable_clubs_hooks )
		{
			\IPS\Output::i()->sidebar['actions']['clubs'] = array(
				'icon'		=> 'users',
				'title'		=> 'ms_clubs',
				'link'		=> \IPS\Http\Url::internal( 'app=membersshop&module=points&controller=clubs' ),
				'data'		=> array( )
			);	
		}
		
		
		if ( \IPS\Application::appIsEnabled( 'calendar') AND !\IPS\Settings::i()->ms_disable_calendar_hooks )
		{
			\IPS\Output::i()->sidebar['actions']['calendar'] = array(
				'icon'		=> 'calendar',
				'title'		=> 'ms_calendar',
				'link'		=> \IPS\Http\Url::internal( 'app=membersshop&module=points&controller=calendars' ),
				'data'		=> array( )
			);	
		}
		
		if ( \IPS\Application::appIsEnabled( 'gallery') AND !\IPS\Settings::i()->ms_disable_gallery_hooks )
		{
			\IPS\Output::i()->sidebar['actions']['gallery'] = array(
				'icon'		=> 'image',
				'title'		=> 'ms_gallery',
				'link'		=> \IPS\Http\Url::internal( 'app=membersshop&module=points&controller=gallerys' ),
				'data'		=> array( )
			);	
		}
        
		if ( \IPS\Application::appIsEnabled( 'downloads') AND !\IPS\Settings::i()->ms_disable_downloads_hooks )
		{
			\IPS\Output::i()->sidebar['actions']['add'] = array(
				'icon'		=> 'download',
				'title'		=> 'ms_downloads',
				'link'		=> \IPS\Http\Url::internal( 'app=membersshop&module=points&controller=downloads' ),
				'data'		=> array( )
			);	
		}
		
		if ( \IPS\Application::appIsEnabled( 'forums') AND !\IPS\Settings::i()->ms_disable_forum_hooks )
		{
			\IPS\Output::i()->sidebar['actions']['removeondelete'] = array(
				'primary'	=> true,
				'icon'		=> 'cogs',
				'title'		=> 'ms_settings',
				'link'		=> \IPS\Http\Url::internal( 'app=membersshop&module=points&controller=forums&do=settings' ),
				'data'		=> array( 'ipsDialog' => "", 'ipsDialog-size' => "medium", 'ipsDialog-title' => \IPS\Member::loggedIn()->language()->addToStack('ms_settings') )
			);	
		}
		
		if ( \IPS\Application::appIsEnabled( 'forums') AND !\IPS\Settings::i()->ms_disable_forum_hooks )
		{
			$matrix = new \IPS\Helpers\Form\Matrix;
			
			$matrix->manageable = FALSE;
			$matrix->widths = array( 'label' => 30 );
			$matrix->langPrefix = 'ms_forums_';
			
			$matrix->columns = array(
				'label'	=> function( $key, $value, $data )
				{
					return $value;
				}
			);
			
			$matrix->columns[ 'ms_points_topic' ] = function( $key, $value, $data ) 
			{
				return ( \is_null( $value ) ) ? NULL : new \IPS\Helpers\Form\Number( 'topics_'.$key, $value, FALSE, array( 'disabled' => (bool) \is_null( $value ) ) );
			};                
			$matrix->columns[ 'ms_points_reply' ] = function( $key, $value, $data )
			{
				return ( \is_null( $value ) ) ? NULL : new \IPS\Helpers\Form\Number( 'replies_'.$key, $value, FALSE, array( 'disabled' => (bool) \is_null( $value ) ) );
			};
			$matrix->columns[ 'ms_points_starter' ] = function( $key, $value, $data )
			{
				return ( \is_null( $value ) ) ? NULL : new \IPS\Helpers\Form\Number( 'starter_'.$key, $value, FALSE, array( 'disabled' => (bool) \is_null( $value ) ) );
			};
			$matrix->columns[ 'ms_points_best_answer' ] = function( $key, $value, $data )
			{
				return ( \is_null( $value ) ) ? NULL : new \IPS\Helpers\Form\Number( 'banswer_'.$key, $value, FALSE, array( 'disabled' => (bool) \is_null( $value ) ) );
			};
			
			$rows = array();
			
			foreach( \IPS\forums\Forum::roots( NULL ) AS $root )
			{
				self::populatePermissionMatrix( $rows, $root );
			}
			
			$matrix->rows = $rows;
			
			if( $values = $matrix->values() )
			{
				if ( !empty( $matrix->changedRows ) )
				{
					foreach ( $matrix->changedRows as $key )
					{
						try
						{
							$points = \IPS\membersshop\Forums\Points::load( $key, 'forum_id' );
						}
						catch ( \OutOfRangeException $e )
						{
							$points = new \IPS\membersshop\Forums\Points;
							$points->forum_id = $key;
						}
						$points->points_per_topic = (int) $values[ $key ]['ms_points_topic'];
						$points->points_per_reply = (int) $values[ $key ]['ms_points_reply'];
						$points->points_to_ts_pr = (int) $values[ $key ]['ms_points_starter'];
						$points->points_per_best_answer = (int) $values[ $key ]['ms_points_best_answer'];
						$points->save();
					}
				}
			}
			
			\IPS\Output::i()->output = $matrix;
		}
		else
		{
			\IPS\Output::i()->output = '<div class="ipsPad ipsBox">You must have the forums app installed and the disable forum settings switched off to use this feature</div>';
		}
	}
	
	public static function populatePermissionMatrix( &$rows, $forum, $level=0 )
	{
		$rows[ $forum->_id ] = array( '_level' => $level, 'label' => $forum->_title );
        
        if( $forum->sub_can_post AND !$forum->redirect_url )
        {
			try
			{
				$points = \IPS\membersshop\Forums\Points::load( $forum->_id, 'forum_id' );
				$pointsPerTopic = $points->points_per_topic;
				$pointsPerReply = $points->points_per_reply;
				$pointsToTSPR = $points->points_to_ts_pr;
				$pointsBestAnswer = $points->points_per_best_answer;
            }
			catch ( \OutOfRangeException $e )
			{
				$pointsPerTopic = 0;
				$pointsPerReply = 0;
				$pointsToTSPR = 0;
				$pointsBestAnswer = 0;
			}
			$rows[ $forum->_id ]['ms_points_topic']   = (int) $pointsPerTopic;
    		$rows[ $forum->_id ]['ms_points_reply']   = (int) $pointsPerReply;
            $rows[ $forum->_id ]['ms_points_starter'] = (int) $pointsToTSPR;
			if ( $forum->forums_bitoptions['bw_enable_answers'] )
			{
				$rows[ $forum->_id ]['ms_points_best_answer'] = $pointsBestAnswer;
			}
        }
        
        if ( $forum->hasChildren( NULL ) === TRUE )
		{
			$level++;
			foreach( $forum->children( NULL ) AS $child )
			{
				self::populatePermissionMatrix( $rows, $child, $level );
			}
			$level--;
		}
	}
	
	protected function settings()
	{
		$form = new \IPS\Helpers\Form;
		$form->class = 'ipsForm_vertical ipsForm_fullWidth';
		$form->add( new \IPS\Helpers\Form\YesNo( 'ms_remove_on_delete', \IPS\Settings::i()->ms_remove_on_delete, TRUE, array(), NULL, NULL, NULL, 'ms_remove_on_delete' ) );
		if ( $values = $form->values() )
        {
			$form->saveAsSettings();
	        \IPS\Output::i()->redirect( \IPS\Http\Url::internal( 'app=membersshop&module=points&controller=forums' ), 'saved' );
	        return TRUE;
        }
		\IPS\Output::i()->output = $form;
	}
	
	
	protected function pointssettings()
	{
		if ( \IPS\Settings::i()->ms_remove_on_delete )
		{
			\IPS\Db::i()->update( 'core_sys_conf_settings', array( 'conf_value' => 0 ), array( 'conf_key=?', 'ms_remove_on_delete' ) );
			unset( \IPS\Data\Store::i()->settings );
		} 
		else 
		{
			\IPS\Db::i()->update( 'core_sys_conf_settings', array( 'conf_value' => 1 ), array( 'conf_key=?', 'ms_remove_on_delete' ) );
			unset( \IPS\Data\Store::i()->settings );
		}
		\IPS\Output::i()->redirect( \IPS\Http\Url::internal( 'app=membersshop&module=points&controller=forums' ), 'saved' );
	}
}