<?php


namespace IPS\membersshop\modules\admin\points;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * gallerys
 */
class _gallerys extends \IPS\Dispatcher\Controller
{
	/**
	 * Execute
	 *
	 * @return	void
	 */
	public function execute()
	{
		
		parent::execute();
	}

	/**
	 * ...
	 *
	 * @return	void
	 */
	protected function manage()
	{
		\IPS\Output::i()->title = \IPS\Member::loggedIn()->language()->addToStack('module__membersshop_points_gallery');	   
		
		\IPS\Output::i()->sidebar['actions']['misc'] = array(
			'icon'		=> 'superpowers',
			'title'		=> 'ms_misc',
			'link'		=> \IPS\Http\Url::internal( 'app=membersshop&module=points&controller=misc' ),
			'data'		=> array( )
		);	
		
		if ( \IPS\Settings::i()->clubs AND !\IPS\Settings::i()->ms_disable_clubs_hooks )
		{
			\IPS\Output::i()->sidebar['actions']['clubs'] = array(
				'icon'		=> 'users',
				'title'		=> 'ms_clubs',
				'link'		=> \IPS\Http\Url::internal( 'app=membersshop&module=points&controller=clubs' ),
				'data'		=> array( )
			);	
		}
		
		if ( \IPS\Application::appIsEnabled( 'calendar') AND !\IPS\Settings::i()->ms_disable_calendar_hooks )
		{
			\IPS\Output::i()->sidebar['actions']['calendar'] = array(
				'icon'		=> 'calendar',
				'title'		=> 'ms_calendar',
				'link'		=> \IPS\Http\Url::internal( 'app=membersshop&module=points&controller=calendars' ),
				'data'		=> array( )
			);	
		}
		
		if ( \IPS\Application::appIsEnabled( 'downloads') AND !\IPS\Settings::i()->ms_disable_downloads_hooks )
		{
			\IPS\Output::i()->sidebar['actions']['downloads'] = array(
				'icon'		=> 'download',
				'title'		=> 'ms_downloads',
				'link'		=> \IPS\Http\Url::internal( 'app=membersshop&module=points&controller=downloads' ),
				'data'		=> array( )
			);	
		}
		
		if( \IPS\Application::appIsEnabled( 'forums') AND !\IPS\Settings::i()->ms_disable_forum_hooks )
		{
			\IPS\Output::i()->sidebar['actions']['add'] = array(
				'icon'		=> 'comment-o',
				'title'		=> 'ms_forums',
				'link'		=> \IPS\Http\Url::internal( 'app=membersshop&module=points&controller=forums' ),
				'data'		=> array( )
			);
		}
		
		if( !\IPS\Settings::i()->ms_disable_gallery_hooks )
		{
			\IPS\Output::i()->sidebar['actions']['settings'] = array(
				'primary'	=> true,
				'icon'		=> 'cogs',
				'title'		=> 'ms_settings',
				'link'		=> \IPS\Http\Url::internal( 'app=membersshop&module=points&controller=gallerys&do=gallerysettings' ),
				'data'		=> array( 'ipsDialog' => '', 'ipsDialog-title' => \IPS\Member::loggedIn()->language()->addToStack('ms_settings') )
		    );
		}
		
		if ( \IPS\Application::appIsEnabled( 'gallery') AND !\IPS\Settings::i()->ms_disable_gallery_hooks )
		{
		    $matrix = new \IPS\Helpers\Form\Matrix;
        
        	$matrix->manageable = FALSE;
	        $matrix->widths = array( 'label' => 30 );
	        $matrix->langPrefix = 'ms_gallery_';
        
	        $matrix->columns = array(
				'label'	=> function( $key, $value, $data )
				{
					return $value;
				}
			);
                
	        $matrix->columns[ 'ms_points_upload' ] = function( $key, $value, $data ) 
			{
				return ( \is_null( $value ) ) ? NULL : new \IPS\Helpers\Form\Number( 'uploads_'.$key, $value, FALSE, array( 'disabled' => (bool) \is_null( $value ) ) );
			};                
			$matrix->columns[ 'ms_points_review' ] = function( $key, $value, $data )
			{
				return ( \is_null( $value ) ) ? NULL : new \IPS\Helpers\Form\Number( 'reviews_'.$key, $value, FALSE, array( 'disabled' => (bool) \is_null( $value ) ) );
			};
			$matrix->columns[ 'ms_points_comment' ] = function( $key, $value, $data )
			{
				return ( \is_null( $value ) ) ? NULL : new \IPS\Helpers\Form\Number( 'comments_'.$key, $value, FALSE, array( 'disabled' => (bool) \is_null( $value ) ) );
			};

	        $rows = array();
		
			foreach( \IPS\gallery\Category::roots( NULL ) AS $root )
			{
				self::populatePermissionMatrix( $rows, $root );
			}
		
			$matrix->rows = $rows;

	        if( $values = $matrix->values() )
   		    {
				if ( !empty( $matrix->changedRows ) )
				{
					foreach ( $matrix->changedRows as $key )
					{
						try
						{
							$points = \IPS\membersshop\Gallery\Points::load( $key, 'c_id' );
						}
						catch ( \OutOfRangeException $e )
						{
						    $points = new \IPS\membersshop\Gallery\Points;
							$points->c_id = $key;
						}
						$points->points_for_upload = (int) $values[ $key ]['ms_points_upload'];
						$points->points_for_review = (int) $values[ $key ]['ms_points_review'];
						$points->points_for_comment = (int) $values[ $key ]['ms_points_comment'];
						$points->save();
					}
				}
	        }
			
	        \IPS\Output::i()->output = $matrix;
		}
		else
		{
			\IPS\Output::i()->output = '<div class="ipsPad ipsBox">You must have the gallery app installed and the disable gallery settings switched off to use this feature</div>';
		}
	}
	
	public static function populatePermissionMatrix( &$rows, $gallery, $level=0 )
	{
		$rows[ $gallery->_id ] = array( '_level' => $level, 'label' => $gallery->_title );
        try
		{
			$points = \IPS\membersshop\Gallery\Points::load( $gallery->_id, 'c_id' );
			$pointsPerUpload = $points->points_for_upload;
			$pointsPerReview = $points->points_for_review;
			$pointsPerComment = $points->points_for_comment;
        }
		catch ( \OutOfRangeException $e )
		{
			$pointsPerUpload = 0;
			$pointsPerReview = 0;
			$pointsPerComment = 0;
		}
		$rows[ $gallery->_id ]['ms_points_upload']   = $pointsPerUpload;
    	$rows[ $gallery->_id ]['ms_points_review']   = $pointsPerReview;
		$rows[ $gallery->_id ]['ms_points_comment']   = $pointsPerComment;
        
        if ( $gallery->hasChildren( NULL ) === TRUE )
		{
			$level++;
			foreach( $gallery->children( NULL ) AS $child )
			{
				self::populatePermissionMatrix( $rows, $child, $level );
			}
			$level--;
		}
	}
	
	protected function gallerysettings()
	{
		$form = new \IPS\Helpers\Form;
		$form->add( new \IPS\Helpers\Form\Radio( 'ms_gallery_multiply_points', \IPS\Settings::i()->ms_gallery_multiply_points, TRUE, array(
		  'options' => array(
			0 	=> 'ms_gallery_multiple_no',
			1 	=> 'ms_gallery_multiple_yes'
		) ), NULL, NULL, NULL, 'ms_gallery_multiply_points' ) );
		
		
		if ( $values = $form->values() )
        {
			$form->saveAsSettings();
	        \IPS\Output::i()->redirect( \IPS\Http\Url::internal( 'app=membersshop&module=points&controller=gallerys' ), 'saved' );
	        return TRUE;
        }
		\IPS\Output::i()->output = $form;
	}
}