<?php


namespace IPS\membersshop\modules\admin\purchases;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * customitems
 */
class _customitems extends \IPS\Dispatcher\Controller
{
	/**
	 * Execute
	 *
	 * @return	void
	 */
	public function execute()
	{
		
		parent::execute();
	}

	/**
	 * ...
	 *
	 * @return	void
	 */
	protected function manage()
	{
		$table = new \IPS\Helpers\Table\Db( 'membersshop_custompurchases', \IPS\Http\Url::internal( 'app=membersshop&module=purchases&controller=customitems' ) );
		$table->langPrefix = 'ms__';
		$table->include = array( 'icon', 'member_id', 'name', 'description', 'price', 'awarded', 'time_date' );
		$table->mainColumn = 'description';
		$table->widths = array( 'icon' => '5', 'price' => '5', 'time_date' => '5', 'open' => '5', 'awarded' => '5' );
		$table->sortBy = $table->sortBy ?: 'time_date';
		$table->sortDirection = $table->sortDirection ?: 'asc';
	    $table->noSort	= array( 'icon', 'description' );
		$table->filters = array( 'ms_filter_awarded' => array( 'awarded=1' ), 'ms_filter_needs_awarding' => array( 'awarded=0' ) );
        $table->quickSearch = 'name';
		$table->advancedSearch = array(
			'member_id'	=> \IPS\Helpers\Table\SEARCH_MEMBER,
			'time_date'	=> \IPS\Helpers\Table\SEARCH_DATE_RANGE
		);
		
		$table->parsers = array(
			'awarded' => function( $val, $row )
			{
				if ( $val == 0)
				{
					$awarded = '<center><i class="fa fa-times fa-lg" aria-hidden="true" style="color:red"></i></center>';
				}
				else if ( $val == 1)
				{
				    $awarded = '<center><i class="fa fa-check fa-lg" aria-hidden="true" style="color:green"></i></center>';
				}
				return $awarded;
			},
			'icon' => function( $val, $row )
			{
				$icon = '<i class="fa '.$val.' fa-lg" aria-hidden="true"></i>';
				return $icon;
			},
			'price' => function( $val, $row )
			{
				$price = number_format($val);
				return $price;
			},
			'time_date' => function( $val, $row )
			{
				$date	= \IPS\DateTime::ts( $val );
				return $date->localeDate() . ' ' . $date->localeTime( FALSE ) ;
			},
			'description' => function( $val, $row )
			{
				return '<span data-ipsTruncate data-ipsTruncate-size="2 lines" data-ipsTruncate-type="remove">'.$val.'</span>';
			},
			'member_id' => function( $val, $row )
			{
				$member_id = \IPS\Member::load( $val );
				return "<a href='" . \IPS\Http\Url::internal( 'app=core&module=members&controller=members&do=edit&id=' ) . $row['member_id'] . "'>$member_id->name</a>";
				
			}
		);

		$table->rowButtons = function( $row )
		{
			if ( $row['awarded'] == 0 )
			{
				$return = array( 'awarded'	=> array(
					'title'	=> 'Mark Awarded',
					'icon'	=> 'star',
					'link'	=> \IPS\Http\Url::internal( 'app=membersshop&module=purchases&controller=customitems&do=award&id=' ) . $row['id'],
					'data'	=> array( 'ipsDialog' => '', 'ipsDialog-title' => \IPS\Member::loggedIn()->language()->addToStack('ms_send_pm_popup') )
				)	);
			}
			else
			{
				$return = array();
			}
			return $return;
		};
		
		\IPS\Output::i()->sidebar['actions']['notyet'] = array(
		    'primary'   => TRUE,
		    'icon'		=> 'exclamation-circle',
			'title'		=> 'ms_not_yet_redeemed',
			'link'		=> \IPS\Http\Url::internal( 'app=membersshop&module=purchases&controller=customitems&do=awaitingredemption' ),
			'data'		=> array( )
		);
		
		\IPS\Output::i()->sidebar['actions']['customcodes'] = array(
		    'icon'		=> 'exclamation-circle',
			'title'		=> 'ms_custom_codes',
			'link'		=> \IPS\Http\Url::internal( 'app=membersshop&module=purchases&controller=customcodes' ),
			'data'		=> array( )
		);
		
		\IPS\Output::i()->sidebar['actions']['normalitems'] = array(
		    'icon'		=> 'exclamation-circle',
			'title'		=> 'ms_normal_items',
			'link'		=> \IPS\Http\Url::internal( 'app=membersshop&module=purchases&controller=normalitems' ),
			'data'		=> array( )
		);
		
		/* Display */
		\IPS\Output::i()->title = \IPS\Member::loggedIn()->language()->addToStack( 'ms_custom_items' );
		\IPS\Output::i()->output	= (string) $table;
	}
	
	protected function award()
	{
		$form = new \IPS\Helpers\Form();
		$form->add( new \IPS\Helpers\Form\YesNo( 'ms_send_pm', NULL, FALSE, array('togglesOn' => array( 'ms_pm_content' ) ) ) );
		$form->add( new \IPS\Helpers\Form\Editor( 'ms_pm_content', NULL, FALSE, array( 'app' => 'core', 'key' => 'Admin', 'autoSaveKey' => NULL ), NULL, NULL, NULL, 'ms_pm_content' ) );
		
		if( $values = $form->values() )
		{
			$id  = \intval( \IPS\Request::i()->id );
			$items = \IPS\membersshop\Custom::load( $id );
			$receiver = $items->member_id;
				
			if ( $values['ms_send_pm'] )
			{		
				$msgTitle = \IPS\Member::loggedIn()->language()->addToStack( 'ms_pm_title' );
	    	    $msgPost  = $values['ms_pm_content'];
		        \IPS\Member::loggedIn()->language()->parseOutputForDisplay( $msgTitle );
		        \IPS\Member::loggedIn()->language()->parseOutputForDisplay( $msgPost );      

   	     	    try
				{
					$pmSender = \IPS\Member::load( \IPS\Member::loggedIn()->member_id );
				}
				catch( \OutOfRangeException $ex )
				{
	   	         return; 
				} 
        
		        if( !$pmSender->member_id )
		        {
		            return;
		        }            

		        $conversation = \IPS\core\Messenger\Conversation::createItem( $pmSender, $pmSender->ip_address, \IPS\DateTime::ts( time() ) );
		        $conversation->title = $msgTitle;
		        $conversation->is_system = TRUE;
		        $conversation->to_member_id = $receiver;
		        $conversation->save();
				$c_members[] = $receiver;
		        $c_members[] = $pmSender->member_id;
				$conversation->authorize( $c_members );
				$message = \IPS\core\Messenger\Message::create( $conversation, $msgPost, TRUE, NULL, NULL, $pmSender );
				$conversation->first_msg_id = $message->id;
				$conversation->save(); 
			}
			
			$items->awarded = 1;
			$items->save();
			
			\IPS\core\AdminNotification::remove( 'membersshop', 'Customitem', \IPS\Request::i()->id );
			
			\IPS\Output::i()->redirect( \IPS\Request::i()->referrer() ?: \IPS\Http\Url::internal( 'app=membersshop&module=purchases&controller=customitems' ), 'saved' );
		}
		else
		{
			\IPS\Output::i()->output = $form;
		}
	}
	
	protected function awaitingredemption()
	{
		$table = new \IPS\Helpers\Table\Db( 'membersshop_purchases', \IPS\Http\Url::internal( 'app=membersshop&module=purchases&controller=customitems&do=awaitingredemption' ) );
		$table->langPrefix = 'ms__';
		$table->include = array( 'icon', 'member_id', 'name', 'description', 'price', 'time_date' );
		$table->mainColumn = 'description';
		$table->widths = array( 'icon' => '5', 'price' => '5', 'time_date' => '5', 'open' => '5', 'redeemed' => '5' );
		$table->sortBy = $table->sortBy ?: 'time_date';
		$table->sortDirection = $table->sortDirection ?: 'desc';
	    $table->noSort	= array( 'icon', 'description' );
		$table->where[] = array( 'redeemed=0 AND item_id>99', );
        $table->quickSearch = 'name';
		$table->advancedSearch = array(
			'member_id'	=> \IPS\Helpers\Table\SEARCH_MEMBER,
			'time_date'	=> \IPS\Helpers\Table\SEARCH_DATE_RANGE
		);
		
		$table->parsers = array(
			'icon' => function( $val, $row )
			{
				$icon = '<i class="fa '.$val.' fa-lg" aria-hidden="true"></i>';
				return $icon;
			},
			'price' => function( $val, $row )
			{
				$price = number_format($val);
				return $price;
			},
			'time_date' => function( $val, $row )
			{
				$date	= \IPS\DateTime::ts( $val );
				return $date->localeDate() . ' ' . $date->localeTime( FALSE ) ;
			},
			'description' => function( $val, $row )
			{
				return $val;
			},
			'member_id' => function( $val, $row )
			{
				$member_id = \IPS\Member::load( $val );
				return "<a href='" . \IPS\Http\Url::internal( 'app=core&module=members&controller=members&do=edit&id=' ) . $row['member_id'] . "'>$member_id->name</a>";
				
			}
		);
		
		\IPS\Output::i()->sidebar['actions']['normalitems'] = array(
		    'icon'		=> 'exclamation-circle',
			'title'		=> 'ms_normal_items',
			'link'		=> \IPS\Http\Url::internal( 'app=membersshop&module=purchases&controller=normalitems' ),
			'data'		=> array( )
		);
		
		\IPS\Output::i()->sidebar['actions']['customcodes'] = array(
		    'icon'		=> 'exclamation-circle',
			'title'		=> 'ms_custom_codes',
			'link'		=> \IPS\Http\Url::internal( 'app=membersshop&module=purchases&controller=customcodes' ),
			'data'		=> array( )
		);
		
		\IPS\Output::i()->sidebar['actions']['awaiting'] = array(
		    'icon'		=> 'exclamation-circle',
			'title'		=> 'ms_custom_items',
			'link'		=> \IPS\Http\Url::internal( 'app=membersshop&module=purchases&controller=customitems' ),
			'data'		=> array( )
		);
		
		/* Display */
		\IPS\Output::i()->title = \IPS\Member::loggedIn()->language()->addToStack( 'ms_awaiting_redemption' );
		\IPS\Output::i()->output	= (string) $table;
	}
}