<?php


namespace IPS\membersshop\modules\admin\purchases;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * membershipupgrades
 */
class _membershipupgrades extends \IPS\Dispatcher\Controller
{
	/**
	 * Execute
	 *
	 * @return	void
	 */
	public function execute()
	{
		parent::execute();
	}

	/**
	 * ...
	 *
	 * @return	void
	 */
	protected function manage()
	{
		$table = new \IPS\Helpers\Table\Db( 'membersshop_upgrades', \IPS\Http\Url::internal( 'app=membersshop&module=purchases&controller=membershipupgrades' ) );
		$table->langPrefix = 'ms__';
		$table->include = array( 'id', 'member_id', 'original_group', 'reward_group', 'time_date', 'expire_time_date', 'expired', );
		$table->exclude = array( 'id' );
		$table->mainColumn = 'member_id';
		$table->sortBy = $table->sortBy ?: 'time_date';
		$table->sortDirection = $table->sortDirection ?: 'asc';
		$table->filters = array( 'ms_filter_mu_active' => array( 'expired=0' ), 'ms_filter_mu_expired' => array( 'expired=1' ) );
        
		$table->parsers = array(
			'time_date' => function( $val, $row )
			{
				$date	= date('F j, Y, G:i a', $val);
				return $date;
			},
			'expire_time_date' => function( $val, $row )
			{
				if ( $val == -1 )
				{
					$date = \IPS\Member::loggedIn()->language()->addToStack('ms_this_is_perm_upgrade');
				}
				elseif ( $val < time() )
				{
					$date = \IPS\Member::loggedIn()->language()->addToStack('ms_expired');
				}
				else
				{
					$date	= date('F j, Y, G:i a', $val);
				}
				return $date;
			},
			'member_id' => function( $val, $row )
			{
				$member_id = \IPS\Member::load( $val );
				return "<a href='" . \IPS\Http\Url::internal( 'app=core&module=members&controller=members&do=edit&id=' ) . $row['member_id'] . "'>$member_id->name</a>";
			},
			'original_group' => function( $val, $row )
			{
				try
				{
					$group = \IPS\Member\Group::load( $val );
					return $group->formattedName;
				}
				catch ( \Exception $e )
				{
					return "Deleted Member Group";
				}
			},
			'reward_group' => function( $val, $row )
			{
				try
				{
					$group = \IPS\Member\Group::load( $val );
					return $group->formattedName;
				}
				catch ( \Exception $e )
				{
					return "Deleted Member Group";
				}
			},
			'expired' => function( $val, $row )
			{
				if ( $row['expire_time_date'] == -1 )
				{
					$expired = \IPS\Member::loggedIn()->language()->addToStack('ms_not_needed');
				}
				elseif ( $val == 0 )
				{
					if ( $row['expire_time_date'] < time() )
					{
						$expired = \IPS\Member::loggedIn()->language()->addToStack('ms_pending_expiration');
					}
					else
					{
						$expired = '<i class="fa fa-remove fa-lg" aria-hidden="true" style="color:red"></i>';
					}
				}
				else if ( $val == 1 )
				{
				    $expired = '<i class="fa fa-check fa-lg" aria-hidden="true" style="color:green"></i>';
				}
				return $expired;
			},
			
		);
		
		\IPS\Output::i()->title = \IPS\Member::loggedIn()->language()->addToStack( 'ms_membership_upgrades' );
		\IPS\Output::i()->output = (string) $table;
	}
}