<?php


namespace IPS\membersshop\modules\admin\purchases;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * normalitems
 */
class _normalitems extends \IPS\Dispatcher\Controller
{
	/**
	 * Execute
	 *
	 * @return	void
	 */
	public function execute()
	{
		parent::execute();
	}

	/**
	 * ...
	 *
	 * @return	void
	 */
	protected function manage()
	{
		/* Create the table */
		$table = new \IPS\Helpers\Table\Db( 'membersshop_purchases', \IPS\Http\Url::internal( 'app=membersshop&module=purchases&controller=normalitems' ) );
		$table->langPrefix = 'ms__';

		/* Column stuff */
		$table->include = array( 'icon', 'member_id', 'name', 'description', 'price', 'redeemed', 'time_date' );
		$table->mainColumn = 'description';
		$table->widths = array( 'icon' => '5', 'price' => '5', 'time_date' => '5', 'open' => '5', 'redeemed' => '5' );
		
		/* Sort stuff */
		$table->sortBy = $table->sortBy ?: 'time_date';
		$table->sortDirection = $table->sortDirection ?: 'desc';
	    $table->noSort	= array( 'icon', 'description' );
		$table->where[] = array( 'item_id!=26 AND item_id!=100', );
		$table->filters = array( 'ms_filter_not_redeemed' => array( 'redeemed=0' ), 'ms_filter_redeemed' => array( 'redeemed=1' ) );
        $table->quickSearch = 'name';
		$table->advancedSearch = array(
			'member_id'	=> \IPS\Helpers\Table\SEARCH_MEMBER,
			'time_date'	=> \IPS\Helpers\Table\SEARCH_DATE_RANGE
		);
		
		/* Formatters */
		$table->parsers = array(
			'redeemed' => function( $val, $row )
			{
				if ( $val == 0)
				{
					$awarded = '<center><i class="fa fa-times fa-lg" aria-hidden="true" style="color:red"></i></center>';
				}
				else if ( $val == 1)
				{
				    $awarded = '<center><i class="fa fa-check fa-lg" aria-hidden="true" style="color:green"></i></center>';
				}
				return $awarded;
			},
			'icon' => function( $val, $row )
			{
				$icon = '<i class="fa '.$val.' fa-lg" aria-hidden="true"></i>';
				return $icon;
			},
			'price' => function( $val, $row )
			{
				$price = number_format($val);
				return $price;
			},
			'description' => function( $val, $row )
			{
				return '<span data-ipsTruncate data-ipsTruncate-size="2 lines" data-ipsTruncate-type="remove">'.$val.'</span>';
			},
			'time_date' => function( $val, $row )
			{
				$date	= \IPS\DateTime::ts( $val );
				return $date->localeDate() . ' ' . $date->localeTime( FALSE ) ;
			},
			'member_id' => function( $val, $row )
			{
				$member_id = \IPS\Member::load( $val );
				return "<a href='" . \IPS\Http\Url::internal( 'app=core&module=members&controller=members&do=edit&id=' ) . $row['member_id'] . "'>$member_id->name</a>";
				
			}
		);
		
		\IPS\Output::i()->sidebar['actions']['customcodes'] = array(
		    'icon'		=> 'exclamation-circle',
			'title'		=> 'ms_custom_codes',
			'link'		=> \IPS\Http\Url::internal( 'app=membersshop&module=purchases&controller=customcodes' ),
			'data'		=> array( )
		);
		
		\IPS\Output::i()->sidebar['actions']['customitems'] = array(
		    'icon'		=> 'exclamation-circle',
			'title'		=> 'ms_custom_items',
			'link'		=> \IPS\Http\Url::internal( 'app=membersshop&module=purchases&controller=customitems' ),
			'data'		=> array( )
		);
		
		/* Display */
		\IPS\Output::i()->title = \IPS\Member::loggedIn()->language()->addToStack( 'ms_normal_items' );
		\IPS\Output::i()->output	= (string) $table;
	}
}