<?php


namespace IPS\membersshop\modules\front\shop;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * main
 */
class _main extends \IPS\Dispatcher\Controller
{
	/**
	 * Execute
	 *
	 * @return	void
	 */
	public function execute()
	{
		\IPS\Output::i()->jsVars['ms_flash_msg_pos'] = \IPS\Settings::i()->ms_flash_msg_pos;
		\IPS\Output::i()->globalControllers[] = 'membersshop.shop';
		\IPS\Output::i()->jsFiles = array_merge( \IPS\Output::i()->jsFiles, \IPS\Output::i()->js( 'global_shop.js', 'membersshop', 'global' ) );
		
		parent::execute();
	}

	/**
	 * ...
	 *
	 * @return	void
	 */
	protected function manage()
	{
		$featuredWhere = array( 'featured=1' );
		$featured = iterator_to_array( \IPS\membersshop\Shopitems::featured( \IPS\Settings::i()->ms_featured_block_amount, '_rand' ) );
		
		$whatsNewWhere = array( );
		$whatsNew = \IPS\membersshop\Shopitems::getItemsWithPermission( $whatsNewWhere, 'id DESC', \IPS\Settings::i()->ms_whats_new_block_amount, 'view', \IPS\Content\Hideable::FILTER_AUTOMATIC, 0, NULL, TRUE );
		
		$mostPopularWhere = array( );
		$mostPopular = \IPS\membersshop\Shopitems::getItemsWithPermission( $mostPopularWhere, 'purchases DESC', \IPS\Settings::i()->ms_popular_block_amount, 'view', \IPS\Content\Hideable::FILTER_AUTOMATIC, 0, NULL, TRUE );
		
		$runningOutWhere = array( 'instock!=0 AND instock!=-1 AND instock<=?', \IPS\Settings::i()->ms_low_stock_block_when );
		$runningOut = \IPS\membersshop\Shopitems::getItemsWithPermission( $runningOutWhere, 'instock ASC', \IPS\Settings::i()->ms_low_stock_block_amount, 'view', \IPS\Content\Hideable::FILTER_AUTOMATIC, 0, NULL, TRUE );
		
		$mostExpensiveWhere = array( );
		$mostExpensive = \IPS\membersshop\Shopitems::getItemsWithPermission( $mostExpensiveWhere, 'price DESC', \IPS\Settings::i()->ms_most_expensive_block_amount, 'view', \IPS\Content\Hideable::FILTER_AUTOMATIC, 0, NULL, TRUE );
		
		$randomWhere = array( );
		$random = \IPS\membersshop\Shopitems::getItemsWithPermission( $randomWhere, 'RAND()', \IPS\Settings::i()->ms_random_block_amount, 'view', \IPS\Content\Hideable::FILTER_AUTOMATIC, 0, NULL, TRUE );
		
		$text = \IPS\Settings::i()->ms_header_text;
		
		$categories = \IPS\membersshop\Categories::roots();
		$url = \IPS\Http\Url::internal( 'app=membersshop&module=shop&controller=main&do=shopItems', 'front', 'membersshop_shopItems' );
		
		\IPS\Output::i()->breadcrumb['module'] = array( \IPS\Http\Url::internal( 'app=membersshop&module=shop&controller=main', 'front', 'membersshop' ), \IPS\Member::loggedIn()->language()->addToStack('__app_membersshop') );
		
		if ( \IPS\Request::i()->isAjax() )
		{
			\IPS\Output::i()->json( array(
				'contents' 	=> \IPS\Theme::i()->getTemplate( 'shop' )->index( $featured, $whatsNew, $mostPopular, $runningOut, $mostExpensive, $random, $text ),
				'sidebar'	=> \IPS\Theme::i()->getTemplate( 'shop' )->indexSidebar( $categories, $url )
			)) ;
			return;
		}
		else
		{
			\IPS\Session::i()->setLocation( \IPS\Http\Url::internal( 'app=membersshop&module=shop&controller=main' ), array(), 'loc_ms_shop' );
			\IPS\Output::i()->sidebar['contextual'] = \IPS\Theme::i()->getTemplate( 'shop' )->indexSidebar( $categories, $url );
			\IPS\Output::i()->title = \IPS\Member::loggedIn()->language()->addToStack('ms_index_page');
			\IPS\Output::i()->output = \IPS\Theme::i()->getTemplate( 'shop' )->index( $featured, $whatsNew, $mostPopular, $runningOut, $mostExpensive, $random, $text );
		}
	}
	
	protected function shopItems()
	{
		$member = \IPS\Member::loggedIn();
		$pointsName = \IPS\Settings::i()->ms_points_name;
		$url = \IPS\Http\Url::internal( 'app=membersshop&module=shop&controller=main&do=shopItems', 'front', 'membersshop_shopItems' );
		
		$categoryIDs = array();
		$where = array();
		
		if ( isset( \IPS\Request::i()->catID ) )
		{
			$url = $url->setQueryString( 'catID', \IPS\Request::i()->catID );
			$categoryIDs = explode( ',', \IPS\Request::i()->catID );
			$where[] = array( \IPS\Db::i()->in( 'membersshop_items.cat_id', array_filter( $categoryIDs ) ) );
		}
		
		if( isset( \IPS\Request::i()->sortby ) )
		{
			$url = $url->setQueryString( 'sortby', \IPS\Request::i()->sortby );
		}
		
		if( \IPS\Request::i()->stock == 'in_stock' )
		{
			$url = $url->setQueryString( 'stock', \IPS\Request::i()->stock );
			$where[] = array( 'membersshop_items.instock!=-0'  );
		}
		
		if( \IPS\Request::i()->stock == 'out_stock' )
		{
			$url = $url->setQueryString( 'stock', \IPS\Request::i()->stock );
			$where[] = array( 'membersshop_items.instock=-0'  );
		}
		
		if( \IPS\Request::i()->stock == 'unlimited_stock' )
		{
			$url = $url->setQueryString( 'stock', \IPS\Request::i()->stock );
			$where[] = array( 'membersshop_items.instock=-1'  );
		}
		
		$table = new \IPS\Helpers\Table\Content( 'IPS\membersshop\Shopitems', $url, $where, NULL, NULL, 'view' );
		if ( \IPS\Settings::i()->ms_shop_view_set == 0 )
		{
			$table->rowsTemplate = array( \IPS\Theme::i()->getTemplate('shop'), 'rows' );
		}
		else
		{
			if ( \IPS\Settings::i()->ms_grid_style == 0 )
			{
				$table->rowsTemplate = array( \IPS\Theme::i()->getTemplate('shop'), 'rowsGridMini' );
			}
			else
			{
				$table->rowsTemplate = array( \IPS\Theme::i()->getTemplate('shop'), 'rowsGrid' );
			}
		}
		$table->limit = \IPS\Settings::i()->ms_shop_table_limit;
		$table->sortOptions = array( 'name' => 'name', 'price' => 'price', 'purchases' => 'purchases', 'position' => 'position' );
		$table->sortBy = $table->sortBy ?: 'position';
		
		switch ( \IPS\Request::i()->sortby )
		{
			case 'name':
				$table->sortBy = 'name';
				$table->sortDirection = 'asc';
				break;
			case 'price_low':
				$table->sortBy = 'price';
				$table->sortDirection = 'asc';
				break;
			case 'price_high':
				$table->sortBy = 'price';
				$table->sortDirection = 'desc';
				break;
			case 'purchases':
				$table->sortBy = 'purchases';
				$table->sortDirection = 'desc';
				break;
			default:
				$table->sortBy = 'position';
				$table->sortDirection = 'asc';
				$url = $url->setQueryString( 'sortby', NULL );
				break;
		}
		
		$categories = \IPS\membersshop\Categories::roots();
		
		\IPS\Output::i()->sidebar['contextual'] = \IPS\Theme::i()->getTemplate( 'shop' )->indexSidebar( $categories, $url );
		\IPS\Session::i()->setLocation( $url, array(), 'loc_ms_shop' );
		\IPS\Output::i()->title = \IPS\Member::loggedIn()->language()->addToStack('ms_items');
		
		\IPS\Output::i()->breadcrumb['module'] = array( \IPS\Http\Url::internal( 'app=membersshop&module=shop&controller=main', 'front', 'membersshop' ), \IPS\Member::loggedIn()->language()->addToStack('__app_membersshop') );
		\IPS\Output::i()->breadcrumb[] = array( NULL, \IPS\Member::loggedIn()->language()->addToStack('ms_items') );
		
		$title = NULL;
		if( $member->member_id )
		{
			$title = \IPS\Member::loggedIn()->language()->addToStack('ms_table_header_points', FALSE, array( 'sprintf' => array( number_format( $member->shopPoints() ), $pointsName ) ) );
		}
		
		if ( \IPS\Request::i()->isAjax() )
		{
			\IPS\Output::i()->json( array(
				'contents' 	=> \IPS\Theme::i()->getTemplate('shop')->template( (string) $table, $member, $title ),
				'sidebar'	=> \IPS\Theme::i()->getTemplate( 'shop' )->indexSidebar( $categories, $url )
			)) ;
			return;
		}
		else
		{
			\IPS\Output::i()->output = \IPS\Theme::i()->getTemplate('shop')->template( (string) $table, $member, $title );
		}
	}
	
	protected function buyItem()
	{
		$member = \IPS\Member::loggedIn();
		$id  = \intval( \IPS\Request::i()->id );
		$pointsName = \IPS\Settings::i()->ms_points_name;
		
		if ( !$member->member_id )
		{
			if ( \IPS\Request::i()->isAjax() )
			{
				\IPS\Output::i()->json( array( 'error' => \IPS\Member::loggedIn()->language()->addToStack('ms_no_module_permission_guest') ) );
				return;
			}
			\IPS\Output::i()->error( 'ms_no_module_permission_guest', 'MEMBERSSHOPGUEST/1', 403, '' );
		}
		
		try
		{
			$item = \IPS\membersshop\Items::load( $id );
		}
		catch ( \OutOfRangeException $e )
		{
			if ( \IPS\Request::i()->isAjax() )
			{
				\IPS\Output::i()->json( array( 'error' => \IPS\Member::loggedIn()->language()->addToStack('node_error') ) );
				return;
			}
			\IPS\Output::i()->error( 'node_error', 'MEMBERSSHOPNODE/1', 404, '' );
		}
		
		if ( $item->groups_buy != '*' AND !\IPS\Member::loggedIn()->inGroup( explode(',', $item->groups_buy ) ) )
		{
			if ( \IPS\Request::i()->isAjax() )
			{
				\IPS\Output::i()->json( array( 'error' => \IPS\Member::loggedIn()->language()->addToStack('ms_no_permission_buying') ) );
				return;
			}
			\IPS\Output::i()->error( 'ms_no_permission_buying', 'MEMBERSSHOPGROUPBUYING/1', 404, '' );
		}
		
		if ( $item->instock < 1 AND $item->instock != -1 )
		{
			if ( \IPS\Request::i()->isAjax() )
			{
				\IPS\Output::i()->json( array( 'error' => \IPS\Member::loggedIn()->language()->addToStack('ms_out_of_stock') ) );
				return;
			}
			\IPS\Output::i()->error( 'ms_out_of_stock', 'MEMBERSSHOPINSTOCK/1', 403, '' );
		}
		
		if( $item->purchase_delay )
		{
			try
			{
				$purchase = \IPS\Db::i()->select( '*', 'membersshop_purchases', array( 'member_id=? AND item_id=?', $member->member_id, $item->item_id ), 'id DESC' )->first();
				$purchaseTime = $purchase['time_date'];
				$delayedTime = $purchaseTime + $item->purchase_delay;
				if( time() <= $delayedTime )
				{
					$timeToWait = time() - $delayedTime;
					if ( \IPS\Request::i()->isAjax() )
					{
						\IPS\Output::i()->json( array( 'error' => \IPS\Member::loggedIn()->language()->addToStack('ms_you_need_to_wait_', FALSE, array( 'sprintf' => array( number_format( abs( $timeToWait ) ) ) ) ) ) );
						return;
					}
					\IPS\Output::i()->error( \IPS\Member::loggedIn()->language()->addToStack('ms_you_need_to_wait_', FALSE, array( 'sprintf' => array( number_format( abs( $timeToWait ) ) ) ) ), 'MEMBERSSHOP/1', 403, '' );
				}
			}
			catch( \Exception $ex ){}
		}
		
		$cost = $item->price;
		
		if ( !$member->modPermission( 'membersshop_can_buy_free' ) )
		{
			if ( $member->shopPoints() < $cost AND $member->shopPoints() !== $cost )
			{
				if ( \IPS\Request::i()->isAjax() )
				{
					\IPS\Output::i()->json( array( 'error' => \IPS\Member::loggedIn()->language()->addToStack('ms_you_dont_have_enough_points', FALSE, array( 'sprintf' => array( $pointsName ) ) ) ) );
					return;
				}
			    \IPS\Output::i()->error( \IPS\Member::loggedIn()->language()->addToStack('ms_you_dont_have_enough_points', FALSE, array( 'sprintf' => array( $pointsName ) ) ), 'MEMBERSSHOP/1', 403, '' );
			}
			
			$logsACP = sprintf( \IPS\Member::loggedIn()->language()->get('ms_acp_log_item_purchase'), $item->name );
			$member->removePoints( $cost, $logsACP );
		}
		
		if ( $member->modPermission( 'membersshop_can_buy_free' ) )
		{
			\IPS\Session::i()->modLog( 'modlog__action_ms_buy_free', array(
				$item->name				=> FALSE
			) );
		}
		
		$purchase = new \IPS\membersshop\Purchases;
		$purchase->item_id = $item->item_id;
		$purchase->member_id = $member->member_id;
		$purchase->name = $item->name;
		$purchase->description = $item->desc;
		$purchase->icon = $item->icon;
		$purchase->time_date = time();
		$purchase->price = $cost;
		$purchase->perm_id = $item->id;
		if ( $item->minRand )
		{
			$purchase->minRand = $item->minRand;
		}
		if ( $item->maxRand == 0 or $item->maxRand )
		{
			$purchase->maxRand = $item->maxRand;
		}
		if ( $item->extra_field == 0 or $item->extra_field )
		{
			$purchase->extra_field = $item->extra_field;
		}
		if ( $item->extra_field_text )
		{
			$purchase->extra_field_text = $item->extra_field_text;
		}
		$purchase->save();
		
		$item->purchases = $item->purchases + 1;
		
		if ( $item->instock != -1 )
		{
			$item->instock = $item->instock - 1;
		}
		$item->save();
		
		$extensions = \IPS\Application::allExtensions( 'membersshop', 'Purchase', FALSE );
		foreach ( $extensions as $k => $class )
		{
			if ( method_exists( $class, 'afterPurchase' ) )
			{	
				$class->afterPurchase( $item );
			}
		}
		
		if ( \IPS\Request::i()->isAjax() )
		{
			\IPS\Output::i()->json( array( 'message' => \IPS\Member::loggedIn()->language()->addToStack( 'ms_purchased' ) ));
			return;
		}
		else
		{
			$category = \IPS\membersshop\Categories::load( $item->cat_id );
			\IPS\Output::i()->redirect( \IPS\Request::i()->referrer() ?: $category->url(), \IPS\Member::loggedIn()->language()->addToStack( 'ms_purchased' ) );
		}
	}
	
	protected function donatepoints()
	{
		$msmid  = \intval( \IPS\Request::i()->msmid );
		$member = \IPS\Member::loggedIn();
		$pointsName = \IPS\Settings::i()->ms_points_name;
		if ( $member->member_id == $msmid )
		{
			\IPS\Output::i()->error( 'ms_donate_self', 'MEMBERSHOP/1', 403, '' );
		}
		
		if ( \IPS\Settings::i()->ms_donate_groups != 'all' AND !\IPS\Member::loggedIn()->inGroup( explode(',', \IPS\Settings::i()->ms_donate_groups ) ) )
		{
			\IPS\Output::i()->error( \IPS\Member::loggedIn()->language()->addToStack('ms_no_permission_donate', FALSE, array( 'sprintf' => array( $pointsName ) ) ), 'MEMBERSHOP/1', 403, '' );
		}
		
		$points = $member->shopPoints();
		
		$form = new \IPS\Helpers\Form( 'membersshop', 'Send' );
		$form->class = 'ipsForm_vertical ipsForm_fullWidth';
		
        $form->addTab('Donate');
		if ( \IPS\Settings::i()->ms_donate_max == -1 )
		{
			$max = NULL;
		}
		else
		{
			$max = \IPS\Settings::i()->ms_donate_max;
		}
		
		if ( !\IPS\Request::i()->msmid )
		{
			$form->add(new \IPS\Helpers\Form\Member('ms_donate_member', NULL, TRUE, array( ), function( $val )
    	    {
				if ( $val == \IPS\Member::loggedIn() )
		        {
			        throw new \DomainException('ms_donate_self');
		        }			
			}
			));
		}
		
		$form->add( new \IPS\Helpers\Form\Number('ms_donate_amount', \IPS\Settings::i()->ms_donate_min, FALSE, array( 'min' => \IPS\Settings::i()->ms_donate_min, 'max' => $max ), NULL, NULL, \IPS\Member::loggedIn()->language()->addToStack('ms_points', FALSE, array( 'sprintf' => array( $pointsName ) ) ), 'ms_donate_amount' , function( $val )
   		{
			if ( $points < $val )
			{
		        throw new \DomainException('ms_not_enough_points');
		    }		
   	    }
		));
		
		if ( $values = $form->values() )
		{
			if ( $points < $values['ms_donate_amount'] AND $points != $values['ms_donate_amount']  )
			{
				\IPS\Output::i()->error( \IPS\Member::loggedIn()->language()->addToStack('ms_you_dont_have_enough_points_donate', FALSE, array( 'sprintf' => array( $pointsName ) ) ), 'MEMBERSSHOP/1', 403, '' );
		    }		
			
			if ( \IPS\Request::i()->msmid == TRUE )
			{
				$receiver = \IPS\Member::load( $msmid );
			}
			else
			{
				$receiver = \IPS\Member::load( $values['ms_donate_member']->member_id );
			}
			
			$memberMessage = ''.\IPS\Member::loggedIn()->language()->get("ms_donated_log").'';
			$member->removePoints( $values['ms_donate_amount'], $memberMessage );
			
			$receiverMessage = ''.\IPS\Member::loggedIn()->language()->get("ms_for_a_donation").'';
			$receiver->addPoints( $values['ms_donate_amount'], $receiverMessage );
			
			$donation = new \IPS\membersshop\Donations;
			$donation->donator_id = $member->member_id;
			$donation->receiver_id = $receiver->member_id;
			$donation->amount = $values['ms_donate_amount'];
			$donation->time_date = time();
			$donation->save();
			
			$items = \IPS\membersshop\Donations::load( $donation->id );
			$notification = new \IPS\Notification( \IPS\Application::load( 'membersshop' ), 'donate_donations', $items, array ( $items ) );
			$notification->recipients->attach( $receiver );
			$notification->send();
			
			\IPS\Output::i()->redirect( \IPS\Request::i()->referrer() ?: \IPS\Http\Url::internal( '' ) );
		}
		else
		{
			\IPS\Output::i()->output = \IPS\Theme::i()->getTemplate( 'shop', 'membersshop' )->form( $form );
		}
	}
	
	protected function getDesc()
	{
		$id  = \intval( \IPS\Request::i()->id );
		$getItem = \IPS\membersshop\Items::load( $id )->desc;
		\IPS\Output::i()->output = \IPS\Theme::i()->getTemplate( 'shop', 'membersshop' )->desc( $getItem );
	}
	
	protected function myitemDesc()
	{
		$id  = \intval( \IPS\Request::i()->id );
		$getItem = \IPS\membersshop\Purchases::load( $id )->description;
		\IPS\Output::i()->output = \IPS\Theme::i()->getTemplate( 'shop', 'membersshop' )->desc( $getItem );
	}
	
	protected function purchasePoints()
	{
		if ( !\IPS\Member::loggedIn()->member_id )
		{
			\IPS\Output::i()->error( 'no_module_permission_guest', 'MEMBERSHOP/1', 403, '' );
		}
		
		if( !\IPS\Settings::i()->ms_points_sale_amount )
		{
			\IPS\Output::i()->error( 'ms_no_prices_set_points_purchase', 'MEMBERSHOP/1', 403, '' );
		}
		
		$pointsName = \IPS\Settings::i()->ms_points_name;
		$url = \IPS\Http\Url::internal( 'app=membersshop&module=shop&controller=main&do=sendAmount' );
		$currency = \IPS\nexus\Customer::loggedIn()->defaultCurrency();
		$settings = json_decode( \IPS\Settings::i()->ms_points_sale_amount, true );
		$getPoints = $settings[ $currency ];
		
		\IPS\Member::loggedIn()->language()->words[ 'ms_purchase_points_description' ] =  sprintf( \IPS\Member::loggedIn()->language()->get('ms_purchase_points_description'), number_format( $getPoints ), $pointsName, $currency );
		\IPS\Member::loggedIn()->language()->words[ 'ms_purchase_points_desc' ] =  sprintf( \IPS\Member::loggedIn()->language()->get('ms_purchase_points_desc_settings'), number_format( \IPS\Settings::i()->ms_minimum_amount_to_buy ), $pointsName, number_format( \IPS\Settings::i()->ms_maximum_amount_to_buy ), $pointsName );
		\IPS\Member::loggedIn()->language()->words[ 'ms_enter_amount_of_points' ] =  sprintf( \IPS\Member::loggedIn()->language()->get('ms_enter_amount_of_points'), $pointsName );
		
		$form = new \IPS\Helpers\Form( 'membersshop', 'checkout' );
		$form->class = 'ipsForm_vertical ipsForm_noLabels ipsForm_fullWidth';
		$form->add( new \IPS\Helpers\Form\Number('ms_purchase_points', \IPS\Settings::i()->ms_minimum_amount_to_buy, TRUE, array( 'min' => \IPS\Settings::i()->ms_minimum_amount_to_buy, 'max' => \IPS\Settings::i()->ms_maximum_amount_to_buy ), NULL, NULL, $pointsName, 'ms_purchase_points' ) );
		
		if( $values = $form->values() )
		{
			$cost = $values['ms_purchase_points'] / $getPoints;
			$price = new \IPS\nexus\Money( $cost, $currency );
			
			$item = new \IPS\membersshop\extensions\nexus\Item\Points( 'Points Purchase', $price );
			
			$invoice = new \IPS\nexus\Invoice;
			$invoice->member = \IPS\nexus\Customer::loggedIn();
			$invoice->currency = $currency;
			$invoice->return_uri = 'app=membersshop&module=shop&controller=main';
			$invoice->addItem( $item );
			$invoice->save();
			
			$extra = $invoice->status_extra;
			$extra['points'] = $values['ms_purchase_points'];
			$invoice->status_extra = $extra;
			$invoice->save();
			
			\IPS\Output::i()->redirect( $invoice->checkoutUrl() );
		}
		\IPS\Output::i()->jsFiles = array_merge( \IPS\Output::i()->jsFiles, \IPS\Output::i()->js( 'global_shop.js', 'membersshop', 'global' ) );
		
		\IPS\Output::i()->output = \IPS\Theme::i()->getTemplate( 'shop', 'membersshop' )->buyPoints( $form, $pointsName, $getPoints, $currency );
	}
	
	protected function sendAmount()
	{
		$currency = \IPS\nexus\Customer::loggedIn()->defaultCurrency();
		$settings = json_decode( \IPS\Settings::i()->ms_points_sale_amount, true );
		$getPoints = $settings[ $currency ];
		$cost = \IPS\Request::i()->amount / $getPoints;
		
		\IPS\Output::i()->json( (string) new \IPS\nexus\Money( $cost, \IPS\nexus\Customer::loggedIn()->defaultCurrency() ) );
	}
}