<?php


namespace IPS\membersshop\modules\front\shop;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * myitems
 */
class _myitems extends \IPS\Dispatcher\Controller
{
	/**
	 * ...
	 *
	 * @return	void
	 */
	public function execute()
	{
		\IPS\Output::i()->jsVars['ms_flash_msg_pos'] = \IPS\Settings::i()->ms_flash_msg_pos;
		\IPS\Output::i()->globalControllers[] = 'membersshop.shop';
		\IPS\Output::i()->jsFiles = array_merge( \IPS\Output::i()->jsFiles, \IPS\Output::i()->js( 'global_shop.js', 'membersshop', 'global' ) );
		
		parent::execute();
	}

	/**
	 * ...
	 *
	 * @return	void
	 */
	protected function manage()
	{
		$member = \IPS\Member::loggedIn();

		if ( !$member->member_id )
		{
			\IPS\Output::i()->error( 'no_module_permission_guest', 'MEMBERSSHOP/1', 403, '' );
		}
		
		$table = new \IPS\Helpers\Table\Db( 'membersshop_purchases', \IPS\Http\Url::internal( 'app=membersshop&module=shop&controller=myitems', 'front', 'myitems' ) );
		$table->include = array( 'id', 'time_date', 'redeemed', 'price', 'item_id', 'member_id' );
		if ( \IPS\Settings::i()->ms_myitems_view_set == 0 )
		{
			$table->rowsTemplate = array( \IPS\Theme::i()->getTemplate('shop'), 'myitemsrows' );
		}
		else
		{
			if ( \IPS\Settings::i()->ms_grid_style == 0 )
			{
				$table->rowsTemplate = array( \IPS\Theme::i()->getTemplate('shop'), 'myitemsrowsGridMini' );
			}
			else
			{
				$table->rowsTemplate = array( \IPS\Theme::i()->getTemplate('shop'), 'myitemsrowsGrid' );
			}	
		}
		$table->joins = array();
		$table->where[] = array( 'redeemed=? AND member_id=?', 0, $member->member_id );
		$table->langPrefix = 'ms__';
		$table->limit = \IPS\Settings::i()->ms_myitems_table_limit;
		$table->sortOptions = array( 'name' => 'name', 'item_id' => 'item_id', 'price' => 'price', 'time_date' => 'time_date' );
		$table->sortBy = $table->sortBy ?: 'time_date';
		if ( $table->sortBy === 'time_date' )
		{
			$table->sortDirection = 'desc';
		}
		if ( $table->sortBy === 'name' )
		{
			$table->sortDirection = 'asc';
		}
		
		$title = \IPS\Member::loggedIn()->language()->addToStack('ms_myitems');
		\IPS\Output::i()->title = \IPS\Member::loggedIn()->language()->addToStack('ms_myitems');
		\IPS\Session::i()->setLocation( \IPS\Http\Url::internal( 'app=membersshop&module=shop&controller=main', 'front', 'membersshop' ), array(), 'loc_ms_shop' );
		
		\IPS\Output::i()->breadcrumb['module'] = array( \IPS\Http\Url::internal( 'app=membersshop&module=shop&controller=main', 'front', 'membersshop' ), \IPS\Member::loggedIn()->language()->addToStack('__app_membersshop') );
		\IPS\Output::i()->breadcrumb[] = array( \IPS\Http\Url::internal( 'app=membersshop&module=shop&controller=myitems', 'front', 'myitems' ), \IPS\Member::loggedIn()->language()->addToStack('ms_myitems') );
		
		if( $member->modPermission( 'membersshop_can_buy_free' ) )
		{
			\IPS\Output::i()->jsVars['sellBackMessage'] = \IPS\Member::loggedIn()->language()->addToStack('ms_sellback_message_mod');
		}
		else
		{
			\IPS\Output::i()->jsVars['sellBackMessage'] = \IPS\Member::loggedIn()->language()->addToStack('ms_sellback_message', FALSE, array( 'sprintf' => array( 100 - \IPS\Settings::i()->ms_charge_sell_back ) ) );
		}
		
		if ( \IPS\Request::i()->isAjax() )
		{
			\IPS\Output::i()->json( array(
				'contents' 	=> \IPS\Theme::i()->getTemplate('shop')->template( (string) $table, $member, $title )
			)) ;
			return;
		}
		else
		{
			\IPS\Output::i()->output = \IPS\Theme::i()->getTemplate('shop')->template( (string) $table, $member, $title );
		}
	}
	
	protected function buyandUse()
	{
		$member = \IPS\Member::loggedIn();
		$id  = \intval( \IPS\Request::i()->id );
		$pointsName = \IPS\Settings::i()->ms_points_name;
		
		if ( !$member->member_id )
		{
			if ( \IPS\Request::i()->isAjax() )
			{
				\IPS\Output::i()->json( array( 'error' => \IPS\Member::loggedIn()->language()->addToStack('ms_no_module_permission_guest') ) );
				return;
			}
			
			\IPS\Output::i()->error( 'ms_no_module_permission_guest', 'MEMBERSSHOPGUEST/1', 403, '' );
		}
		
		try
		{
			$item = \IPS\membersshop\Items::load( $id );
		}
		catch ( \OutOfRangeException $e )
		{
			if ( \IPS\Request::i()->isAjax() )
			{
				\IPS\Output::i()->json( array( 'error' => \IPS\Member::loggedIn()->language()->addToStack('node_error') ) );
				return;
			}
			\IPS\Output::i()->error( 'node_error', 'MEMBERSSHOPNODE/1', 404, '' );
		}
		
		if ( $item->groups_buy != '*' AND !\IPS\Member::loggedIn()->inGroup( explode(',', $item->groups_buy ) ) )
		{
			if ( \IPS\Request::i()->isAjax() )
			{
				\IPS\Output::i()->json( array( 'error' => \IPS\Member::loggedIn()->language()->addToStack('ms_no_permission_buying') ) );
				return;
			}
			\IPS\Output::i()->error( 'ms_no_permission_buying', 'MEMBERSSHOPGROUPBUYING/1', 404, '' );
		}
		
		if ( $item->instock < 1 AND $item->instock != -1 )
		{
			if ( \IPS\Request::i()->isAjax() )
			{
				\IPS\Output::i()->json( array( 'error' => \IPS\Member::loggedIn()->language()->addToStack('ms_out_of_stock') ) );
				return;
			}
			\IPS\Output::i()->error( 'ms_out_of_stock', 'MEMBERSSHOPINSTOCK/1', 403, '' );
		}
		
		if( $item->purchase_delay )
		{
			try
			{
				$purchase = \IPS\Db::i()->select( '*', 'membersshop_purchases', array( 'member_id=? AND item_id=?', $member->member_id, $item->item_id ), 'id DESC' )->first();
				$purchaseTime = $purchase['time_date'];
				$delayedTime = $purchaseTime + $item->purchase_delay;
				if( time() <= $delayedTime )
				{
					$timeToWait = time() - $delayedTime;
					if ( \IPS\Request::i()->isAjax() )
					{
						\IPS\Output::i()->json( array( 'error' => \IPS\Member::loggedIn()->language()->addToStack('ms_you_need_to_wait_', FALSE, array( 'sprintf' => array( number_format( abs( $timeToWait ) ) ) ) ) ) );
						return;
					}
					\IPS\Output::i()->error( \IPS\Member::loggedIn()->language()->addToStack('ms_you_need_to_wait_', FALSE, array( 'sprintf' => array( number_format( abs( $timeToWait ) ) ) ) ), 'MEMBERSSHOP/1', 403, '' );
				}
			}
			catch( \Exception $ex ){}
		}
		
		$cost = $item->price;
		
		if ( !$member->modPermission( 'membersshop_can_buy_free' ) )
		{
    	    if ( $member->shopPoints() < $cost AND $member->shopPoints() !== $cost )
			{
				if ( \IPS\Request::i()->isAjax() )
				{
					\IPS\Output::i()->json( array( 'error' => \IPS\Member::loggedIn()->language()->addToStack('ms_you_dont_have_enough_points', FALSE, array( 'sprintf' => array( $pointsName ) ) ) ) );
					return;
				}
			    \IPS\Output::i()->error( \IPS\Member::loggedIn()->language()->addToStack('ms_you_dont_have_enough_points', FALSE, array( 'sprintf' => array( $pointsName ) ) ), 'MEMBERSSHOP/1', 403, '' );
			}
			
			$pointsName = \IPS\Settings::i()->ms_points_name;
			$logsACP = sprintf( \IPS\Member::loggedIn()->language()->get('ms_acp_log_item_purchase'), $item->name );
			$member->removePoints( $cost, $logsACP );
		}
		
		if ( $member->modPermission( 'membersshop_can_buy_free' ) )
		{
			\IPS\Session::i()->modLog( 'modlog__action_ms_buy_free', array(
				$item->name				=> FALSE
			) );
		}
		
		$purchase = new \IPS\membersshop\Purchases;
		$purchase->item_id = $item->item_id;
		$purchase->member_id = $member->member_id;
		$purchase->name = $item->name;
		$purchase->description = $item->desc;
		$purchase->icon = $item->icon;
		$purchase->time_date = time();
		$purchase->price = $cost;
		$purchase->perm_id = $item->id;
		
		if ( $item->minRand )
		{
			$purchase->minRand = $item->minRand;
		}
		
		if ( $item->maxRand == 0 or $item->maxRand )
		{
			$purchase->maxRand = $item->maxRand;
		}
		
		if ( $item->extra_field == 0 or $item->extra_field )
		{
			$purchase->extra_field = $item->extra_field;
		}
		
		if ( $item->extra_field_text )
		{
			$purchase->extra_field_text = $item->extra_field_text;
		}
		
		$purchase->save();
		
		$item->purchases = $item->purchases + 1;
		
		if ( $item->instock != -1 )
		{
			$item->instock = $item->instock - 1;
		}
		
		$item->save();
		
		$extensions = \IPS\Application::allExtensions( 'membersshop', 'Purchase', FALSE );
		foreach ( $extensions as $k => $class )
		{
			if ( method_exists( $class, 'afterPurchase' ) )
			{	
				$class->afterPurchase( $item );
			}
		}
		
		/*
		$base        = 'front';
		$seoTemplate = 'my_items_redeem';
		$seoTitles   = array( \IPS\Http\Url\Friendly::seoTitle( $purchase->name ) );
		
		$url = \IPS\Http\Url::internal( "app=membersshop&module=shop&controller=myitems&do=redeem&id={$purchase->id}&b=1&c={$item->cat_id}", $base, $seoTemplate, $seoTitles  );
		\IPS\Output::i()->redirect( $url );
		*/
		
		$categoryURL = \IPS\Http\Url::internal( 'app=membersshop&module=shop&controller=main&do=shopItems&catID='.$item->cat_id.''  );
		return $this->redeem( $purchase->id, TRUE, $categoryURL );
	}
	
	protected function redeem( $purchaseID=NULL, $boughtAndUsed=FALSE, $categoryURL=NULL )
	{
		$member = \IPS\Member::loggedIn();
		if( \IPS\Request::i()->b == 1 )
		{
			$boughtAndUsed = TRUE;
		}
		
		if( \IPS\Request::i()->c )
		{
			$categoryURL = \IPS\Http\Url::internal( 'app=membersshop&module=shop&controller=main&do=shopItems&catID='.\IPS\Request::i()->c.''  );
		}
		
		if ( $purchaseID )
		{
			$id = $purchaseID;
		}
		else
		{
			$id  = \IPS\Request::i()->id;
		}
		
		try
		{
			try
			{
			    $getItem = \IPS\membersshop\Purchases::load( $id );
			}
			catch ( \OutOfRangeException $e )
			{
				\IPS\Output::i()->error( 'node_error', 'MEMBERSSHOP/1', 404, '' );
			}
			
			$item_id = $getItem->item_id;
			
			if ( $getItem->redeemed == 1 OR $getItem->member_id !== $member->member_id )
			{
				\IPS\Output::i()->error( 'ms_node_error', 'MEMBERSHOP/1', 403, '' );
			}
			
			$extensions = \IPS\Application::allExtensions( 'membersshop', 'Items', FALSE );
			foreach ( $extensions as $k => $class )
			{
				if ( method_exists( $class, 'redeemItem' ) )
				{	
					$class->redeemItem( $item_id, $getItem, $member, $boughtAndUsed, $categoryURL );
				}
			}
			
			if ( $item_id == 1 ) { return $this->addPosts( $getItem, $member, $boughtAndUsed, $categoryURL ); }
			elseif ( $item_id == 2 ) { return $this->addRep( $getItem, $member, $boughtAndUsed, $categoryURL ); }
			elseif ( $item_id == 3 ) { return $this->stealRep( $getItem, $member, $boughtAndUsed, $categoryURL ); }
			elseif ( $item_id == 4 ) { return $this->changeUsername( $getItem, $member, $boughtAndUsed, $categoryURL ); }
			elseif ( $item_id == 5 ) { return $this->gamblePoints( $getItem, $member, $boughtAndUsed, $categoryURL ); }
			elseif ( $item_id == 6 ) { return $this->changeTitle( $getItem, $member, $boughtAndUsed, $categoryURL ); }
			elseif ( $item_id == 7 ) { return $this->resetWarnPoints( $getItem, $member, $boughtAndUsed, $categoryURL ); }
			elseif ( $item_id == 8 ) { return $this->forumPassword( $getItem, $member, $boughtAndUsed, $categoryURL ); }
			elseif ( $item_id == 9 ) { return $this->addSignature( $getItem, $member, $boughtAndUsed, $categoryURL ); }
			elseif ( $item_id == 10 ) { return $this->rockpaperscissors( $getItem, $member, $boughtAndUsed, $categoryURL ); }
			elseif ( $item_id == 11 ) { \IPS\Output::i()->error( 'ms_11_not_redeemable', 'MEMBERSHOP/1', 403, '' ); }
			elseif ( $item_id == 12 ) { \IPS\Output::i()->error( 'ms_12_not_redeemable', 'MEMBERSHOP/1', 403, '' ); }
			elseif ( $item_id == 13 ) { return $this->upgradeUsergroup( $getItem, $member, $boughtAndUsed, $categoryURL ); }
			elseif ( $item_id == 14 ) { return $this->upgradeUsergroup( $getItem, $member, $boughtAndUsed, $categoryURL ); }
			elseif ( $item_id == 15 ) { return $this->upgradeUsergroup( $getItem, $member, $boughtAndUsed, $categoryURL ); }
			elseif ( $item_id == 16 ) { return $this->gamblePoints( $getItem, $member, $boughtAndUsed, $categoryURL ); }
			elseif ( $item_id == 17 ) { return $this->gamblePoints( $getItem, $member, $boughtAndUsed, $categoryURL ); }
			elseif ( $item_id == 18 ) { return $this->randomTrophy( $getItem, $member, $boughtAndUsed, $categoryURL ); }
			elseif ( $item_id == 19 ) { return $this->uploadAvatar( $getItem, $member, $boughtAndUsed, $categoryURL ); }
			elseif ( $item_id == 20 ) { return $this->goAnonymous( $getItem, $member, $boughtAndUsed, $categoryURL ); }
			elseif ( $item_id == 21 ) { return $this->guesstheNumber( $getItem, $member, $boughtAndUsed, $categoryURL ); }
			elseif ( $item_id == 22 ) { return $this->profileVideo( $getItem, $member, $boughtAndUsed, $categoryURL ); }
			elseif ( $item_id == 23 ) { return $this->profileBackground( $getItem, $member, $boughtAndUsed, $categoryURL ); }
			elseif ( $item_id == 24 ) { return $this->stickyNote( $getItem, $member, $boughtAndUsed, $categoryURL ); }
			elseif ( $item_id == 25 ) { return $this->memberssocialInfo( $getItem, $member, $boughtAndUsed, $categoryURL ); }
			elseif ( $item_id == 26 ) { return $this->customcode( $getItem, $member, $boughtAndUsed, $categoryURL ); }
			elseif ( $item_id == 27 ) { return $this->buytrophy( $getItem, $member, $boughtAndUsed, $categoryURL ); }
			elseif ( $item_id == 28 ) { return $this->buymedal( $getItem, $member, $boughtAndUsed, $categoryURL ); }
			elseif ( $item_id == 29 ) { return $this->resetRep( $getItem, $member, $boughtAndUsed, $categoryURL ); }
			elseif ( $item_id == 30 ) { return $this->mysteryItem( $getItem, $member, $boughtAndUsed, $categoryURL ); }
			elseif ( $item_id == 100 ) { return $this->customPurchase( $getItem, $member, $boughtAndUsed, $categoryURL ); }
			/*else { \IPS\Output::i()->error( 'ms_no_item_id', 'MEMBERSHOP/1', 403, '' ); }*/
		}
		catch( \UnderflowException $ex )
		{
			\IPS\Output::i()->error( 'node_error', 'MEMBERSSHOP/1', 403, '' );
		}
	}
	
	protected function sendTo()
	{
		$member = \IPS\Member::loggedIn();
		$id  = \intval( \IPS\Request::i()->id );
		$permission = \intval( \IPS\Request::i()->perm );
		$pointsName = \IPS\Settings::i()->ms_points_name;

		try
		{
			$perm = \IPS\membersshop\Items::load( $permission );
			if ( $perm->groups_send != '*' AND !\IPS\Member::loggedIn()->inGroup( explode( ',', $perm->groups_send ) ) )
			{
				\IPS\Output::i()->error( 'ms_no_permission_sending', 'MEMBERSSHOP/1', 404, '' );
			}
		}
		catch ( \OutOfRangeException $e ) {}
		
		try
		{
			$getItem = \IPS\membersshop\Purchases::load( $id );
			
			if ( $getItem->redeemed == 1 OR $getItem->member_id !== $member->member_id )
			{
				\IPS\Output::i()->error( 'ms_node_error', 'MEMBERSHOP/1', 403, '' );
			}
		}
		catch( \UnderflowException $ex )
		{
			\IPS\Output::i()->error( 'node_error', 'MEMBERSSHOP/8', 403, '' );
		}
		
		try
		{
			$myNumber = $getItem->price;
			$percentToGet = \IPS\Settings::i()->ms_charge_to_send;
			$percentInDecimal = $percentToGet / 100;
			$cost = $percentInDecimal * $myNumber;
		}
		catch( \UnderflowException $ex )
		{
			\IPS\Output::i()->error( 'node_error', 'MEMBERSSHOP/8', 403, '' );
		}
		
		$form = new \IPS\Helpers\Form( 'membersshop', 'Send' );
		$form->class = 'ipsForm_vertical ipsForm_fullWidth ipsForm_noLabels';
		$form->actionButtons[0] = \IPS\Theme::i()->getTemplate( 'forms', 'core', 'global' )->button( 'ms_send', 'submit', null, 'ipsButton ipsButton_primary', array( 'tabindex' => '2', 'accesskey' => 's', 'data-action' => 'sendItem' ) );
		$form->add( new \IPS\Helpers\Form\Member('ms_send_to', NULL, TRUE, array( ), function( $val )
        {
			if ( $val == \IPS\Member::loggedIn() )
	        {
				if ( \IPS\Request::i()->isAjax() )
				{
					\IPS\Output::i()->json( array( 'message' => \IPS\Member::loggedIn()->language()->addToStack('ms_SendSelf') ) );
					return;
				}
		        throw new \DomainException('ms_SendSelf');
	        }
        }));
			
		if ( $values = $form->values() )
		{
			if ( $member->shopPoints() < $cost AND $member->shopPoints() !== $cost )
			{
				if ( \IPS\Request::i()->isAjax() )
				{
					\IPS\Output::i()->json( array( 'message' => \IPS\Member::loggedIn()->language()->addToStack('ms_you_dont_have_enough_points_send', FALSE, array( 'sprintf' => array( $pointsName ) ) ) ) );
					return;
				}
			    \IPS\Output::i()->error( \IPS\Member::loggedIn()->language()->addToStack('ms_you_dont_have_enough_points_send', FALSE, array( 'sprintf' => array( $pointsName ) ) ), 'MEMBERSSHOP/1', 403, '' );
			}
			
			$logsACP = sprintf( \IPS\Member::loggedIn()->language()->get('ms_acp_log_sent_item'), $getItem->name, $values['ms_send_to']->name );
			$member->removePoints( $cost, $logsACP );
			
			$getItem->member_id = $values['ms_send_to']->member_id;
			$getItem->save();
			
			$received = new \IPS\membersshop\Received;
			$received->item_id = $getItem->item_id;
			$received->sender_id = $member->member_id;
			$received->receiver_id = $values['ms_send_to']->member_id;
			$received->time_date = time();
			$received->save();
			
			$items = \IPS\membersshop\Received::load( $received->id );
			
			$memberNotifi = \IPS\Member::load( $values['ms_send_to']->member_id );
			$notification = new \IPS\Notification( \IPS\Application::load( 'membersshop' ), 'received_item', $items, array ( $items ) );
			$notification->recipients->attach( $memberNotifi );
			$notification->send();
			
			if ( \IPS\Request::i()->isAjax() )
			{
				\IPS\Output::i()->json( array( 'message' => 'Item Sent' ));
				return;
			}
			
			\IPS\Output::i()->redirect( \IPS\Request::i()->referrer() ?: \IPS\Http\Url::internal( 'app=membersshop&module=shop&controller=myitems', 'front', 'myitems' ), 'Item Sent' );
		}
		else
		{
			\IPS\Output::i()->output = $form->customTemplate( array( \call_user_func_array( array( \IPS\Theme::i(), 'getTemplate' ), array( 'forms', 'core' ) ), 'popupTemplate' ) );
		}
	}
	
	protected function addPosts( $getItem, $member, $boughtAndUsed, $categoryURL )
	{
		$minRand = $getItem->minRand;
		$maxRand = $getItem->maxRand;
		
		if ( $getItem->extra_field )
		{
			if ( $getItem->extra_field != -1 )
			{
				$winPercentage = $getItem->extra_field;
				$chance = rand( 1, 100 );
				if ( $chance > $winPercentage )
				{
					$getItem->redeemed = 1;
					$getItem->save();
					$outputLang = \IPS\Member::loggedIn()->language()->addToStack('ms_sorry_you_lost');
					
					if ( \IPS\Request::i()->isAjax() )
					{
						\IPS\Output::i()->json( array( 'message' => $outputLang ));
						return;
					}
					else
					{
						if ( $boughtAndUsed )
						{
							$redirectURL = $categoryURL;
						}
						else
						{
							$redirectURL = \IPS\Request::i()->referrer() ?: \IPS\Http\Url::internal( 'app=membersshop&module=shop&controller=myitems', 'front', 'myitems' );
						}
						
						\IPS\Output::i()->redirect( $redirectURL, $outputLang );
					}
				}
			}
		}
		
		$rand = rand( $minRand, $maxRand );
		$member->member_posts = $member->member_posts + $rand;
		$member->save();
		
		$getItem->redeemed = 1;
		$getItem->save();
		
		$reward = new \IPS\membersshop\Rewards;
		$reward->item_id = $getItem->item_id;
		$reward->member_id = $member->member_id;
		$reward->time_date = time();
		$reward->reward = $rand;
		$reward->name = $getItem->name;
		$reward->save();
		
		$outputLang = \IPS\Member::loggedIn()->language()->addToStack('ms_posts_awarded', FALSE, array( 'sprintf' => array( number_format( $rand ) ) ) );
   	    
		if ( \IPS\Request::i()->isAjax() )
		{
			\IPS\Output::i()->json( array( 'message' => $outputLang ));
			return;
		}
		else
		{
			if ( $boughtAndUsed )
			{
				$redirectURL = $categoryURL;
			}
			else
			{
				$redirectURL = \IPS\Request::i()->referrer() ?: \IPS\Http\Url::internal( 'app=membersshop&module=shop&controller=myitems', 'front', 'myitems' );
			}
			
			\IPS\Output::i()->redirect( $redirectURL, $outputLang );
		}
	}
	
	protected function addRep( $getItem, $member, $boughtAndUsed, $categoryURL )
	{
		$minRand = $getItem->minRand;
		$maxRand = $getItem->maxRand;
		
		if ( $getItem->extra_field )
		{
			if ( $getItem->extra_field != -1 )
			{
				$winPercentage = $getItem->extra_field;
				$chance = rand( 1, 100 );
				if ( $chance > $winPercentage )
				{
					$getItem->redeemed = 1;
					$getItem->save();
					$outputLang = \IPS\Member::loggedIn()->language()->addToStack('ms_sorry_you_lost');
					
					if ( \IPS\Request::i()->isAjax() )
					{
						\IPS\Output::i()->json( array( 'message' => $outputLang ));
						return;
					}
					else
					{
						if ( $boughtAndUsed )
						{
							$redirectURL = $categoryURL;
						}
						else
						{
							$redirectURL = \IPS\Request::i()->referrer() ?: \IPS\Http\Url::internal( 'app=membersshop&module=shop&controller=myitems', 'front', 'myitems' );
						}
						
						\IPS\Output::i()->redirect( $redirectURL, $outputLang );
					}
				}
			}
		}
		
		$rand = rand( $minRand, $maxRand );
		$member->pp_reputation_points = $member->pp_reputation_points + $rand;
		$member->save();
		
		$getItem->redeemed = 1;
		$getItem->save();
		
		$reward = new \IPS\membersshop\Rewards;
		$reward->item_id = $getItem->item_id;
		$reward->member_id = $member->member_id;
		$reward->time_date = time();
		$reward->reward = $rand;
		$reward->name = $getItem->name;
		$reward->save();
		
		$outputLang = ( number_format( $rand ) ) ? \IPS\Member::loggedIn()->language()->addToStack('ms_rep_awarded', FALSE, array( 'sprintf' => array( number_format( $rand ) ) ) ) : 'saved';
   	    
		if ( \IPS\Request::i()->isAjax() )
		{
			\IPS\Output::i()->json( array( 'message' => $outputLang ));
			return;
		}
		else
		{
			if ( $boughtAndUsed )
			{
				$redirectURL = $categoryURL;
			}
			else
			{
				$redirectURL = \IPS\Request::i()->referrer() ?: \IPS\Http\Url::internal( 'app=membersshop&module=shop&controller=myitems', 'front', 'myitems' );
			}
		
			\IPS\Output::i()->redirect( $redirectURL, $outputLang );
		}
	}
	
	protected function stealRep( $getItem, $member, $boughtAndUsed, $categoryURL )
	{
		\IPS\Output::i()->jsFiles = array_merge( \IPS\Output::i()->jsFiles, \IPS\Output::i()->js( 'global_shop.js', 'membersshop', 'global' ) );
		
		$form = new \IPS\Helpers\Form( 'membersshop', 'ms_steal' );
		$form->class = 'ipsForm_vertical ipsForm_fullWidth ipsForm_noLabels ipsBox';
		if ( $boughtAndUsed )
		{
			$base        = 'front';
			$seoTemplate = 'my_items_redeem';
			$seoTitles   = array( \IPS\Http\Url\Friendly::seoTitle( $getItem->name ) );
			$categoryID = \IPS\membersshop\Items::load( $getItem->item_id, 'item_id' )->cat_id;
			$url = \IPS\Http\Url::internal( "app=membersshop&module=shop&controller=myitems&do=redeem&id={$getItem->id}&b=1&c={$categoryID}", $base, $seoTemplate, $seoTitles  );
			$form->action = $url;
		}
		
		$form->add(new \IPS\Helpers\Form\Member('ms_stealfrom', NULL, TRUE, array( ), function( $val )
        {
			if ( $val == \IPS\Member::loggedIn() )
	        {
		        throw new \DomainException('ms_StealSelf');
	        }
        }
		));
		
		if ( $values = $form->values( ) )
		{
			$minRand = $getItem->minRand;
		    $maxRand = $getItem->maxRand;
		
		    $stealingfrom = $values['ms_stealfrom'];
		    $rand = rand( $minRand, $maxRand );
		    $member->pp_reputation_points = $member->pp_reputation_points + $rand;
		    $member->save();
			$stealingfrom->pp_reputation_points = $stealingfrom->pp_reputation_points - $rand;
			$stealingfrom->save();
			
			$getItem->redeemed = 1;
		    $getItem->save();
			
			$reward = new \IPS\membersshop\Rewards;
		    $reward->item_id = $getItem->item_id;
		    $reward->member_id = $member->member_id;
		    $reward->time_date = time();
		    $reward->reward = $rand ;
			$reward->stolen_from = $stealingfrom->member_id;
			$reward->name = $getItem->name;
		    $reward->save();
		
		    $items = \IPS\membersshop\Rewards::load( $reward->id );
			
			$memberNotifi = \IPS\Member::load( $values['ms_stealfrom']->member_id );
			$notification = new \IPS\Notification( \IPS\Application::load( 'membersshop' ), 'stolen_rep', $items, array ( $items ) );
			$notification->recipients->attach( $memberNotifi );
			$notification->send();	
			
			$outputLang = ( number_format( $rand ) ) ? \IPS\Member::loggedIn()->language()->addToStack('ms_rep_stolen', FALSE, array( 'sprintf' => array( number_format( $rand ) ) ) ) : 'saved';
			
			if ( \IPS\Request::i()->isAjax() )
			{
				\IPS\Output::i()->json( array( 'message' => $outputLang ));
				return;
			}
			else
			{
				if ( $boughtAndUsed )
				{
					$redirectURL = $categoryURL;
				}
				else
				{
					$redirectURL = \IPS\Request::i()->referrer() ?: \IPS\Http\Url::internal( 'app=membersshop&module=shop&controller=myitems', 'front', 'myitems' );
				}
				
				\IPS\Output::i()->redirect( $redirectURL, $outputLang );
			}
		}
		else
		{
			if ( \IPS\Request::i()->isAjax() )
			{
				\IPS\Output::i()->json( array(
					'contents' 	=> $form->customTemplate( array( \IPS\Theme::i()->getTemplate( 'shop', 'membersshop' ), 'settingsForm' ), $getItem->name )
				)) ;
				return;
			}
			else
			{
				\IPS\Output::i()->title = $getItem->name;
				\IPS\Output::i()->output = $form->customTemplate( array( \IPS\Theme::i()->getTemplate( 'shop', 'membersshop' ), 'settingsForm' ), $getItem->name );
			}
		}
	}
	
	protected function changeUsername( $getItem, $member, $boughtAndUsed, $categoryURL )
	{
		\IPS\Output::i()->jsFiles = array_merge( \IPS\Output::i()->jsFiles, \IPS\Output::i()->js( 'global_shop.js', 'membersshop', 'global' ) );
		
		$form = new \IPS\Helpers\Form( 'form', 'ms_change', NULL, array( 'data-controller' => 'core.front.system.register') );
		$form->class = 'ipsForm_vertical ipsForm_fullWidth ipsForm_noLabels ipsBox';
		if ( $boughtAndUsed )
		{
			$base        = 'front';
			$seoTemplate = 'my_items_redeem';
			$seoTitles   = array( \IPS\Http\Url\Friendly::seoTitle( $getItem->name ) );
			$categoryID = \IPS\membersshop\Items::load( $getItem->item_id, 'item_id' )->cat_id;
			$url = \IPS\Http\Url::internal( "app=membersshop&module=shop&controller=myitems&do=redeem&id={$getItem->id}&b=1&c={$categoryID}", $base, $seoTemplate, $seoTitles  );
			$form->action = $url;
		}
		$form->add( new \IPS\Helpers\Form\Text( 'username', NULL, TRUE, array( 'accountUsername' => TRUE ) ) );
		
		if ( $values = $form->values( ) )
		{
		    $member->name = $values['username'];
			$member->members_seo_name = \IPS\Http\Url\Friendly::seoTitle( $values['username'] );
		    $member->save();
		    
			$getItem->redeemed = 1;
		    $getItem->save();

			$reward = new \IPS\membersshop\Rewards;
			$reward->item_id = $getItem->item_id;
			$reward->member_id = $member->member_id;
			$reward->time_date = time();
			$reward->name = $getItem->name;
		    $reward->save();

            $outputLang = ( $member->name ) ? \IPS\Member::loggedIn()->language()->addToStack('ms_changed_username', FALSE, array( 'sprintf' => array( $member->name ) ) ) : 'saved';
   	        
			if ( \IPS\Request::i()->isAjax() )
			{
				\IPS\Output::i()->json( array( 'message' => $outputLang ));
				return;
			}
			else
			{
				if ( $boughtAndUsed )
				{
					$redirectURL = $categoryURL;
				}
				else
				{
					$redirectURL = \IPS\Request::i()->referrer() ?: \IPS\Http\Url::internal( 'app=membersshop&module=shop&controller=myitems', 'front', 'myitems' );
				}
				
				\IPS\Output::i()->redirect( $redirectURL, $outputLang );
			}
		}
		else
		{
			if ( \IPS\Request::i()->isAjax() )
			{
				\IPS\Output::i()->json( array(
					'contents' 	=> $form->customTemplate( array( \IPS\Theme::i()->getTemplate( 'shop', 'membersshop' ), 'settingsForm' ), $getItem->name )
				)) ;
				return;
			}
			else
			{
				\IPS\Output::i()->title = $getItem->name;
				\IPS\Output::i()->output = $form->customTemplate( array( \IPS\Theme::i()->getTemplate( 'shop', 'membersshop' ), 'settingsForm' ), $getItem->name );
			}
		}
	}
	
	protected function gamblePoints( $getItem, $member, $boughtAndUsed, $categoryURL )
	{
		$pointsName = \IPS\Settings::i()->ms_points_name;
		$minRand = $getItem->minRand;
		$maxRand = $getItem->maxRand;
		
		if ( $getItem->extra_field )
		{
			if ( $getItem->extra_field != -1 )
			{
				$winPercentage = $getItem->extra_field;
				$chance = rand( 1, 100 );
				if ( $chance > $winPercentage )
				{
					$getItem->redeemed = 1;
					$getItem->save();
					$outputLang = \IPS\Member::loggedIn()->language()->addToStack('ms_sorry_you_lost');
					
					if ( \IPS\Request::i()->isAjax() )
					{
						\IPS\Output::i()->json( array( 'message' => $outputLang ));
						return;
					}
					else
					{
						if ( $boughtAndUsed )
						{
							$redirectURL = $categoryURL;
						}
						else
						{
							$redirectURL = \IPS\Request::i()->referrer() ?: \IPS\Http\Url::internal( 'app=membersshop&module=shop&controller=myitems', 'front', 'myitems' );
						}
						
						\IPS\Output::i()->redirect( $redirectURL, $outputLang );
					}
				}
			}
		}
		
		$rand = rand( $minRand, $maxRand );
		
		$logsACP = sprintf( \IPS\Member::loggedIn()->language()->get('ms_acp_log_gamble_points'), $getItem->name );
		$member->addPoints( $rand, $logsACP );
		
		$getItem->redeemed = 1;
		$getItem->save();
		
		$reward = new \IPS\membersshop\Rewards;
		$reward->item_id = $getItem->item_id;
		$reward->member_id = $member->member_id;
		$reward->time_date = time();
		$reward->reward = $rand ;
		$reward->name = $getItem->name;
		$reward->save();
		
		$outputLang = ( number_format( $rand ) ) ? \IPS\Member::loggedIn()->language()->addToStack('ms_points_won', FALSE, array( 'sprintf' => array( number_format( $rand ), $pointsName ) ) ) : 'saved';
   	    
		if ( \IPS\Request::i()->isAjax() )
		{
			\IPS\Output::i()->json( array( 'message' => $outputLang ));
			return;
		}
		else
		{
			if ( $boughtAndUsed )
			{
				$redirectURL = $categoryURL;
			}
			else
			{
				$redirectURL = \IPS\Request::i()->referrer() ?: \IPS\Http\Url::internal( 'app=membersshop&module=shop&controller=myitems', 'front', 'myitems' );
			}
			
			\IPS\Output::i()->redirect( $redirectURL, $outputLang );
		}
	}
	
	protected function changeTitle( $getItem, $member, $boughtAndUsed, $categoryURL )
	{
		\IPS\Output::i()->jsFiles = array_merge( \IPS\Output::i()->jsFiles, \IPS\Output::i()->js( 'global_shop.js', 'membersshop', 'global' ) );
		
		$form = new \IPS\Helpers\Form( 'form', 'ms_change', NULL, array( 'data-controller' => 'core.front.system.register') );
		$form->class = 'ipsForm_vertical ipsForm_fullWidth ipsForm_noLabels ipsBox';
		if ( $boughtAndUsed )
		{
			$base        = 'front';
			$seoTemplate = 'my_items_redeem';
			$seoTitles   = array( \IPS\Http\Url\Friendly::seoTitle( $getItem->name ) );
			$categoryID = \IPS\membersshop\Items::load( $getItem->item_id, 'item_id' )->cat_id;
			$url = \IPS\Http\Url::internal( "app=membersshop&module=shop&controller=myitems&do=redeem&id={$getItem->id}&b=1&c={$categoryID}", $base, $seoTemplate, $seoTitles  );
			$form->action = $url;
		}
		$form->add( new \IPS\Helpers\Form\Text( 'ms_title', $member->member_title, TRUE, array( ) ) );
		
		if ( $values = $form->values( ) )
		{
		    $member->member_title = $values['ms_title'];
			$member->save();
		    
			$getItem->redeemed = 1;
		    $getItem->save();
			
			$reward = new \IPS\membersshop\Rewards;
			$reward->item_id = $getItem->item_id;
			$reward->member_id = $member->member_id;
			$reward->time_date = time();
			$reward->name = $getItem->name;
		    $reward->save();
			
			$outputLang = ( $member->member_title ) ? \IPS\Member::loggedIn()->language()->addToStack('ms_changed_title', FALSE, array( 'sprintf' => array( $member->member_title ) ) ) : 'saved';
   	        
			if ( \IPS\Request::i()->isAjax() )
			{
				\IPS\Output::i()->json( array( 'message' => $outputLang ));
				return;
			}
			else
			{
				if ( $boughtAndUsed )
				{
					$redirectURL = $categoryURL;
				}
				else
				{
					$redirectURL = \IPS\Request::i()->referrer() ?: \IPS\Http\Url::internal( 'app=membersshop&module=shop&controller=myitems', 'front', 'myitems' );
				}
				
				\IPS\Output::i()->redirect( $redirectURL, $outputLang );
			}
		}
		else
		{
			if ( \IPS\Request::i()->isAjax() )
			{
				\IPS\Output::i()->json( array(
					'contents' 	=> $form->customTemplate( array( \IPS\Theme::i()->getTemplate( 'shop', 'membersshop' ), 'settingsForm' ), $getItem->name )
				)) ;
				return;
			}
			else
			{
				\IPS\Output::i()->title = $getItem->name;
				\IPS\Output::i()->output = $form->customTemplate( array( \IPS\Theme::i()->getTemplate( 'shop', 'membersshop' ), 'settingsForm' ), $getItem->name );
			}
		}
	}
	
	protected function resetWarnPoints( $getItem, $member, $boughtAndUsed, $categoryURL )
	{
		$member->warn_level = 0;
		$member->save();
		
		$getItem->redeemed = 1;
		$getItem->save();
		
		$reward = new \IPS\membersshop\Rewards;
		$reward->item_id = $getItem->item_id;
		$reward->member_id = $member->member_id;
		$reward->time_date = time();
		$reward->name = $getItem->name;
		$reward->save();
		
		$outputLang = \IPS\Member::loggedIn()->language()->addToStack('ms_reset_warning_points');
		
		if ( \IPS\Request::i()->isAjax() )
		{
			\IPS\Output::i()->json( array( 'message' => $outputLang ));
			return;
		}
		else
		{
			if ( $boughtAndUsed )
			{
				$redirectURL = $categoryURL;
			}
			else
			{
				$redirectURL = \IPS\Request::i()->referrer() ?: \IPS\Http\Url::internal( 'app=membersshop&module=shop&controller=myitems', 'front', 'myitems' );
			}
		
			\IPS\Output::i()->redirect( $redirectURL, 'ms_reset_warning_points' );
		}
	}
	
	protected function forumPassword( $getItem, $member, $boughtAndUsed, $categoryURL )
	{
		\IPS\Output::i()->jsFiles = array_merge( \IPS\Output::i()->jsFiles, \IPS\Output::i()->js( 'global_shop.js', 'membersshop', 'global' ) );
		
		$form = new \IPS\Helpers\Form( 'membersshop', 'ms_show_password' );
		$form->class = 'ipsForm_vertical ipsForm_fullWidth ipsForm_noLabels ipsBox';
		if ( $boughtAndUsed )
		{
			$base        = 'front';
			$seoTemplate = 'my_items_redeem';
			$seoTitles   = array( \IPS\Http\Url\Friendly::seoTitle( $getItem->name ) );
			$categoryID = \IPS\membersshop\Items::load( $getItem->item_id, 'item_id' )->cat_id;
			$url = \IPS\Http\Url::internal( "app=membersshop&module=shop&controller=myitems&do=redeem&id={$getItem->id}&b=1&c={$categoryID}", $base, $seoTemplate, $seoTitles  );
			$form->action = $url;
		}
		$form->add( new \IPS\Helpers\Form\Node( 'ms_ForumCat', NULL, TRUE, array( 'class' => 'IPS\forums\Forum', 'permissionCheck' => function ( $forum ) { return $forum->sub_can_post and !$forum->redirect_url and $forum->password; } ), NULL, NULL, NULL, 'ms_ForumCat' ) );
		
		if( $values = $form->values() )
		{
		    $forum_id = $values['ms_ForumCat']->id;
			$getItem->redeemed = 1;
		    $getItem->save();
			
			$reward = new \IPS\membersshop\Rewards;
			$reward->item_id = $getItem->item_id;
			$reward->member_id = $member->member_id;
			$reward->time_date = time();
			$reward->name = $getItem->name;
		    $reward->save();
			
			if ( \IPS\Request::i()->isAjax() )
			{
				\IPS\Output::i()->json( array( 
					'extra' => \IPS\Theme::i()->getTemplate( 'shop', 'membersshop' )->forumpassword( $forum_id )
				));
				return;
			}
			else
			{
				\IPS\Output::i()->output = \IPS\Theme::i()->getTemplate( 'shop', 'membersshop' )->forumpassword( $forum_id );
			}
		}
		else
		{
			if ( \IPS\Request::i()->isAjax() )
			{
				\IPS\Output::i()->json( array(
					'contents' 	=> $form->customTemplate( array( \IPS\Theme::i()->getTemplate( 'shop', 'membersshop' ), 'settingsForm' ), $getItem->name )
				)) ;
				return;
			}
			else
			{
				\IPS\Output::i()->output = $form->customTemplate( array( \IPS\Theme::i()->getTemplate( 'shop', 'membersshop' ), 'settingsForm' ), $getItem->name );
			}
		}
	}
	
	protected function addSignature( $getItem, $member, $boughtAndUsed, $categoryURL )
	{
		\IPS\Output::i()->jsFiles = array_merge( \IPS\Output::i()->jsFiles, \IPS\Output::i()->js( 'global_shop.js', 'membersshop', 'global' ) );
		
		$form = new \IPS\Helpers\Form( 'membersshop', 'save' );
		$form->class = 'ipsForm_vertical ipsForm_fullWidth ipsForm_noLabels ipsBox';
		if ( $boughtAndUsed )
		{
			$base        = 'front';
			$seoTemplate = 'my_items_redeem';
			$seoTitles   = array( \IPS\Http\Url\Friendly::seoTitle( $getItem->name ) );
			$categoryID = \IPS\membersshop\Items::load( $getItem->item_id, 'item_id' )->cat_id;
			$url = \IPS\Http\Url::internal( "app=membersshop&module=shop&controller=myitems&do=redeem&id={$getItem->id}&b=1&c={$categoryID}", $base, $seoTemplate, $seoTitles  );
			$form->action = $url;
		}
		$form->add( new \IPS\Helpers\Form\Editor( 'signature',  $member->signature, FALSE, array( 'app' => 'core', 'key' => 'Signatures', 'autoSaveKey' => "frontsig-" . $member->member_id, 'attachIds' => array(  $member->member_id ) ) ) );
		
		if( $values = $form->values() )
		{
		    \IPS\Db::i()->update( 'core_members', array( 'signature' => $values['signature'] ), array ( 'member_id=?', $member->member_id ) );
			$getItem->redeemed = 1;
		    $getItem->save();
			
			$reward = new \IPS\membersshop\Rewards;
			$reward->item_id = $getItem->item_id;
			$reward->member_id = $member->member_id;
			$reward->time_date = time();
			$reward->name = $getItem->name;
		    $reward->save();
			
			$outputLang = \IPS\Member::loggedIn()->language()->addToStack('ms_changed_signature');
			
			if ( \IPS\Request::i()->isAjax() )
			{
				\IPS\Output::i()->json( array( 'message' => $outputLang ));
				return;
			}
			else
			{
				if ( $boughtAndUsed )
				{
					$redirectURL = $categoryURL;
				}
				else
				{
					$redirectURL = \IPS\Request::i()->referrer() ?: \IPS\Http\Url::internal( 'app=membersshop&module=shop&controller=myitems', 'front', 'myitems' );
				}
				
				\IPS\Output::i()->redirect( $redirectURL, 'ms_changed_signature' );
			}
		}
		else
		{
			if ( \IPS\Request::i()->isAjax() )
			{
				\IPS\Output::i()->json( array(
					'contents' 	=> $form->customTemplate( array( \IPS\Theme::i()->getTemplate( 'shop', 'membersshop' ), 'settingsForm' ), $getItem->name )
				)) ;
				return;
			}
			else
			{
				\IPS\Output::i()->title = $getItem->name;
				\IPS\Output::i()->output = $form->customTemplate( array( \IPS\Theme::i()->getTemplate( 'shop', 'membersshop' ), 'settingsForm' ), $getItem->name );
			}
		}
	}
	
	protected function rockpaperscissors( $getItem, $member, $boughtAndUsed, $categoryURL )
	{
		\IPS\Output::i()->jsFiles = array_merge( \IPS\Output::i()->jsFiles, \IPS\Output::i()->js( 'global_shop.js', 'membersshop', 'global' ) );
		
		$pointsName = \IPS\Settings::i()->ms_points_name;
		
		$form = new \IPS\Helpers\Form( 'membersshop', \IPS\Member::loggedIn()->language()->addToStack('ms_play') );
		$form->class = 'ipsBox ipsSpacer_top ipsForm_vertical ipsForm_fullWidth';
		if ( $boughtAndUsed )
		{
			$base        = 'front';
			$seoTemplate = 'my_items_redeem';
			$seoTitles   = array( \IPS\Http\Url\Friendly::seoTitle( $getItem->name ) );
			$categoryID = \IPS\membersshop\Items::load( $getItem->item_id, 'item_id' )->cat_id;
			$url = \IPS\Http\Url::internal( "app=membersshop&module=shop&controller=myitems&do=redeem&id={$getItem->id}&b=1&c={$categoryID}", $base, $seoTemplate, $seoTitles  );
			$form->action = $url;
		}
		$form->addMessage('ms_rpsls_description');
		$form->add( new \IPS\Helpers\Form\Radio( 'ms_rps', NULL, TRUE, array( 'options' => array( 1 => 'ms_rock', 2 => 'ms_paper', 3 => 'ms_scissors', 4 => 'ms_lizard', 5 => 'ms_spock' )  ) ) );
		
		if ( $getItem->extra_field )
		{
			$pointsName = \IPS\Settings::i()->ms_points_name;
		    $form->addMessage('ms_rpsls_how_much_to_win');
			$form->add( new \IPS\Helpers\Form\Number('ms_rpsls_amount', NULL, FALSE, array( 'max' => $getItem->maxRand ), NULL, NULL, \IPS\Member::loggedIn()->language()->addToStack('ms_points', FALSE, array( 'sprintf' => array( $pointsName ) ) ), 'ms_rpsls_amount' , function( $val )
   		    {
				if ( \IPS\Member::loggedIn()->shopPoints() < $val )
				{
				    throw new \DomainException('ms_not_enough_points');
		    	}
   	        }
			));
	    }
		
		if( $values = $form->values() )
		{
			if ( $getItem->extra_field )
		    {
			    if ( $values['ms_rpsls_amount'] > 0 )
				{
					$logsACP = sprintf( \IPS\Member::loggedIn()->language()->get('ms_acp_log_upped_the_ante'), $getItem->name );
					$member->removePoints( $values['ms_rpsls_amount'], $logsACP );
			
					$getItem->price = $getItem->price + $values['ms_rpsls_amount'];
					$getItem->save();
			    }
			}
			
		    $rand = rand( 1, 5 );
			
			    if ( $rand == 1 AND $values['ms_rps'] == 1) { $weThrew = 1; $youthrew = 1; $outcome = 333; }
			elseif ( $rand == 1 AND $values['ms_rps'] == 2) { $weThrew = 1; $youthrew = 2; $outcome = 111; }
			elseif ( $rand == 1 AND $values['ms_rps'] == 3) { $weThrew = 1; $youthrew = 3; $outcome = 222; }
			elseif ( $rand == 1 AND $values['ms_rps'] == 4) { $weThrew = 1; $youthrew = 4; $outcome = 222; }
			elseif ( $rand == 1 AND $values['ms_rps'] == 5) { $weThrew = 1; $youthrew = 5; $outcome = 111; }
			
			elseif ( $rand == 2 AND $values['ms_rps'] == 1) { $weThrew = 2; $youthrew = 1; $outcome = 222; }
			elseif ( $rand == 2 AND $values['ms_rps'] == 2) { $weThrew = 2; $youthrew = 2; $outcome = 333; }
			elseif ( $rand == 2 AND $values['ms_rps'] == 3) { $weThrew = 2; $youthrew = 3; $outcome = 111; }
			elseif ( $rand == 2 AND $values['ms_rps'] == 4) { $weThrew = 2; $youthrew = 4; $outcome = 111; }
			elseif ( $rand == 2 AND $values['ms_rps'] == 5) { $weThrew = 2; $youthrew = 5; $outcome = 222; }
			
			elseif ( $rand == 3 AND $values['ms_rps'] == 1) { $weThrew = 3; $youthrew = 1; $outcome = 111; }
			elseif ( $rand == 3 AND $values['ms_rps'] == 2) { $weThrew = 3; $youthrew = 2; $outcome = 222; }
			elseif ( $rand == 3 AND $values['ms_rps'] == 3) { $weThrew = 3; $youthrew = 3; $outcome = 333; }
			elseif ( $rand == 3 AND $values['ms_rps'] == 4) { $weThrew = 3; $youthrew = 4; $outcome = 222; }
			elseif ( $rand == 3 AND $values['ms_rps'] == 5) { $weThrew = 3; $youthrew = 5; $outcome = 111; }
			
			elseif ( $rand == 4 AND $values['ms_rps'] == 1) { $weThrew = 4; $youthrew = 1; $outcome = 111; }
			elseif ( $rand == 4 AND $values['ms_rps'] == 2) { $weThrew = 4; $youthrew = 2; $outcome = 222; }
			elseif ( $rand == 4 AND $values['ms_rps'] == 3) { $weThrew = 4; $youthrew = 3; $outcome = 111; }
			elseif ( $rand == 4 AND $values['ms_rps'] == 4) { $weThrew = 4; $youthrew = 4; $outcome = 333; }
			elseif ( $rand == 4 AND $values['ms_rps'] == 5) { $weThrew = 4; $youthrew = 5; $outcome = 222; }
			
			elseif ( $rand == 5 AND $values['ms_rps'] == 1) { $weThrew = 5; $youthrew = 1; $outcome = 222; }
			elseif ( $rand == 5 AND $values['ms_rps'] == 2) { $weThrew = 5; $youthrew = 2; $outcome = 111; }
			elseif ( $rand == 5 AND $values['ms_rps'] == 3) { $weThrew = 5; $youthrew = 3; $outcome = 222; }
			elseif ( $rand == 5 AND $values['ms_rps'] == 4) { $weThrew = 5; $youthrew = 4; $outcome = 111; }
			elseif ( $rand == 5 AND $values['ms_rps'] == 5) { $weThrew = 5; $youthrew = 5; $outcome = 333; }
			
			if ( $outcome == 111 )
			{
				$amount = $getItem->price * 2;
				$logsACP = sprintf( \IPS\Member::loggedIn()->language()->get('ms_acp_log_won_rock_paper'), $getItem->name );
				$member->addPoints( $amount, $logsACP );
			}
			elseif ( $outcome == 222 )
			{
			
			}
			elseif ( $outcome == 333 )
			{
				$amount = $getItem->price;
				$logsACP = sprintf( \IPS\Member::loggedIn()->language()->get('ms_acp_log_won_rock_paper'), $getItem->name );
				$member->addPoints( $amount, $logsACP );
			}
			
			$getItem->redeemed = 1;
		    $getItem->save();
			
			    if ( $weThrew == 1 ){ $us = \IPS\Member::loggedIn()->language()->addToStack('ms_rock'); $ourIcon = '<i class="fa fa-hand-rock-o fa-5x"></i>'; }
			elseif ( $weThrew == 2 ){ $us = \IPS\Member::loggedIn()->language()->addToStack('ms_paper'); $ourIcon = '<i class="fa fa-hand-stop-o fa-5x"></i>'; }
			elseif ( $weThrew == 3 ){ $us = \IPS\Member::loggedIn()->language()->addToStack('ms_scissors'); $ourIcon = '<i class="fa fa-hand-scissors-o fa-5x"></i>'; }
			elseif ( $weThrew == 4 ){ $us = \IPS\Member::loggedIn()->language()->addToStack('ms_lizard'); $ourIcon = '<i class="fa fa-hand-lizard-o fa-5x"></i>'; }
			elseif ( $weThrew == 5 ){ $us = \IPS\Member::loggedIn()->language()->addToStack('ms_spock'); $ourIcon = '<i class="fa fa-hand-spock-o fa-5x"></i>'; }
			
			    if ( $youthrew == 1 ){ $you = \IPS\Member::loggedIn()->language()->addToStack('ms_rock'); $yourIcon = '<i class="fa fa-hand-rock-o fa-5x"></i>'; }
			elseif ( $youthrew == 2 ){ $you = \IPS\Member::loggedIn()->language()->addToStack('ms_paper'); $yourIcon = '<i class="fa fa-hand-stop-o fa-5x"></i>'; }
			elseif ( $youthrew == 3 ){ $you = \IPS\Member::loggedIn()->language()->addToStack('ms_scissors'); $yourIcon = '<i class="fa fa-hand-scissors-o fa-5x"></i>'; }
			elseif ( $youthrew == 4 ){ $you = \IPS\Member::loggedIn()->language()->addToStack('ms_lizard'); $yourIcon = '<i class="fa fa-hand-lizard-o fa-5x"></i>'; }
			elseif ( $youthrew == 5 ){ $you = \IPS\Member::loggedIn()->language()->addToStack('ms_spock'); $yourIcon = '<i class="fa fa-hand-spock-o fa-5x"></i>'; }
			
			    if ( $outcome == 111 ){ $result = \IPS\Member::loggedIn()->language()->addToStack('ms_win'); $amount = $getItem->price * 2; }
			elseif ( $outcome == 222 ){ $result = \IPS\Member::loggedIn()->language()->addToStack('ms_lose'); $amount = 0; }
			elseif ( $outcome == 333 ){ $result = \IPS\Member::loggedIn()->language()->addToStack('ms_draw'); $amount = $getItem->price; }
			
			$reward = new \IPS\membersshop\Rewards;
			$reward->item_id = $getItem->item_id;
			$reward->member_id = $member->member_id;
			$reward->time_date = time();
			$reward->reward = $amount;
			$reward->name = $getItem->name;
		    $reward->save();
			
			if ( \IPS\Request::i()->isAjax() )
			{
				\IPS\Output::i()->json( array( 
					'extra' => \IPS\Theme::i()->getTemplate( 'shop', 'membersshop' )->rpsls( $us, $you, $result, $ourIcon, $yourIcon, $amount )
				));
				return;
			}
			else
			{
				\IPS\Output::i()->title = \IPS\Member::loggedIn()->language()->addToStack('ms_rps_result');
				return \IPS\Output::i()->output = \IPS\Theme::i()->getTemplate( 'shop', 'membersshop' )->rpsls( $us, $you, $result, $ourIcon, $yourIcon, $amount );
			}
		}
		if ( \IPS\Request::i()->isAjax() )
		{
			\IPS\Output::i()->json( array(
				'contents' 	=> $form->customTemplate( array( \IPS\Theme::i()->getTemplate( 'shop', 'membersshop' ), 'settingsForm' ), $getItem->name )
			)) ;
			return;
		}
		else
		{
			\IPS\Output::i()->title = $getItem->name;
			\IPS\Output::i()->output = $form->customTemplate( array( \IPS\Theme::i()->getTemplate( 'shop', 'membersshop' ), 'settingsForm' ), $getItem->name );
		}
	}
	
	protected function pinTopic()
	{
		$member = \IPS\Member::loggedIn();
		$id  = \intval( \IPS\Request::i()->id );
		$tid = \intval( \IPS\Request::i()->tid );
		try
		{
			$getItem = \IPS\membersshop\Purchases::load( $id );
			if ( $getItem->redeemed == 1 OR $getItem->member_id !== $member->member_id )
			{
				\IPS\Output::i()->error( 'ms_node_error', 'MEMBERSHOP/1', 403, '' );
			}
		}
		catch( \UnderflowException $ex )
		{
			\IPS\Output::i()->error( 'node_error', 'MEMBERSSHOP/8', 403, '' );
		}
		
		$topic = \IPS\forums\Topic::load( $tid );
		if ( $topic->pinned ) 
		{
			\IPS\Output::i()->error( 'ms_already_pinned', 'MEMBERSHOP/1', 403, '' );
		}
		
		if ( \IPS\Settings::i()->ms_pin_own_topics AND $topic->author() != $member )
		{
			\IPS\Output::i()->error( 'ms_pin_own_topics_error', 'MEMBERSHOP/1', 403, '' );
		}
		
		$topic->pinned = 1;
		$topic->save();
		
		$pinned = new \IPS\membersshop\Pinned;
		$pinned->member_id = $member->member_id;
		$pinned->topic_id = $tid;
		$pinned->time_date = time();
		$pinned->expired = 0;
		$pinned->expire_time_date = \IPS\DateTime::create()->add( new \DateInterval( 'P' . $getItem->extra_field . 'D' ) )->getTimestamp();
		$pinned->save();
		
		$getItem->redeemed = 1;
		$getItem->save();
		
		$reward = new \IPS\membersshop\Rewards;
		$reward->item_id = 11;
		$reward->member_id = $member->member_id;
		$reward->time_date = time();
		$reward->name = $getItem->name;
		$reward->reward = $tid;
		$reward->save();
		
		$expire = \IPS\DateTime::create()->setTimestamp( \IPS\DateTime::create()->add( new \DateInterval( 'P' . $getItem->extra_field . 'D' ) )->getTimestamp() );
		$topic = \IPS\forums\Topic::load( $tid );
		
		$outputLang = ( $expire ) ? \IPS\Member::loggedIn()->language()->addToStack('ms_pinned_topic_popup', FALSE, array( 'sprintf' => array( $topic->title, $expire ) ) ) : 'saved';
   	    \IPS\Output::i()->redirect( \IPS\Request::i()->referrer() ?: $topic->url(), $outputLang );
		
	}
	
	protected function featureFile()
	{
		if ( \IPS\Application::appIsEnabled( 'downloads') )
		{
			$member = \IPS\Member::loggedIn();
			$id  = \intval( \IPS\Request::i()->id );
			$fid = \intval( \IPS\Request::i()->fid );
			try
			{
				$getItem = \IPS\membersshop\Purchases::load( $id );
				if ( $getItem->redeemed == 1 OR $getItem->member_id !== $member->member_id )
				{
					\IPS\Output::i()->error( 'ms_node_error', 'MEMBERSHOP/1', 403, '' );
				}
			}
			catch( \UnderflowException $ex )
			{
				\IPS\Output::i()->error( 'node_error', 'MEMBERSSHOP/8', 403, '' );
			}
		
			$file = \IPS\downloads\File::load( $fid );
			
			if ( $file->featured ) 
			{
				\IPS\Output::i()->error( 'ms_already_featured', 'MEMBERSHOP/1', 403, '' );
			}
			
			$file->featured = 1;
			$file->save();
		
			$featured = new \IPS\membersshop\Featured;
			$featured->member_id = $member->member_id;
			$featured->fid = $file->id;
			$featured->time_date = time();
			$featured->expired = 0;
			$featured->expire_time_date = \IPS\DateTime::create()->add( new \DateInterval( 'P' . $getItem->extra_field . 'D' ) )->getTimestamp();
			$featured->save();
		
			$getItem->redeemed = 1;
			$getItem->save();
			
			$reward = new \IPS\membersshop\Rewards;
			$reward->item_id = 12;
			$reward->member_id = $member->member_id;
			$reward->time_date = time();
			$reward->name = $getItem->name;
			$reward->reward = $file->id;
		    $reward->save();
			
			$expire = \IPS\DateTime::create()->setTimestamp( \IPS\DateTime::create()->add( new \DateInterval( 'P' . $getItem->extra_field . 'D' ) )->getTimestamp() );
			$getFile = \IPS\downloads\File::load( $fid );
			
			$outputLang = ( $expire ) ? \IPS\Member::loggedIn()->language()->addToStack('ms_featured_file_popup', FALSE, array( 'sprintf' => array( $getFile->name, $expire ) ) ) : 'saved';
   		    \IPS\Output::i()->redirect( \IPS\Request::i()->referrer() ?: $file->url(), $outputLang );
		
		}
		else
		{
			\IPS\Output::i()->error( 'ms_downloads_error', 'MEMBERSHOP/1', 403, '' );
		}
	}
	
	protected function upgradeUsergroup( $getItem, $member, $boughtAndUsed, $categoryURL )
	{
		if ( $member->isAdmin() AND $member->modPermission() )
		{
			\IPS\Output::i()->error( 'ms_your_usergroup_is_protected', 'MEMBERSHOP/1', 403, '' );
		}
		
		try
		{
			$count = \IPS\membersshop\Upgrades::count();
            if ( $count > 0 )
			{
                \IPS\Output::i()->error( 'ms_wait_before_upgrading', 'MEMBERSHOP/1', 403, '' );
			}
		}
		catch( \UnderflowException $ex )
		{
			\IPS\Output::i()->error( 'node_error', 'MEMBERSSHOP/8', 403, '' );
		}
		
		$group = \IPS\Member\Group::load( $getItem->minRand );
		if ( $getItem->extra_field != NULL )
		{
			$upgradeMembership = new \IPS\membersshop\Upgrades;
			$upgradeMembership->member_id = $member->member_id;
			$upgradeMembership->original_group = $member->member_group_id;
			$upgradeMembership->reward_group = $group->g_id;
			$upgradeMembership->expired = 0;
			$upgradeMembership->time_date = time();
			if ( $getItem->maxRand == 0 )
			{
			    $upgradeMembership->value = $getItem->extra_field;
				$upgradeMembership->period = 'D';
				$amount = \IPS\DateTime::create()->add( new \DateInterval( 'P' . $getItem->extra_field . 'D' ) )->getTimestamp();
				$upgradeMembership->expire_time_date = $amount;
			}
			if ( $getItem->maxRand == 1 )
			{
			    $upgradeMembership->value = $getItem->extra_field;
				$upgradeMembership->period = 'M';
				$amount = \IPS\DateTime::create()->add( new \DateInterval( 'P' . $getItem->extra_field . 'M' ) )->getTimestamp();
				$upgradeMembership->expire_time_date = $amount;
			}
			if ( $getItem->maxRand == 2 )
			{
			    $upgradeMembership->value = $getItem->extra_field;
				$upgradeMembership->period = 'Y';
				$amount = \IPS\DateTime::create()->add( new \DateInterval( 'P' . $getItem->extra_field . 'Y' ) )->getTimestamp();
				$upgradeMembership->expire_time_date = $amount;
			}
			$upgradeMembership->save();
		}
		else
		{
			$upgradeMembership = new \IPS\membersshop\Upgrades;
			$upgradeMembership->member_id = $member->member_id;
			$upgradeMembership->original_group = $member->member_group_id;
			$upgradeMembership->reward_group = $group->g_id;
			$upgradeMembership->expired = 0;
			$upgradeMembership->time_date = time();
			$upgradeMembership->expire_time_date = -1;
			$upgradeMembership->save();
		}
		$member->member_group_id = $group->g_id;
		$member->save();
		
		$getItem->redeemed = 1;
		$getItem->save();
		
		$reward = new \IPS\membersshop\Rewards;
		$reward->item_id = $getItem->item_id;
		$reward->member_id = $member->member_id;
		$reward->time_date = time();
		$reward->name = $getItem->name;
		$reward->save();
		
		if ( $getItem->extra_field != NULL )
		{
			$items = \IPS\membersshop\Rewards::load( $reward->id );
			$notification = new \IPS\Notification( \IPS\Application::load( 'membersshop' ), 'rewards_reward', $items, array ( $items ) );
			$notification->recipients->attach( $member );
			$notification->send();
			
			if ( \IPS\Request::i()->isAjax() )
			{
				\IPS\Output::i()->json( array( 'message' => 'saved' ));
				return;
			}
			else
			{
				if ( $boughtAndUsed )
				{
					$redirectURL = $categoryURL;
				}
				else
				{
					$redirectURL = \IPS\Request::i()->referrer() ?: \IPS\Http\Url::internal( 'app=membersshop&module=shop&controller=myitems', 'front', 'myitems' );
				}
				
				\IPS\Output::i()->redirect( $redirectURL );
			}
		}
		else
		{
			$outputLang = ( $group->name ) ? \IPS\Member::loggedIn()->language()->addToStack('ms_perm_upgrade', FALSE, array( 'sprintf' => array( $group->name ) ) ) : 'saved';
			
			if ( \IPS\Request::i()->isAjax() )
			{
				\IPS\Output::i()->json( array( 'message' => $outputLang ));
				return;
			}
			else
			{
				if ( $boughtAndUsed )
				{
					$redirectURL = $categoryURL;
				}
				else
				{
					$redirectURL = \IPS\Request::i()->referrer() ?: \IPS\Http\Url::internal( 'app=membersshop&module=shop&controller=myitems', 'front', 'myitems' );
				}
				
				\IPS\Output::i()->redirect( $redirectURL, $outputLang );
			}
		}
	}
	
	protected function randomTrophy( $getItem, $member, $boughtAndUsed, $categoryURL )
	{
		if ( \IPS\Application::appIsEnabled( 'trophies') )
		{
			$count = (int) \IPS\Db::i()->select( 'count(*)', 'trophies_trophy', array( 'id IS NOT NULL' ) )->first();
			foreach ( \IPS\Db::i()->select( '*', 'trophies_trophy', array( 'sort=? AND enabled=1', rand( 1, $count ) ), NULL, 1, NULL ) as $trophy )
			{
				$trophyCount =  (int) \IPS\Db::i()->select( 'count(*)', 'trophies_members', array( 'member_id=? AND trophy_id=?', $member->member_id, $trophy['id'] ) )->first();
				if ( $trophyCount > 0 )
				{
					$getItem->redeemed = 1;
					$getItem->save();
					$outputLang = \IPS\Member::loggedIn()->language()->addToStack('ms_trophy_try_again');
					if ( \IPS\Request::i()->isAjax() )
					{
						\IPS\Output::i()->json( array( 'message' => $outputLang ));
						return;
					}
					else
					{
						\IPS\Output::i()->redirect( \IPS\Http\Url::internal( 'app=membersshop&module=shop&controller=myitems' ), 'ms_trophy_try_again' );
					}
				}
				else
				{
					$reward = new \IPS\membersshop\Rewards;
					$reward->item_id = $getItem->item_id;
					$reward->member_id = $member->member_id;
					$reward->time_date = time();
					$reward->name = $getItem->name;
					$reward->save();
			        
					\IPS\Db::i()->insert( 'trophies_members', array ( 'member_id' => $member->member_id, 'date' => time(), 'data' => '[]', 'trophy_id' => $trophy['id'], 'awarded_by' => 0, 'hidden' => 0, 'replaced_by_other_trophy' => 0 ) );
				
					$getItem->redeemed = 1;
					$getItem->save();
					$trophyName = \IPS\trophies\Trophy::load( $trophy['id'] );	
			
					$outputLang = ( $trophyName->_title ) ? \IPS\Member::loggedIn()->language()->addToStack('ms_trophy_won', FALSE, array( 'sprintf' => array( $trophyName->_title ) ) ) : 'saved';
   	                
					if ( \IPS\Request::i()->isAjax() )
					{
						\IPS\Output::i()->json( array( 'message' => $outputLang ));
						return;
					}
					else
					{
						if ( $boughtAndUsed )
						{
							$redirectURL = $categoryURL;
						}
						else
						{
							$redirectURL = \IPS\Request::i()->referrer() ?: \IPS\Http\Url::internal( 'app=membersshop&module=shop&controller=myitems', 'front', 'myitems' );
						}
		
						\IPS\Output::i()->redirect( $redirectURL, $outputLang );
					}
				}
			}
		}
		else
		{
			\IPS\Output::i()->error( 'ms_trophies_error', 'MEMBERSHOP/1', 403, '' );
		}	
	}
	
	protected function customPurchase( $getItem, $member, $boughtAndUsed, $categoryURL )
	{
		$getItem->redeemed = 1;
		$getItem->save();
		
		$CustomPurchase = new \IPS\membersshop\Custom;
		$CustomPurchase->member_id = $member->member_id;
		$CustomPurchase->name = $getItem->name;
		$CustomPurchase->description = $getItem->description;
		$CustomPurchase->time_date = time();
		$CustomPurchase->item_id = $getItem->id;
		$CustomPurchase->price = $getItem->price;
		$CustomPurchase->icon = $getItem->icon;
		$CustomPurchase->save();
		
		try
		{
			\IPS\core\AdminNotification::send( 'membersshop', 'Customitem', $CustomPurchase->id, TRUE );
		}
		catch( \Exception $ex ){}
		
		/*
		if ( \IPS\Settings::i()->ms_cp_notifi_members_enable )
		{
			$items = \IPS\membersshop\Custom::load( $CustomPurchase->id );
			$members = \IPS\Member::load( \IPS\Settings::i()->ms_cp_notifi_members );
			$notification = new \IPS\Notification( \IPS\Application::load( 'membersshop' ), 'custom_purchase_mod', $items, array ( $items ) );
			$notification->recipients->attach( $members );
			$notification->send();
		}
		if ( \IPS\Settings::i()->ms_cp_email_members_enable )
		{
			try
			{
				$emailMessage = ( $member->name ) ? \IPS\Member::loggedIn()->language()->addToStack('ms_email_message', FALSE, array( 'sprintf' => array( $member->name, $getItem->name ) ) ) : 'saved';
   		    	$emailmember = \IPS\Member::load( \IPS\Settings::i()->ms_cp_email_members );
				$email = \IPS\Email::buildFromContent( \IPS\Member::loggedIn()->language()->addToStack('ms_email_subject'), $emailMessage, $emailMessage, \IPS\Email::TYPE_TRANSACTIONAL );
				$result = $email->_send( $emailmember->email, array(), array(), \IPS\Member::loggedIn()->email );
			}
			catch( \Exception $ex ){}
		}
		*/
		
		$outputLang = \IPS\Member::loggedIn()->language()->addToStack('ms_custom_item_redeemed');
		
		if ( \IPS\Request::i()->isAjax() )
		{
			\IPS\Output::i()->json( array( 'message' => $outputLang ));
			return;
		}
		else
		{
			if ( $boughtAndUsed )
			{
				$redirectURL = $categoryURL;
			}
			else
			{
				$redirectURL = \IPS\Request::i()->referrer() ?: \IPS\Http\Url::internal( 'app=membersshop&module=shop&controller=myitems', 'front', 'myitems' );
			}
			
			\IPS\Output::i()->redirect( $redirectURL, 'ms_custom_item_redeemed' );
		}
	}
	
	protected function uploadAvatar( $getItem, $member, $boughtAndUsed, $categoryURL )
	{
		\IPS\Output::i()->jsFiles = array_merge( \IPS\Output::i()->jsFiles, \IPS\Output::i()->js( 'global_shop.js', 'membersshop', 'global' ) );
		
		$photoVars = explode( ':', $member->group['g_photo_max_vars'] );
		$form = new \IPS\Helpers\Form( 'membersshop', 'save' );
		$form->class = 'ipsForm_vertical ipsForm_fullWidth ipsForm_noLabels ipsBox';
		if ( $boughtAndUsed )
		{
			$base        = 'front';
			$seoTemplate = 'my_items_redeem';
			$seoTitles   = array( \IPS\Http\Url\Friendly::seoTitle( $getItem->name ) );
			$categoryID = \IPS\membersshop\Items::load( $getItem->item_id, 'item_id' )->cat_id;
			$url = \IPS\Http\Url::internal( "app=membersshop&module=shop&controller=myitems&do=redeem&id={$getItem->id}&b=1&c={$categoryID}", $base, $seoTemplate, $seoTitles  );
			$form->action = $url;
		}
		$form->add( new \IPS\Helpers\Form\Upload( 'member_photo_upload', NULL, FALSE, array( 'image' => array( 'maxWidth' => $photoVars[1], 'maxHeight' => $photoVars[2] ), 'storageExtension' => 'core_Profile', 'maxFileSize' => $photoVars[0] ? $photoVars[0] / 1024 : NULL ), function( $val ) {
			if ( $val instanceof \IPS\File )
			{
				$image = \IPS\Image::create( $val->contents() );
				if( $image->isAnimatedGif and !$member->group['g_upload_animated_photos'] )
				{
					throw new \DomainException('member_photo_upload_no_animated');
				}
			}
		}, NULL, NULL, 'member_photo_upload' ) );
		
		if( $values = $form->values() )
		{
			$member->pp_photo_type  = 'custom';
			$member->pp_main_photo  = (string) $values['member_photo_upload'];
			$thumbnail = $values['member_photo_upload']->thumbnail( 'core_Profile', \IPS\PHOTO_THUMBNAIL_SIZE, \IPS\PHOTO_THUMBNAIL_SIZE, TRUE );
			$member->pp_thumb_photo = (string) $thumbnail;
			$member->photo_last_update = time();
			$member->save();
		    
			$getItem->redeemed = 1;
		    $getItem->save();
			
			$reward = new \IPS\membersshop\Rewards;
			$reward->item_id = $getItem->item_id;
			$reward->member_id = $member->member_id;
			$reward->time_date = time();
			$reward->name = $getItem->name;
		    $reward->save();
			
			$outputLang = \IPS\Member::loggedIn()->language()->addToStack('ms_uploaded_avatar');
			if ( \IPS\Request::i()->isAjax() )
			{
				\IPS\Output::i()->json( array( 'message' => $outputLang ));
				return;
			}
			else
			{
				if ( $boughtAndUsed )
				{
					$redirectURL = $categoryURL;
				}
				else
				{
					$redirectURL = \IPS\Request::i()->referrer() ?: \IPS\Http\Url::internal( 'app=membersshop&module=shop&controller=myitems', 'front', 'myitems' );
				}
				
				\IPS\Output::i()->redirect( $redirectURL, 'ms_uploaded_avatar' );
			}
		}
		else
		{
			if ( \IPS\Request::i()->isAjax() )
			{
				\IPS\Output::i()->json( array(
					'contents' 	=> $form->customTemplate( array( \IPS\Theme::i()->getTemplate( 'shop', 'membersshop' ), 'settingsForm' ), $getItem->name )
				)) ;
				return;
			}
			else
			{
				\IPS\Output::i()->title = $getItem->name;
				\IPS\Output::i()->output = $form->customTemplate( array( \IPS\Theme::i()->getTemplate( 'shop', 'membersshop' ), 'settingsForm' ), $getItem->name );
			}
		}
	}
	
	protected function goAnonymous( $getItem, $member, $boughtAndUsed, $categoryURL )
	{
		\IPS\Session::i()->setAnon();
		
		$getItem->redeemed = 1;
		$getItem->save();
		
		$outputLang = \IPS\Member::loggedIn()->language()->addToStack('ms_gone_anonymous');
		
		if ( \IPS\Request::i()->isAjax() )
		{
			\IPS\Output::i()->json( array( 'message' => $outputLang ));
			return;
		}
		else
		{
			if ( $boughtAndUsed )
			{
				$redirectURL = $categoryURL;
			}
			else
			{
				$redirectURL = \IPS\Request::i()->referrer() ?: \IPS\Http\Url::internal( 'app=membersshop&module=shop&controller=myitems', 'front', 'myitems' );
			}
			
			\IPS\Output::i()->redirect( $redirectURL, 'ms_gone_anonymous' );
		}
	}
	
	protected function guesstheNumber( $getItem, $member, $boughtAndUsed, $categoryURL )
	{
		$pointsName = \IPS\Settings::i()->ms_points_name;
		$formHTML = \IPS\Member::loggedIn()->language()->addToStack('ms_guess_the_number_message', FALSE, array( 'sprintf' => array( number_format( $getItem->maxRand ), number_format( $getItem->minRand ), number_format( $getItem->price ), $pointsName, number_format( $getItem->price * $getItem->minRand ), $pointsName ) ) );
		
		\IPS\Output::i()->jsFiles = array_merge( \IPS\Output::i()->jsFiles, \IPS\Output::i()->js( 'global_shop.js', 'membersshop', 'global' ) );
		
		$form = new \IPS\Helpers\Form( 'membersshop', \IPS\Member::loggedIn()->language()->addToStack('ms_guess') );
		$form->class = 'ipsForm_vertical ipsForm_fullWidth ipsForm_noLabels ipsBox';
		
		if ( $boughtAndUsed )
		{
			$base        = 'front';
			$seoTemplate = 'my_items_redeem';
			$seoTitles   = array( \IPS\Http\Url\Friendly::seoTitle( $getItem->name ) );
			$categoryID = \IPS\membersshop\Items::load( $getItem->item_id, 'item_id' )->cat_id;
			$url = \IPS\Http\Url::internal( "app=membersshop&module=shop&controller=myitems&do=redeem&id={$getItem->id}&b=1&c={$categoryID}", $base, $seoTemplate, $seoTitles  );
			$form->action = $url;
		}
		
		$form->add( new \IPS\Helpers\Form\Number('ms_guess_number', NULL, FALSE, array( 'min' => 1, 'max' => $getItem->maxRand ), NULL, NULL, NULL, 'ms_guess_number' ));
		$form->addHtml( $formHTML );
		
		if( $values = $form->values() )
		{
			$reward = new \IPS\membersshop\Rewards;
			
			$rand = rand( 1, $getItem->maxRand );
			
			if ( $values['ms_guess_number'] > $rand )
			{
				$text = \IPS\Member::loggedIn()->language()->addToStack('ms_too_high');
			}
			elseif ( $values['ms_guess_number'] < $rand )
			{
				$text = \IPS\Member::loggedIn()->language()->addToStack('ms_too_low');
			}
			elseif ( $values['ms_guess_number'] = $rand )
			{
				$text = \IPS\Member::loggedIn()->language()->addToStack('ms_well_done_you_won');
				$winnings = $getItem->price * $getItem->minRand;
				$reward->reward = $winnings;
		        $logsACP = sprintf( \IPS\Member::loggedIn()->language()->get('ms_acp_log_won_guess_number'), $getItem->name );
				$member->addPoints( $winnings, $logsACP );
			}
			
			$reward->item_id = $getItem->item_id;
			$reward->member_id = $member->member_id;
			$reward->time_date = time();
			$reward->name = $getItem->name;
		    $reward->save();
			
			$getItem->redeemed = 1;
			$getItem->save();
			
			if ( \IPS\Request::i()->isAjax() )
			{
				\IPS\Output::i()->json( array( 'message' => $text ));
				return;
			}
			else
			{
				if ( $boughtAndUsed )
				{
					$redirectURL = $categoryURL;
				}
				else
				{
					$redirectURL = \IPS\Request::i()->referrer() ?: \IPS\Http\Url::internal( 'app=membersshop&module=shop&controller=myitems', 'front', 'myitems' );
				}
				\IPS\Output::i()->redirect( $redirectURL, $text );
			}
		}
		else
		{
			if ( \IPS\Request::i()->isAjax() )
			{
				\IPS\Output::i()->json( array(
					'contents' 	=> $form->customTemplate( array( \IPS\Theme::i()->getTemplate( 'shop', 'membersshop' ), 'settingsForm' ), $getItem->name )
				)) ;
				return;
			}
			else
			{
				\IPS\Output::i()->title = $getItem->name;
				\IPS\Output::i()->output = $form->customTemplate( array( \IPS\Theme::i()->getTemplate( 'shop', 'membersshop' ), 'settingsForm' ), $getItem->name );
			}
		}		
	}
	
	protected function profileVideo( $getItem, $member, $boughtAndUsed, $categoryURL )
	{
		\IPS\Output::i()->jsFiles = array_merge( \IPS\Output::i()->jsFiles, \IPS\Output::i()->js( 'global_shop.js', 'membersshop', 'global' ) );
		
		$form = new \IPS\Helpers\Form( 'membersshop', 'save' );
		$form->class = 'ipsForm_vertical ipsForm_fullWidth ipsForm_noLabels ipsBox';
		if ( $boughtAndUsed )
		{
			$base        = 'front';
			$seoTemplate = 'my_items_redeem';
			$seoTitles   = array( \IPS\Http\Url\Friendly::seoTitle( $getItem->name ) );
			$categoryID = \IPS\membersshop\Items::load( $getItem->item_id, 'item_id' )->cat_id;
			$url = \IPS\Http\Url::internal( "app=membersshop&module=shop&controller=myitems&do=redeem&id={$getItem->id}&b=1&c={$categoryID}", $base, $seoTemplate, $seoTitles  );
			$form->action = $url;
		}
		$form->add( new \IPS\Helpers\Form\Url( 'ProfileVideo', $member->profileVideo(), FALSE, array( 'maxLength' => 255 ) ) );
		
		if( $values = $form->values() )
		{
		    if( $member->profileVideoSettings() )
			{
				\IPS\Db::i()->update( 'profile_videos', array( 'location' => $values['ProfileVideo'] ), array( 'member_id=?', $member->member_id ) );
				
				if ( preg_match( '%(?:youtube(?:-nocookie)?\.com/(?:[^/]+/.+/|(?:v|e(?:mbed)?)/|.*[?&]v=)|youtu\.be/)([^"&?/ ]{11})%i', $values['ProfileVideo'], $match ) ) 
				{
        			$youtube_id = $match[\count( $match ) - 1];
					\IPS\Db::i()->update( 'profile_videos', array( 'location' => "https://www.youtube.com/embed/$youtube_id" ), array( 'member_id=?', $member->member_id ) );
    			}
			}
			else
			{
				\IPS\Db::i()->insert( 'profile_videos', array(
					'member_id'		=> $member->member_id,
					'location'		=> $values['ProfileVideo']
				) );
			}
		
			$getItem->redeemed = 1;
		    $getItem->save();
			
			$reward = new \IPS\membersshop\Rewards;
			$reward->item_id = $getItem->item_id;
			$reward->member_id = $member->member_id;
			$reward->time_date = time();
			$reward->name = $getItem->name;
		    $reward->save();
			
			$outputLang = \IPS\Member::loggedIn()->language()->addToStack('saved');
			
			if ( \IPS\Request::i()->isAjax() )
			{
				\IPS\Output::i()->json( array( 'message' => $outputLang ));
				return;
			}
			else
			{
				if ( $boughtAndUsed )
				{
					$redirectURL = $categoryURL;
				}
				else
				{
					$redirectURL = \IPS\Request::i()->referrer() ?: \IPS\Http\Url::internal( 'app=membersshop&module=shop&controller=myitems', 'front', 'myitems' );
				}
				
				\IPS\Output::i()->redirect( $redirectURL, 'saved' );
			}
		}
		else
		{
			if ( \IPS\Request::i()->isAjax() )
			{
				\IPS\Output::i()->json( array(
					'contents' 	=> $form->customTemplate( array( \IPS\Theme::i()->getTemplate( 'shop', 'membersshop' ), 'settingsForm' ), $getItem->name )
				)) ;
				return;
			}
			else
			{
				\IPS\Output::i()->title = $getItem->name;
				\IPS\Output::i()->output = $form->customTemplate( array( \IPS\Theme::i()->getTemplate( 'shop', 'membersshop' ), 'settingsForm' ), $getItem->name );
			}
		}
	}
	
	protected function profileBackground( $getItem, $member, $boughtAndUsed, $categoryURL )
	{
		\IPS\Output::i()->jsFiles = array_merge( \IPS\Output::i()->jsFiles, \IPS\Output::i()->js( 'global_shop.js', 'membersshop', 'global' ) );
		
		$form = new \IPS\Helpers\Form( 'membersshop', 'Upload' );
		$form->class = 'ipsForm_vertical ipsForm_fullWidth ipsForm_noLabels ipsBox';
		if ( $boughtAndUsed )
		{
			$base        = 'front';
			$seoTemplate = 'my_items_redeem';
			$seoTitles   = array( \IPS\Http\Url\Friendly::seoTitle( $getItem->name ) );
			$categoryID = \IPS\membersshop\Items::load( $getItem->item_id, 'item_id' )->cat_id;
			$url = \IPS\Http\Url::internal( "app=membersshop&module=shop&controller=myitems&do=redeem&id={$getItem->id}&b=1&c={$categoryID}", $base, $seoTemplate, $seoTitles  );
			$form->action = $url;
		}
		$form->add( new \IPS\Helpers\Form\Upload( 'ProfileBackgroundImage_Upload', $member->profileBackgroundImage() ? \IPS\File::get( 'core_Theme', $member->profileBackgroundImage() ) : NULL, FALSE, array( 'image' => true, 'storageExtension' => 'core_Theme' ), NULL, NULL, NULL, "ProfileBackgroundImage_Upload" ) );

		if( $values = $form->values() )
		{
		    if( $member->profileBackgroundSettings() )
			{
				\IPS\Db::i()->update( 'profile_backgrounds', array( 'location' => $values['ProfileBackgroundImage_Upload'] ), array( 'member_id=?', $member->member_id ) );
			}
			else
			{
				\IPS\Db::i()->insert( 'profile_backgrounds', array(
					'member_id'		=> $member->member_id,
					'location'		=> $values['ProfileBackgroundImage_Upload']
				) );
			}
			
			$getItem->redeemed = 1;
		    $getItem->save();
			
			$reward = new \IPS\membersshop\Rewards;
			$reward->item_id = $getItem->item_id;
			$reward->member_id = $member->member_id;
			$reward->time_date = time();
			$reward->name = $getItem->name;
		    $reward->save();
			
			if ( \IPS\Request::i()->isAjax() )
			{
				\IPS\Output::i()->json( array( 'message' => 'saved' ));
				return;
			}
			else
			{
				if ( $boughtAndUsed )
				{
					$redirectURL = $categoryURL;
				}
				else
				{
					$redirectURL = \IPS\Request::i()->referrer() ?: \IPS\Http\Url::internal( 'app=membersshop&module=shop&controller=myitems', 'front', 'myitems' );
				}
				
				\IPS\Output::i()->redirect( $redirectURL, 'saved' );
			}
		}
		else
		{
			if ( \IPS\Request::i()->isAjax() )
			{
				\IPS\Output::i()->json( array(
					'contents' 	=> $form->customTemplate( array( \IPS\Theme::i()->getTemplate( 'shop', 'membersshop' ), 'settingsForm' ), $getItem->name )
				)) ;
				return;
			}
			else
			{
				\IPS\Output::i()->title = $getItem->name;
				\IPS\Output::i()->output = $form->customTemplate( array( \IPS\Theme::i()->getTemplate( 'shop', 'membersshop' ), 'settingsForm' ), $getItem->name );
			}
		}
	}
	
	protected function stickyNote( $getItem, $member, $boughtAndUsed, $categoryURL )
	{
		\IPS\Output::i()->jsFiles = array_merge( \IPS\Output::i()->jsFiles, \IPS\Output::i()->js( 'global_shop.js', 'membersshop', 'global' ) );
		
		$form = new \IPS\Helpers\Form( 'stickynotes', 'send' );
		$form->class = 'ipsForm_vertical ipsForm_fullWidth ipsForm_noLabels ipsBox';
		if ( $boughtAndUsed )
		{
			$base        = 'front';
			$seoTemplate = 'my_items_redeem';
			$seoTitles   = array( \IPS\Http\Url\Friendly::seoTitle( $getItem->name ) );
			$categoryID = \IPS\membersshop\Items::load( $getItem->item_id, 'item_id' )->cat_id;
			$url = \IPS\Http\Url::internal( "app=membersshop&module=shop&controller=myitems&do=redeem&id={$getItem->id}&b=1&c={$categoryID}", $base, $seoTemplate, $seoTitles  );
			$form->action = $url;
		}
        $form->addTab('StickyNoteTab');
		$form->add(new \IPS\Helpers\Form\Member('StickyNoteMember', NULL, TRUE, array( ), function( $val )
        {
			if ( !\IPS\Settings::i()->SN_AllowSelfSend )
		    {
	            if ( $val == \IPS\Member::loggedIn() )
	            {
		            throw new \DomainException('SN_SendSelf');
	            }
		    }
		    
			if ( $val->blockreceiving == 1 )
			{
		        throw new \DomainException('SN_BlockReceiving');
	        }	
			
			try
			{
			    $blocked = \IPS\Db::i()->select( 'ignore_list', 'stickynotes_ignore', array( 'member_id=?', $val->member_id ) )->first();
				
				if ( \in_array( \IPS\Member::loggedIn()->member_id, explode( ',', ( $blocked ) ) ) )
			    {
				    throw new \DomainException('SN_Been_Blocked');
			    }
			}
			catch ( \UnderflowException $e )
		    {
			    $blocked = array();
		    }
			
        }
		));
		
		$form->add( new \IPS\Helpers\Form\Editor  ( 'StickyNote', NULL, TRUE, array( 'app' => 'stickynotes', 'key' => 'StickyNotes', 'autoSaveKey' => '', 'attachIds' => array( $member->member_id ) ) ) );
		
		if ( \IPS\Settings::i()->SN_PersonalAnon )
		{
		    $form->add( new \IPS\Helpers\Form\Select( 'SN_PersonalUseAnon', 'no', FALSE, array( 'options' => array_merge( array( 'no' => 'No', 'yes' => 'Yes' ) ) ) ) );
		}
		
		if ( $values = $form->values() )
		{
			$sender_id = $member->member_id;
		    $group_id = $member->member_group_id;
		    $member_id = $values['StickyNoteMember']->member_id;
		
		    if ( \IPS\Settings::i()->SN_PersonalAnon AND $values['SN_PersonalUseAnon'] == 'no' )
		    {
	            \IPS\Db::i()->insert( 'stickynotes_notes', array( 'note' => $values['StickyNote'], 'sender_id' => $sender_id, 'member_id' => $member_id, 'time_date' => time(), 'group_id' => $group_id, 'anonymous' => NULL, ) );
		    }
		    else if ( \IPS\Settings::i()->SN_PersonalAnon AND $values['SN_PersonalUseAnon'] == 'yes' )
		    {
	            \IPS\Db::i()->insert( 'stickynotes_notes', array( 'note' => $values['StickyNote'], 'sender_id' => $sender_id, 'member_id' => $member_id, 'time_date' => time(), 'group_id' => $group_id, 'anonymous' => 1, ) );
		    }
		    else 
		    {
	            \IPS\Db::i()->insert( 'stickynotes_notes', array( 'note' => $values['StickyNote'], 'sender_id' => $sender_id, 'member_id' => $member_id, 'time_date' => time(), 'group_id' => $group_id, 'anonymous' => NULL, ) );
		    }
			
			$getItem->redeemed = 1;
		    $getItem->save();
			
			$reward = new \IPS\membersshop\Rewards;
			$reward->item_id = $getItem->item_id;
			$reward->member_id = $member->member_id;
			$reward->time_date = time();
			$reward->name = $getItem->name;
		    $reward->save();
		    
			$outputLang = \IPS\Member::loggedIn()->language()->addToStack('sticky_note_sent');
			
			if ( \IPS\Request::i()->isAjax() )
			{
				\IPS\Output::i()->json( array( 'message' => $outputLang ));
				return;
			}
			else
			{
				if ( $boughtAndUsed )
				{
					$redirectURL = $categoryURL;
				}
				else
				{
					$redirectURL = \IPS\Request::i()->referrer() ?: \IPS\Http\Url::internal( 'app=membersshop&module=shop&controller=myitems', 'front', 'myitems' );
				}
				
				\IPS\Output::i()->redirect( $redirectURL, 'sticky_note_sent' );
			}
		}
		else
		{
			if ( \IPS\Request::i()->isAjax() )
			{
				\IPS\Output::i()->json( array(
					'contents' 	=> $form->customTemplate( array( \IPS\Theme::i()->getTemplate( 'shop', 'membersshop' ), 'settingsForm' ), $getItem->name )
				)) ;
				return;
			}
			else
			{
				\IPS\Output::i()->title = $getItem->name;
				\IPS\Output::i()->output = $form->customTemplate( array( \IPS\Theme::i()->getTemplate( 'shop', 'membersshop' ), 'settingsForm' ), $getItem->name );
			}
		}
	}
	
	protected function memberssocialInfo( $getItem, $member, $boughtAndUsed, $categoryURL )
	{
		\IPS\Output::i()->jsFiles = array_merge( \IPS\Output::i()->jsFiles, \IPS\Output::i()->js( 'global_shop.js', 'membersshop', 'global' ) );
		
		$member = \IPS\Member::loggedIn();
		
		$form = new \IPS\Helpers\Form;
		$form->class = 'ipsForm_vertical ipsForm_fullWidth ipsForm_noLabels ipsBox';
		if ( $boughtAndUsed )
		{
			$base        = 'front';
			$seoTemplate = 'my_items_redeem';
			$seoTitles   = array( \IPS\Http\Url\Friendly::seoTitle( $getItem->name ) );
			$categoryID = \IPS\membersshop\Items::load( $getItem->item_id, 'item_id' )->cat_id;
			$url = \IPS\Http\Url::internal( "app=membersshop&module=shop&controller=myitems&do=redeem&id={$getItem->id}&b=1&c={$categoryID}", $base, $seoTemplate, $seoTitles  );
			$form->action = $url;
		}
		
		foreach( \IPS\memberssocialinfo\Categories::roots() as $category )
		{
			foreach( $category->children() as $settings )
			{
				try
				{
					$memberSettingInfo = \IPS\Db::i()->select( 'site_info', 'memberssocialinfo_member_info', array( 'member_id=? AND social_site_id=?', $member->member_id, $settings->id ) )->first();
				}
				catch( \Exception $ex )
				{
					$memberSettingInfo = FALSE;
				}
				
				$member->language()->words[ 'msi_'.$settings->name.'' ] = $settings->name;
				$member->language()->words[ 'msi_'.$settings->name.'_desc' ] = $settings->desc;
				
				if( $settings->enabled )
				{
					$form->add( new \IPS\Helpers\Form\Text( 'msi_'.$settings->name.'', $memberSettingInfo ?: NULL, FALSE, array( 'placeholder' => $settings->placeholder ), NULL, NULL, NULL, $settings->cat_id ) );
				}
				$form->addSeparator();
			}
		}
	
		if ( $values = $form->values() )
		{
		    foreach( \IPS\memberssocialinfo\Sites::roots() as $settings )
			{
				if( isset( $values['msi_'.$settings->name.''] ) AND $values['msi_'.$settings->name.''] )
				{
					try
					{
						$memberSettingID = \IPS\Db::i()->select( 'id', 'memberssocialinfo_member_info', array( 'member_id=? AND social_site_id=?', $member->member_id, $settings->id ) )->first();
						\IPS\Db::i()->update( 'memberssocialinfo_member_info', array( 'site_info' => $values['msi_'.$settings->name.''] ), array( 'id=?', $memberSettingID ) );
					}
					catch( \Exception $ex )
					{
						\IPS\Db::i()->insert( 'memberssocialinfo_member_info', array( 'site_info' => $values['msi_'.$settings->name.''], 'member_id' => $member->member_id, 'social_site_id' => $settings->id ) );
					}
				}
				else
				{
					\IPS\Db::i()->delete( 'memberssocialinfo_member_info', array( 'member_id=?', $member->member_id ) );
				}
			}
			
			$getItem->redeemed = 1;
		    $getItem->save();
			
			$reward = new \IPS\membersshop\Rewards;
			$reward->item_id = $getItem->item_id;
			$reward->member_id = $member->member_id;
			$reward->time_date = time();
			$reward->name = $getItem->name;
		    $reward->save();
			
			if ( \IPS\Request::i()->isAjax() )
			{
				\IPS\Output::i()->json( array( 'message' => 'saved' ));
				return;
			}
			else
			{
				if ( $boughtAndUsed )
				{
					$redirectURL = $categoryURL;
				}
				else
				{
					$redirectURL = \IPS\Request::i()->referrer() ?: \IPS\Http\Url::internal( 'app=membersshop&module=shop&controller=myitems', 'front', 'myitems' );
				}
				
				\IPS\Output::i()->redirect( $redirectURL, 'saved' );
			}
		}
		else
		{
			if ( \IPS\Request::i()->isAjax() )
			{
				\IPS\Output::i()->json( array(
					'contents' 	=> $form->customTemplate( array( \IPS\Theme::i()->getTemplate( 'shop', 'membersshop' ), 'MSIsettingsForm' ), $getItem->name, $member )
				)) ;
				return;
			}
			else
			{
				\IPS\Output::i()->title = $getItem->name;
				\IPS\Output::i()->output = $form->customTemplate( array( \IPS\Theme::i()->getTemplate( 'shop', 'membersshop' ), 'MSIsettingsForm' ), $getItem->name, $member );
			}
		}
	}
	
	protected function customcode( $getItem, $member, $boughtAndUsed, $categoryURL )
	{
		$msgTitle = $getItem->name;
	    $msgPost  = \IPS\Member::loggedIn()->language()->addToStack('ms_custom_item_code_message', FALSE, array( 'sprintf' => array( $getItem->name, $getItem->extra_field_text ) ) );
		\IPS\Member::loggedIn()->language()->parseOutputForDisplay( $msgTitle );
		\IPS\Member::loggedIn()->language()->parseOutputForDisplay( $msgPost );
		try
		{
			$pmSender = \IPS\Member::load( \IPS\Settings::i()->ms_cc_pm_members );
		}
		catch( \OutOfRangeException $ex )
		{
	   	    return; 
		}
		if( !$pmSender->member_id )
		{
		    return;
		}
		$conversation = \IPS\core\Messenger\Conversation::createItem( $pmSender, $pmSender->ip_address, \IPS\DateTime::ts( time() ) );
		$conversation->title = $msgTitle;
		$conversation->is_system = TRUE;
		$conversation->to_member_id = $getItem->member_id;
		$conversation->save();
        $c_members[] = $getItem->member_id;
		$c_members[] = $pmSender->member_id;
		$conversation->authorize( $c_members );         
        $message = \IPS\core\Messenger\Message::create( $conversation, $msgPost, TRUE, NULL, NULL, $pmSender );
		$conversation->first_msg_id = $message->id;
		$conversation->save(); 
		
		$getItem->redeemed = 1;
		$getItem->save();
		
		$outputLang = \IPS\Member::loggedIn()->language()->addToStack('ms_check_pm');
		
		if ( \IPS\Request::i()->isAjax() )
		{
			\IPS\Output::i()->json( array( 'message' => $outputLang ));
			return;
		}
		else
		{
			if ( $boughtAndUsed )
			{
				$redirectURL = $categoryURL;
			}
			else
			{
				$redirectURL = \IPS\Request::i()->referrer() ?: \IPS\Http\Url::internal( 'app=membersshop&module=shop&controller=myitems', 'front', 'myitems' );
			}
			
			\IPS\Output::i()->redirect( $redirectURL, 'ms_check_pm' );
		}
	}
	
	protected function buytrophy( $getItem, $member, $boughtAndUsed, $categoryURL )
	{
		if ( \IPS\Application::appIsEnabled( 'trophies') )
		{
			try
			{
				$trophy = \IPS\trophies\Trophy::load( $getItem->minRand );	
			}
			catch ( \OutOfRangeException $e )
			{
				\IPS\Output::i()->error( 'node_error', 'MEMBERSSHOP/1', 404, '' );
			}
			$trophyCount =  (int) \IPS\Db::i()->select( 'count(*)', 'trophies_members', array( 'member_id=? AND trophy_id=?', $member->member_id, $getItem->minRand ) )->first();
			if ( $trophyCount > 0 )
			{
				\IPS\Output::i()->error( 'ms_trophy_got_error', 'MEMBERSSHOP/1', 403, '' );
			}
			else
			{
				$reward = new \IPS\membersshop\Rewards;
				$reward->item_id = $getItem->item_id;
				$reward->member_id = $member->member_id;
				$reward->time_date = time();
				$reward->name = $getItem->name;
				$reward->save();
				
				\IPS\Db::i()->insert( 'trophies_members', array ( 'member_id' => $member->member_id, 'date' => time(), 'data' => '[]', 'trophy_id' => $getItem->minRand, 'awarded_by' => 0, 'hidden' => 0, 'replaced_by_other_trophy' => 0 ) );
				
				$getItem->redeemed = 1;
				$getItem->save();
				$outputLang = ( $trophy->_title ) ? \IPS\Member::loggedIn()->language()->addToStack('ms_trophy_won', FALSE, array( 'sprintf' => array( $trophy->_title ) ) ) : 'saved';
   		        
				if ( \IPS\Request::i()->isAjax() )
				{
					\IPS\Output::i()->json( array( 'message' => $outputLang ));
					return;
				}
				else
				{
					if ( $boughtAndUsed )
					{
						$redirectURL = $categoryURL;
					}
					else
					{
						$redirectURL = \IPS\Request::i()->referrer() ?: \IPS\Http\Url::internal( 'app=membersshop&module=shop&controller=myitems', 'front', 'myitems' );
					}
					
					\IPS\Output::i()->redirect( $redirectURL, $outputLang );
				}
			}
		}
		else
		{
			\IPS\Output::i()->error( 'ms_trophies_error', 'MEMBERSHOP/1', 403, '' );
		}
	}
	
	protected function buymedal( $getItem, $member, $boughtAndUsed, $categoryURL )
	{
		if ( \IPS\Application::appIsEnabled( 'trophies') )
		{
			try
			{
				$medal = \IPS\trophies\Medals::load( $getItem->minRand );	
			}
			catch ( \OutOfRangeException $e )
			{
				\IPS\Output::i()->error( 'node_error', 'MEMBERSSHOP/1', 404, '' );
			}
			$medalCount =  (int) \IPS\Db::i()->select( 'count(*)', 'trophies_members', array( 'member_id=? AND medal_id=?', $member->member_id, $getItem->minRand ) )->first();
			if ( $medalCount > 0 )
			{
				\IPS\Output::i()->error( 'ms_medal_got_error', 'MEMBERSSHOP/1', 403, '' );
			}
			else
			{
				$reward = new \IPS\membersshop\Rewards;
				$reward->item_id = $getItem->item_id;
				$reward->member_id = $member->member_id;
				$reward->time_date = time();
				$reward->name = $getItem->name;
				$reward->save();
				
				\IPS\Db::i()->insert( 'trophies_members', array ( 'member_id' => $member->member_id, 'date' => time(), 'data' => '[]', 'medal_id' => $getItem->minRand, 'awarded_by' => 0, 'hidden' => 0, 'replaced_by_other_trophy' => 0 ) );
				
				$getItem->redeemed = 1;
				$getItem->save();
				$outputLang = ( $medal->_title ) ? \IPS\Member::loggedIn()->language()->addToStack('ms_trophy_won', FALSE, array( 'sprintf' => array( $medal->_title ) ) ) : 'saved';
   		        
				if ( \IPS\Request::i()->isAjax() )
				{
					\IPS\Output::i()->json( array( 'message' => $outputLang ));
					return;
				}
				else
				{
					if ( $boughtAndUsed )
					{
						$redirectURL = $categoryURL;
					}
					else
					{
						$redirectURL = \IPS\Request::i()->referrer() ?: \IPS\Http\Url::internal( 'app=membersshop&module=shop&controller=myitems', 'front', 'myitems' );
					}
					
					\IPS\Output::i()->redirect( $redirectURL, $outputLang );
				}
			}
		}
		else
		{
			\IPS\Output::i()->error( 'ms_trophies_error', 'MEMBERSHOP/1', 403, '' );
		}
	}
	
	protected function resetRep( $getItem, $member, $boughtAndUsed, $categoryURL )
	{
		$member->pp_reputation_points = 0;
		$member->save();
		
		$getItem->redeemed = 1;
		$getItem->save();
		
		$reward = new \IPS\membersshop\Rewards;
		$reward->item_id = $getItem->item_id;
		$reward->member_id = $member->member_id;
		$reward->time_date = time();
		$reward->name = $getItem->name;
		$reward->save();
		
		$outputLang = \IPS\Member::loggedIn()->language()->addToStack('ms_rep_reset_zero');
		
		if ( \IPS\Request::i()->isAjax() )
		{
			\IPS\Output::i()->json( array( 'message' => $outputLang ));
			return;
		}
		else
		{
			if ( $boughtAndUsed )
			{
				$redirectURL = $categoryURL;
			}
			else
			{
				$redirectURL = \IPS\Request::i()->referrer() ?: \IPS\Http\Url::internal( 'app=membersshop&module=shop&controller=myitems', 'front', 'myitems' );
			}
			
			\IPS\Output::i()->redirect( $redirectURL, 'ms_rep_reset_zero' );
		}
	}
	
	protected function mysteryItem( $getItem, $member, $boughtAndUsed, $categoryURL )
	{
		if ( $getItem->extra_field )
		{
			if ( $getItem->extra_field != -1 )
			{
				$winPercentage = $getItem->extra_field;
				$chance = rand( 1, 100 );
				if ( $chance > $winPercentage )
				{
					$getItem->redeemed = 1;
					$getItem->save();
					$outputLang = \IPS\Member::loggedIn()->language()->addToStack('ms_sorry_you_lost');
					
					if ( \IPS\Request::i()->isAjax() )
					{
						\IPS\Output::i()->json( array( 'message' => $outputLang ));
						return;
					}
					else
					{
						if ( $boughtAndUsed )
						{
							$redirectURL = $categoryURL;
						}
						else
						{
							$redirectURL = \IPS\Request::i()->referrer() ?: \IPS\Http\Url::internal( 'app=membersshop&module=shop&controller=myitems', 'front', 'myitems' );
						}
						
						\IPS\Output::i()->redirect( $redirectURL, $outputLang );
					}
				}
			}
		}
		
		$array = [];
		$array = explode( ",", $getItem->extra_field_text );
		shuffle( $array );
		
		try
		{
			$newItem = \IPS\membersshop\Items::load( $array[0] );
		}
		catch( \Exception $e )
		{
			try
			{
				$newItem = \IPS\membersshop\Items::load( $array[1] );
			}
			catch( \Exception $e )
			{
				try
				{
					$newItem = \IPS\membersshop\Items::load( $array[2] );
				}
				catch( \Exception $e )
				{
					try
					{
						$newItem = \IPS\membersshop\Items::load( $array[3] );
					}
					catch( \Exception $e )
					{
						try
						{
							$newItem = \IPS\membersshop\Items::load( $array[4] );
						}
						catch( \Exception $e )
						{
							\IPS\Output::i()->error( 'ms_mystery_item_error', 'MEMBERSHOP/1', 403, '' );
						}
					}
				}
			}
		}
		
		$getItem->redeemed = 1;
		$getItem->save();
		
		$purchase = new \IPS\membersshop\Purchases;
		$purchase->item_id = $newItem->item_id;
		$purchase->member_id = $member->member_id;
		$purchase->name = $newItem->name;
		$purchase->description = $newItem->desc;
		$purchase->icon = $newItem->icon;
		$purchase->time_date = time();
		$purchase->price = $newItem->price;
		$purchase->perm_id = $newItem->id;
		
		if ( $newItem->minRand )
		{
			$purchase->minRand = $newItem->minRand;
		}
		
		if ( $newItem->maxRand == 0 or $newItem->maxRand )
		{
			$purchase->maxRand = $newItem->maxRand;
		}
		
		if ( $newItem->extra_field == 0 or $newItem->extra_field )
		{
			$purchase->extra_field = $newItem->extra_field;
		}
		
		if ( $newItem->extra_field_text )
		{
			$purchase->extra_field_text = $newItem->extra_field_text;
		}
		
		$purchase->save();
		
		$reward = new \IPS\membersshop\Rewards;
		$reward->item_id = $getItem->item_id;
		$reward->member_id = $member->member_id;
		$reward->time_date = time();
		$reward->name = $getItem->name;
		$reward->extra_field = $newItem->name;
		$reward->save();
		
		$outputLang = \IPS\Member::loggedIn()->language()->addToStack('ms_congrats_won_item', FALSE, array( 'sprintf' => array( $newItem->name ) ) );
   	    
		if ( \IPS\Request::i()->isAjax() )
		{
			\IPS\Output::i()->json( array( 'message' => $outputLang ));
			return;
		}
		else
		{
			if ( $boughtAndUsed )
			{
				$redirectURL = $categoryURL;
			}
			else
			{
				$redirectURL = \IPS\Request::i()->referrer() ?: \IPS\Http\Url::internal( 'app=membersshop&module=shop&controller=myitems', 'front', 'myitems' );
			}
			
			\IPS\Output::i()->redirect( $redirectURL, $outputLang );
		}
	}
	
	protected function sellBack()
	{
		$member = \IPS\Member::loggedIn();
		$id  = \intval( \IPS\Request::i()->id );
		$pointsName = \IPS\Settings::i()->ms_points_name;
		
		try
		{
		    $getItem = \IPS\membersshop\Purchases::load( $id );
		}
		catch ( \OutOfRangeException $e )
		{
			if ( \IPS\Request::i()->isAjax() )
			{
				\IPS\Output::i()->json( array( 'message' => \IPS\Member::loggedIn()->language()->addToStack('node_error') ) );
				return;
			}
			\IPS\Output::i()->error( 'node_error', 'MEMBERSSHOP/1', 404, '' );
		}
		
		if ( $getItem->redeemed == 1 OR $getItem->member_id !== $member->member_id )
		{
			if ( \IPS\Request::i()->isAjax() )
			{
				\IPS\Output::i()->json( array( 'message' => 'Error' ) );
				return;
			}
			\IPS\Output::i()->error( 'ms_node_error', 'MEMBERSHOP/1', 403, '' );
		}
		
		$myNumber = $getItem->price;
		$percentToGet = \IPS\Settings::i()->ms_charge_sell_back;
		$percentInDecimal = $percentToGet / 100;
		$result = $percentInDecimal * $myNumber;
		$return = $myNumber - $result;
		
		if ( !$member->modPermission( 'membersshop_can_buy_free' ) )
		{
			$logsACP = sprintf( \IPS\Member::loggedIn()->language()->get('ms_acp_sold_item_back'), $getItem->name );
			$member->addPoints( $return, $logsACP );
		}
		
		$getItem->delete();
		
		if ( !$member->modPermission( 'membersshop_can_buy_free' ) )
		{
			$outputLang = \IPS\Member::loggedIn()->language()->addToStack('ms_sold_back', FALSE, array( 'sprintf' => array( $getItem->name, number_format( $return ), $pointsName ) ));
   		}
		else
		{
			$outputLang = \IPS\Member::loggedIn()->language()->addToStack('ms_mod_sold_back', FALSE, array( 'sprintf' => array( $getItem->name ) ));
		}
		
		if ( \IPS\Request::i()->isAjax() )
		{
			\IPS\Output::i()->json( array( 'message' => $outputLang ) );
			return;
		}
			
		$redirectURL = \IPS\Request::i()->referrer() ?: \IPS\Http\Url::internal( 'app=membersshop&module=shop&controller=myitems', 'front', 'myitems' );
		
		\IPS\Output::i()->redirect( $redirectURL, $outputLang );
	}
}