<?php

namespace IPS\membersshop\Gateway;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !\defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * Test Gateway
 */
class _Points extends \IPS\nexus\Gateway
{
	const SUPPORTS_REFUNDS = TRUE;
	const SUPPORTS_PARTIAL_REFUNDS = TRUE;
	
	/**
	 * Payment Screen Fields
	 *
	 * @param	\IPS\nexus\Invoice		$invoice	Invoice
	 * @param	\IPS\nexus\Money		$amount		The amount to pay now
	 * @param	\IPS\nexus\Customer		$member		The member the payment screen is for (if in the ACP charging to a member's card) or NULL for currently logged in member
	 * @param	array					$recurrings	Details about recurring costs
	 * @param	bool					$type		'checkout' means the cusotmer is doing this on the normal checkout screen, 'admin' means the admin is doing this in the ACP, 'card' means the user is just adding a card
	 * @return	array
	 */
	public function paymentScreen( \IPS\nexus\Invoice $invoice, \IPS\nexus\Money $amount, \IPS\nexus\Customer $member = NULL, $recurrings = array(), $type = 'checkout' )
	{
		$settings = json_decode( $this->settings, TRUE );
		$pointsName = \IPS\Settings::i()->ms_points_name;
		$memberPoints = \IPS\Member::load( $invoice->member->member_id )->shopPoints();
		$costInPoints = $settings['exchange_rate_'.$amount->currency.''] * (string) $amount->amount;
		
		\IPS\Member::loggedIn()->language()->words[ 'ms_gateway' ] = sprintf( \IPS\Member::loggedIn()->language()->get('ms_gateway_message'), number_format( $costInPoints ), $pointsName );
		\IPS\Member::loggedIn()->language()->words[ 'ms_gateway_msg' ] = sprintf( \IPS\Member::loggedIn()->language()->get('ms_pay_with_points'), number_format( $memberPoints ), $pointsName );
		return array(
			new \IPS\Helpers\Form\Radio( 'ms_gateway', 0, FALSE, array( 'options' => array(
				0	=> 'ms_gateway_msg',
			) ) )
		);
	}
	
	/**
	 * Capture
	 *
	 * @param	\IPS\nexus\Transaction	$transaction	Transaction
	 * @return	void
	 * @throws	\LogicException
	 */
	public function capture( \IPS\nexus\Transaction $transaction )
	{
		$member = \IPS\Member::load( $transaction->invoice->member->member_id );
		$pointsName = \IPS\Settings::i()->ms_points_name;
		$settings = json_decode( $this->settings, TRUE );
		$transactionAmount = (string) $transaction->amount->amount;
		$pointsToCharge = $transactionAmount * $settings[ 'exchange_rate_'.$transaction->amount->currency.'' ];
		
		if ( $member->shopPoints() < $pointsToCharge )
		{
		    \IPS\Output::i()->error( \IPS\Member::loggedIn()->language()->addToStack('ms_you_dont_have_enough_points', FALSE, array( 'sprintf' => array( $pointsName ) ) ), 'MEMBERSSHOP/1', 403, '' );
		}
		
		$logsACP = sprintf( \IPS\Member::loggedIn()->language()->get('ms_acp_log_item_purchase_with_points'), $transaction->invoice->title );
		$member->removePoints( $pointsToCharge, $logsACP );
		
		return TRUE;
	}
	
	/**
	 * Refund
	 *
	 * @param	\IPS\nexus\Transaction	$transaction	Transaction to be refunded
	 * @param	float|NULL				$amount			Amount to refund (NULL for full amount - always in same currency as transaction)
	 * @return	mixed									Gateway reference ID for refund, if applicable
	 * @throws	\Exception
 	 */
	public function refund( \IPS\nexus\Transaction $transaction, $amount = NULL )
	{
		$member = \IPS\Member::load( $transaction->invoice->member->member_id );
		$pointsName = \IPS\Settings::i()->ms_points_name;
		$settings = json_decode( $this->settings, TRUE );
		$transactionAmount = (string) $transaction->amount->amount;
		$pointsToRefund = $transactionAmount * $settings[ 'exchange_rate_'.$transaction->amount->currency.'' ];
		
		$logsACP = sprintf( \IPS\Member::loggedIn()->language()->get('ms_acp_log_item_refund_with_points'), $transaction->invoice->title );
		$member->addPoints( $pointsToRefund, $logsACP );
		
		return TRUE;
	}
	
	/* !ACP Configuration */
	
	/**
	 * Settings
	 *
	 * @param	\IPS\Helpers\Form	$form	The form
	 * @return	void
	 */
	public function settings( &$form )
	{
		$settings = json_decode( $this->settings, TRUE );
		$pointsName = \IPS\Settings::i()->ms_points_name;
		foreach( \IPS\nexus\Money::currencies() as $currency )
		{
			\IPS\Member::loggedIn()->language()->words[ 'shoppoints_exchange_rate_'.$currency.'' ] = sprintf( \IPS\Member::loggedIn()->language()->get('membersshop_points_per_currency'), $pointsName, $currency );
			\IPS\Member::loggedIn()->language()->words[ 'shoppoints_exchange_rate_'.$currency.'_desc' ] =  sprintf( \IPS\Member::loggedIn()->language()->get('membersshop_conversion_rate_desc'), $pointsName, $currency, $currency, $pointsName );
			$form->add( new \IPS\Helpers\Form\Number( 'shoppoints_exchange_rate_'.$currency.'', isset( $settings[ 'exchange_rate_'.$currency.'' ] ) ? $settings[ 'exchange_rate_'.$currency.'' ] : 100, TRUE ) );
		}
	}
	
	/**
	 * Test Settings
	 *
	 * @param	array	$settings	Settings
	 * @return	array
	 * @throws	\InvalidArgumentException
	 */
	public function testSettings( $settings )
	{
		return $settings;
	}
}