<?php

namespace IPS\membersshop;

if( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
    header( ( isset( $_SERVER[ 'SERVER_PROTOCOL' ] ) ? $_SERVER[ 'SERVER_PROTOCOL' ] :
			'HTTP/1.0' ) . ' 403 Forbidden' );
    exit;
}

class _Items extends \IPS\Node\Model
{
	protected static $multitons;
	
	public static $nodeTitle = 'ms_members_shop';
	
	public static $databaseTable = 'membersshop_items';
	
	public static $databasePrefix = '';
	
	public static $databaseColumnOrder = 'position';
	
	public static $parentNodeColumnId = 'cat_id';
	
	protected static $databaseIdFields = [ 'id', 'member_id', 'position', 'cat_id', 'item_id' ];
	
	protected function get__title()
    {
       return $this->name;
    }
	
	protected function get__enabled()
    {
        return $this->open;
    }
    
    protected function set__enabled( $enabled )
    {
        $this->open = $enabled;
    }
	
	public function get__icon()
	{
		$icon =  mb_substr( $this->icon, 3 );
		return $icon;
	}

	public function canAdd()
	{
		return FALSE;
	}
	
	public static function canAddRoot()
	{
		return TRUE;
	}
	
	public function get__description()
	{
		return FALSE;
	}
	
	public function canDelete()
	{
		return TRUE;
	}
	
	public function canCopy()
    {
        return TRUE;
    }
    
    public function delete()
    {
		parent::delete();
    }
	
	public function form( &$form, $options=array() )
    {
		if ( !$this->id )
		{
			$options = array();
			$options['1']   = \IPS\Member::loggedIn()->language()->addToStack( 'ms_item_post_count' );
			$options['2']   = \IPS\Member::loggedIn()->language()->addToStack( 'ms_item_rep_count' ); 
			$options['3']   = \IPS\Member::loggedIn()->language()->addToStack( 'ms_item_steal_rep' );
			$options['4']   = \IPS\Member::loggedIn()->language()->addToStack( 'ms_item_username' );
			$options['5']   = \IPS\Member::loggedIn()->language()->addToStack( 'ms_item_mystery_points_box' );
			$options['6']   = \IPS\Member::loggedIn()->language()->addToStack( 'ms_item_member_title' );
			$options['7']   = \IPS\Member::loggedIn()->language()->addToStack( 'ms_item_warning_points' );
			$options['8']   = \IPS\Member::loggedIn()->language()->addToStack( 'ms_item_forum_password' );
			$options['9']   = \IPS\Member::loggedIn()->language()->addToStack( 'ms_item_signature' );
			$options['10']  = \IPS\Member::loggedIn()->language()->addToStack( 'ms_item_rpsls' );
			$options['11']  = \IPS\Member::loggedIn()->language()->addToStack( 'ms_item_pin_topic' );
			$options['12']  = \IPS\Member::loggedIn()->language()->addToStack( 'ms_item_feature_file' ) ;
			$options['13']  = \IPS\Member::loggedIn()->language()->addToStack( 'ms_item_upgrade_usergroup' );
			
			$options['18']  = \IPS\Member::loggedIn()->language()->addToStack( 'ms_item_trophy' );
			$options['19']  = \IPS\Member::loggedIn()->language()->addToStack( 'ms_item_avatar' );
			$options['20']  = \IPS\Member::loggedIn()->language()->addToStack( 'ms_item_amonymous' );
			$options['21']  = \IPS\Member::loggedIn()->language()->addToStack( 'ms_item_guess_number' );
			$options['22']  = \IPS\Member::loggedIn()->language()->addToStack( 'ms_item_profile_video' );
			$options['23']  = \IPS\Member::loggedIn()->language()->addToStack( 'ms_item_profilebackground' );
			$options['24']  = \IPS\Member::loggedIn()->language()->addToStack( 'ms_item_sticky_note' );
			$options['25']  = \IPS\Member::loggedIn()->language()->addToStack( 'ms_item_members_social_info' );
			$options['26']  = \IPS\Member::loggedIn()->language()->addToStack( 'ms_item_game_code_coupon' );
			$options['27']  = \IPS\Member::loggedIn()->language()->addToStack( 'ms_item_buy_trophy' );
			$options['28']  = \IPS\Member::loggedIn()->language()->addToStack( 'ms_item_buy_medal' );
			$options['29']  = \IPS\Member::loggedIn()->language()->addToStack( 'ms_item_reset_rep' );
			$options['30']  = \IPS\Member::loggedIn()->language()->addToStack( 'ms_item_mystery_item_box' );
			$options['100'] = \IPS\Member::loggedIn()->language()->addToStack( 'ms_item_custom' );
			
			$extensions = \IPS\Application::allExtensions( 'membersshop', 'Items', FALSE );
			foreach ( $extensions as $k => $class )
			{
				if ( method_exists( $class, 'itemOptions' ) )
				{	
					$class->itemOptions( $options );
				}
			}
			
			$form = new \IPS\Helpers\Form( 'ms', 'Add Item' );
			$form->addHeader( 'ms_add_item_to_shop' );
			$form->class = 'ipsForm_horizontal ipsForm_fullWidth ipsForm_noLabels ipsBox';
			$form->add( new \IPS\Helpers\Form\Select( 'ms_add_item_to_shop', 25, TRUE, array( 'options' => $options, 'sort' => TRUE ) ) );		
		}
		else
		{
			
			$images = array();
			foreach( \IPS\Db::i()->select( 'location', 'membersshop_item_images', array( 'item_id=?', $this->id ) ) as $url )
			{
				$images[] = \IPS\File::get( 'membersshop_FileStorage', $url );
			}
			
			$form = new \IPS\Helpers\Form;
			$form->addTab( 'ms_item_setting_header' );
			$form->add( new \IPS\Helpers\Form\Text( 'ms_dName', $this->name, TRUE, array( 'placeholder' => '' ), NULL, NULL, NULL, 'ms_dName' ) );
			$form->add( new \IPS\Helpers\Form\Editor( 'ms_dDescription', $this->desc, TRUE, array( 'app' => 'membersshop', 'key' => 'membersshop', 'autoSaveKey' => 'membersshop_'.$this->id.'', 'attachIds' => NULL, 'minimize' => 'Click to enter a description', 'minimizeIcon' => 'fa fa-chevron-down' ), NULL, NULL, NULL, 'ms_dDescription' ) );
			$form->add( new \IPS\Helpers\Form\Text( 'ms_dFawesome', $this->icon, TRUE, array( ), NULL, NULL, NULL, 'ms_dFawesome' ) );
			
			$form->add( new \IPS\Helpers\Form\Upload( 'ms_images', $images, FALSE, array(
				'storageExtension'	=> 'membersshop_FileStorage',
				'image'				=> TRUE,
				'multiple'			=> TRUE,
			) ) );
			
			$form->add( new \IPS\Helpers\Form\Number( 'ms_dPrice', $this->price, TRUE, array( ), NULL, NULL, NULL, 'ms_dPrice' ) );
			
			$form->add( new \IPS\Helpers\Form\Number( 'ms_purchase_delay', $this->purchase_delay ?: 0, TRUE, array( 'min' => 1, 'unlimited' => 0, 'unlimitedLang' => \IPS\Member::loggedIn()->language()->addToStack( 'ms_set_purchase_delay' ) ), NULL, NULL, 'seconds', 'ms_purchase_delay' ) );
			
			if ( $this->item_id == 1 or $this->item_id == 2 or $this->item_id == 3 or $this->item_id == 5 )
			{
				$form->add( new \IPS\Helpers\Form\Number( 'ms_dMinRand', $this->minRand, TRUE, array( ), NULL, NULL, NULL, 'ms_dMinRand' ) );
				$form->add( new \IPS\Helpers\Form\Number( 'ms_dMaxRand', $this->maxRand, TRUE, array( ), NULL, NULL, NULL, 'ms_dMaxRand' ) );
			}
			
			if ( $this->item_id == 1 or $this->item_id == 2 or $this->item_id == 5 or $this->item_id == 30 )
			{
				$form->add( new \IPS\Helpers\Form\Number( 'ms_win_percent', $this->extra_field, TRUE, array( 'min' => 1, 'max' => 99, 'unlimited' => -1, 'unlimitedLang' => \IPS\Member::loggedIn()->language()->addToStack( 'ms_everyones_a_winner' ) ), NULL, NULL, '%', 'ms_win_percent' ) );
			}
		
			if ( $this->item_id == 10 )
			{
				$form->add( new \IPS\Helpers\Form\YesNo( 'ms_enable_increase', $this->extra_field, TRUE, array( 'togglesOn' => array( 'ms_increase_amount') ), NULL, NULL, NULL, 'ms_enable_increase' ) );
				$form->add( new \IPS\Helpers\Form\Number( 'ms_increase_amount', $this->maxRand, FALSE, array( 'min' => 0 ), NULL, NULL, \IPS\Member::loggedIn()->language()->addToStack( 'ms_points' ), 'ms_increase_amount' ) );
			}
		
			if ( $this->item_id == 11 )
			{
				$form->add( new \IPS\Helpers\Form\Number( 'ms_pin_time_days', $this->extra_field, TRUE, array( 'max' => 31, 'min' => 1 ), NULL, NULL, \IPS\Member::loggedIn()->language()->addToStack( 'ms_days' ), 'ms_pin_time_days' ) );
			}
		
			if ( $this->item_id == 12 )
			{
				$form->add( new \IPS\Helpers\Form\Number( 'ms_feature_time_days', $this->extra_field, TRUE, array( 'max' => 31, 'min' => 1 ), NULL, NULL, \IPS\Member::loggedIn()->language()->addToStack( 'ms_days' ), 'ms_feature_time_days' ) );
			}
		
			if ( $this->item_id == 13 or $this->item_id == 14 or $this->item_id == 15 )
			{
				$form->add( new \IPS\Helpers\Form\Select( 'ms_reward_user_group', $this->minRand, TRUE, array( 'options' => \IPS\Member\Group::groups( TRUE, FALSE ), 'parse' => 'normal' ) ) );
		        $form->add( new \IPS\Helpers\Form\Select( 'ms_select_reward_dmy', $this->maxRand, TRUE, array( 
				'options' => array(
					0  => 'Days',
					1  => 'Months',
					2  => 'Years',
					3  => 'Permanent'),
				 'toggles' => array(
					0 => array( 'ms_upgrade_time_days' ),
					1 => array( 'ms_upgrade_time_months' ),
					2 => array( 'ms_upgrade_time_years' ),
				) ), NULL, NULL, NULL, 'ms_select_reward_dmy'));	
	    		$form->add( new \IPS\Helpers\Form\Number( 'ms_upgrade_time_days', $this->extra_field, TRUE, array( 'min' => 0 ), NULL, NULL, \IPS\Member::loggedIn()->language()->addToStack( 'ms_days' ), 'ms_upgrade_time_days' ) );
			    $form->add( new \IPS\Helpers\Form\Number( 'ms_upgrade_time_months', $this->extra_field, TRUE, array( 'min' => 0 ), NULL, NULL, \IPS\Member::loggedIn()->language()->addToStack( 'ms_months' ), 'ms_upgrade_time_months' ) );
			    $form->add( new \IPS\Helpers\Form\Number( 'ms_upgrade_time_years', $this->extra_field, TRUE, array( 'min' => 0 ), NULL, NULL, \IPS\Member::loggedIn()->language()->addToStack( 'ms_years' ), 'ms_upgrade_time_years' ) );
			}
			
			if ( $this->item_id == 21 )
			{
				$form->add( new \IPS\Helpers\Form\Number( 'ms_multipleguess', $this->minRand, TRUE, array( ), NULL, NULL, NULL, 'ms_multipleguess' ) );
				$form->add( new \IPS\Helpers\Form\Number( 'ms_guessnumbermaxamount', $this->maxRand, TRUE, array( ), NULL, NULL, NULL, 'ms_guessnumbermaxamount' ) );
			}
			
			if ( $this->item_id == 26 )
			{
				$form->add( new \IPS\Helpers\Form\Text( 'ms_custom_code', $this->extra_field_text, TRUE, array( 'placeholder' => 'HDFJ-56D8-T0F2-29JD' ), NULL, NULL, NULL, 'ms_custom_code' ) );
			}
			
			if ( $this->item_id == 27 )
			{
				$form->add( new \IPS\Helpers\Form\Node( 'ms_pick_trophy', $this ? $this->minRand : 0, TRUE, array( 'class' => 'IPS\trophies\Trophy', 'subnodes' => TRUE ) ) );
			}
			
			if ( $this->item_id == 28 )
			{
				$form->add( new \IPS\Helpers\Form\Node( 'ms_pick_medal', $this ? $this->minRand : 0, TRUE, array( 'class' => 'IPS\trophies\Medals', 'subnodes' => TRUE ) ) );
			}
			
			if ( $this->item_id == 30 )
			{
				$form->add( new \IPS\Helpers\Form\Node( 'ms_select_items', $this ? explode( ",", $this->extra_field_text ) : NULL, TRUE, array( 'class' => 'IPS\membersshop\Items', 'multiple' => TRUE ) ) );
			}
			
			$extensions = \IPS\Application::allExtensions( 'membersshop', 'Items', FALSE );
			foreach ( $extensions as $k => $class )
			{
				if ( method_exists( $class, 'editFormSettings' ) )
				{
					$item = $this;
					$class->editFormSettings( $form, $item );
				}
			}
			
	        $form->add( new \IPS\Helpers\Form\Number( 'ms_stock', $this->instock, TRUE, array( 'unlimited' => -1 ), NULL, NULL, NULL, 'ms_stock' ) );
			
			$form->addTab( 'ms_item_permission_header' );
			$form->add( new \IPS\Helpers\Form\Select( 'ms_groups_view', ( $this->groups_view == '*' ) ? '*' : explode( ',', $this->groups_view ), TRUE, array( 'options' => \IPS\Member\Group::groups( TRUE, TRUE ), 'parse' => 'normal', 'multiple' => true, 'unlimited' => '*', 'unlimitedLang' => 'all_groups' ), NULL, NULL, NULL, 'ms_groups_view' ) );
			$form->add( new \IPS\Helpers\Form\Select( 'ms_groups_buy', ( $this->groups_buy == '*' ) ? '*' : explode( ',', $this->groups_buy ), TRUE, array( 'options' => \IPS\Member\Group::groups( TRUE, FALSE ), 'parse' => 'normal', 'multiple' => true, 'unlimited' => '*', 'unlimitedLang' => 'all_groups' ), NULL, NULL, NULL, 'ms_groups_buy' ) );
			$form->add( new \IPS\Helpers\Form\Select( 'ms_groups_send', ( $this->groups_send == '*' ) ? '*' : explode( ',', $this->groups_send ), TRUE, array( 'options' => \IPS\Member\Group::groups( TRUE, FALSE ), 'parse' => 'normal', 'multiple' => true, 'unlimited' => '*', 'unlimitedLang' => 'all_groups' ), NULL, NULL, NULL, 'ms_groups_send' ) );        
		}
	}
	
	public function formatFormValues( $values )
	{
		$member = \IPS\Member::loggedIn();
		if ( !$this->id )
		{
			if ( $values['ms_add_item_to_shop'] == 12 AND !\IPS\Application::appIsEnabled( 'downloads') )
			{
				\IPS\Output::i()->error( 'ms_downloads_error', 'MEMBERSHOP/1', 403, '' );
			}
			if (  $values['ms_add_item_to_shop'] == 18 AND !\IPS\Application::appIsEnabled( 'trophies') )
			{
				\IPS\Output::i()->error( 'ms_trophies_error', 'MEMBERSHOP/1', 403, '' );
			}
			try
			{
				$sql = \IPS\Db::i()->select( 'plugin_id', 'core_plugins', array( 'plugin_name=? AND plugin_enabled=1', 'Profile Video' ) )->first();
				$profileVideo = \IPS\Plugin::load( $sql );
			}
			catch( \Exception $e )
			{
			    $profileVideo = FALSE;
			}
			if (  $values['ms_add_item_to_shop'] == 22 AND $profileVideo == FALSE )
			{
				\IPS\Output::i()->error( 'ms_profile_video_error', 'MEMBERSHOP/1', 403, '' );
			}
			try
			{
				$sql2 = \IPS\Db::i()->select( 'plugin_id', 'core_plugins', array( 'plugin_name=? AND plugin_enabled=1', 'Profile Backgrounds' ) )->first();
				$profileBackground = \IPS\Plugin::load( $sql2 );
			}
			catch( \Exception $e )
			{
			    $profileBackground = FALSE;
			}
			if (  $values['ms_add_item_to_shop'] == 23 AND $profileBackground == FALSE )
			{
				\IPS\Output::i()->error( 'ms_profile_background_error', 'MEMBERSHOP/1', 403, '' );
			}
			if ( $values['ms_add_item_to_shop'] == 24 AND !\IPS\Application::appIsEnabled( 'stickynotes') )
			{
				\IPS\Output::i()->error( 'ms_stickynotes_error', 'MEMBERSHOP/1', 403, '' );
			}
			if ( $values['ms_add_item_to_shop'] == 25 AND !\IPS\Application::appIsEnabled( 'memberssocialinfo') )
			{
				\IPS\Output::i()->error( 'ms_memberssocialinfo_error', 'MEMBERSHOP/1', 403, '' );
			}
			if ( $values['ms_add_item_to_shop'] == 26 AND !\IPS\Settings::i()->ms_cc_pm_members )
			{
				\IPS\Output::i()->error( 'ms_pm_sender_settings', 'MEMBERSHOP/1', 403, '' );
			}
			if (  $values['ms_add_item_to_shop'] == 27 AND !\IPS\Application::appIsEnabled( 'trophies') )
			{
				\IPS\Output::i()->error( 'ms_trophies_error', 'MEMBERSHOP/1', 403, '' );
			}
			if (  $values['ms_add_item_to_shop'] == 28 AND !\IPS\Application::appIsEnabled( 'trophies') )
			{
				\IPS\Output::i()->error( 'ms_trophies_error', 'MEMBERSHOP/1', 403, '' );
			}
			if ( $values['ms_add_item_to_shop'] == 1 )
			{
				$this->name = \IPS\Member::loggedIn()->language()->get( 'ms_item_post_count' );
				$this->price = 1000;
				$this->open = 1;
				$this->purchases = 0;
				$this->desc = 'Gamble for a chance to add a random amount between 10 and 100 to your post count';
				$this->icon = 'fa-comment-o';
				$this->minRand = 10;
				$this->maxRand = 100;
				$this->instock = -1;
				$this->cat_id = \IPS\Request::i()->parent;
				$this->item_id = 1;
				$this->extra_field = 78;
				$this->groups_view = '*';
				$this->groups_buy = '*';
				$this->groups_send = '*';
				$this->save();
			}
			if ( $values['ms_add_item_to_shop'] == 2 )
			{
				$this->name = \IPS\Member::loggedIn()->language()->get( 'ms_item_rep_count' );
				$this->price = 1000;
				$this->open = 1;
				$this->purchases = 0;
				$this->desc = 'Gamble for a chance to add a random amount between 10 and 100 to your profiles reputation';
				$this->icon = 'fa-heart';
				$this->minRand = 10;
				$this->maxRand = 100;
				$this->instock = -1;
				$this->cat_id = \IPS\Request::i()->parent;
				$this->item_id = 2;
				$this->extra_field = 78;
				$this->groups_view = '*';
				$this->groups_buy = '*';
				$this->groups_send = '*';
				$this->save();
			}
			if ( $values['ms_add_item_to_shop'] == 3 )
			{
				$this->name = \IPS\Member::loggedIn()->language()->get( 'ms_item_steal_rep' );
				$this->price = 2500;
				$this->open = 1;
				$this->purchases = 0;
				$this->desc = 'Steal between 1 and 100 reputaion points from your chosen member';
				$this->icon = 'fa-user-secret';
				$this->minRand = 1;
				$this->maxRand = 100;
				$this->instock = -1;
				$this->cat_id = \IPS\Request::i()->parent;
				$this->item_id = 3;
				$this->groups_view = '*';
				$this->groups_buy = '*';
				$this->groups_send = '*';
				$this->save();
			}
			if ( $values['ms_add_item_to_shop'] == 4 )
			{
				$this->name = \IPS\Member::loggedIn()->language()->get( 'ms_item_username' );
				$this->price = 2500;
				$this->open = 1;
				$this->purchases = 0;
				$this->desc = 'Change your username at any given time';
				$this->icon = 'fa-user-o';
				$this->instock = -1;
				$this->cat_id = \IPS\Request::i()->parent;
				$this->item_id = 4;
				$this->groups_view = '*';
				$this->groups_buy = '*';
				$this->groups_send = '*';
				$this->save();
			}
			if ( $values['ms_add_item_to_shop'] == 5 )
			{
				$this->name = \IPS\Member::loggedIn()->language()->get( 'ms_item_mystery_points_box' );
				$this->price = 1250;
				$this->open = 1;
				$this->purchases = 0;
				$this->desc = 'Gamble your points and open a mystery box for a chance to win between 10 and 2,500 points';
				$this->icon = 'fa-gift';
				$this->minRand = 10;
				$this->maxRand = 2500;
				$this->instock = -1;
				$this->cat_id = \IPS\Request::i()->parent;
				$this->item_id = 5;
				$this->extra_field = 78;
				$this->groups_view = '*';
				$this->groups_buy = '*';
				$this->groups_send = '*';
				$this->save();
			}
			if ( $values['ms_add_item_to_shop'] == 6 )
			{
				$this->name = \IPS\Member::loggedIn()->language()->get( 'ms_item_member_title' );
				$this->price = 1750;
				$this->open = 1;
				$this->purchases = 0;
				$this->desc = 'Change your member title at any given time';
				$this->icon = 'fa-address-card';
				$this->instock = -1;
				$this->cat_id = \IPS\Request::i()->parent;
				$this->item_id = 6;
				$this->groups_view = '*';
				$this->groups_buy = '*';
				$this->groups_send = '*';
				$this->save();
			}
			if ( $values['ms_add_item_to_shop'] == 7 )
			{
				$this->name = \IPS\Member::loggedIn()->language()->get( 'ms_item_warning_points' );
				$this->price = 25000;
				$this->open = 1;
				$this->purchases = 0;
				$this->desc = 'This will reset your warning points back to zero';
				$this->icon = 'fa-exclamation-triangle';
				$this->instock = -1;
				$this->cat_id = \IPS\Request::i()->parent;
				$this->item_id = 7;
				$this->groups_view = '*';
				$this->groups_buy = '*';
				$this->groups_send = '*';
				$this->save();
			}
			if ( $values['ms_add_item_to_shop'] == 8 )
			{
				$this->name = \IPS\Member::loggedIn()->language()->get( 'ms_item_forum_password' );
				$this->price = 30000;
				$this->open = 1;
				$this->purchases = 0;
				$this->desc = 'Gain access to locked forums by getting their passwords';
				$this->icon = 'fa-hashtag';
				$this->instock = -1;
				$this->cat_id = \IPS\Request::i()->parent;
				$this->item_id = 8;
				$this->groups_view = '*';
				$this->groups_buy = '*';
				$this->groups_send = '*';
				$this->save();
			}
			if ( $values['ms_add_item_to_shop'] == 9 )
			{
				$this->name = \IPS\Member::loggedIn()->language()->get( 'ms_item_signature' );
				$this->price = 5000;
				$this->open = 1;
				$this->purchases = 0;
				$this->desc = 'This will allow you to add a signature';
				$this->icon = 'fa-edit';
				$this->instock = -1;
				$this->cat_id = \IPS\Request::i()->parent;
				$this->item_id = 9;
				$this->groups_view = '*';
				$this->groups_buy = '*';
				$this->groups_send = '*';
				$this->save();
			}
			if ( $values['ms_add_item_to_shop'] == 10 )
			{
				$this->name = \IPS\Member::loggedIn()->language()->get( 'ms_item_rpsls' );
				$this->price = 500;
				$this->open = 1;
				$this->purchases = 0;
				$this->desc = 'Play the famous rock paper scissors lizard spock game';
				$this->icon = 'fa-hand-spock-o';
				$this->instock = -1;
				$this->cat_id = \IPS\Request::i()->parent;
				$this->item_id = 10;
				$this->groups_view = '*';
				$this->groups_buy = '*';
				$this->groups_send = '*';
				$this->save();
			}
			if ( $values['ms_add_item_to_shop'] == 11 )
			{
				$this->name = \IPS\Member::loggedIn()->language()->get( 'ms_item_pin_topic' );
				$this->price = 1000;
				$this->open = 1;
				$this->purchases = 0;
				$this->desc = 'Pin any topic for 7 days';
				$this->icon = 'fa-map-pin';
				$this->extra_field = 7;
				$this->instock = -1;
				$this->cat_id = \IPS\Request::i()->parent;
				$this->item_id = 11;
				$this->groups_view = '*';
				$this->groups_buy = '*';
				$this->groups_send = '*';
				$this->save();
			}
			if ( $values['ms_add_item_to_shop'] == 12 )
			{
				$this->name = \IPS\Member::loggedIn()->language()->get( 'ms_item_feature_file' );
				$this->price = 1000;
				$this->open = 1;
				$this->purchases = 0;
				$this->desc = 'Feature a uploaded file for 7 days';
				$this->icon = 'fa-download';
				$this->extra_field = 7;
				$this->instock = -1;
				$this->cat_id = \IPS\Request::i()->parent;
				$this->item_id = 12;
				$this->groups_view = '*';
				$this->groups_buy = '*';
				$this->groups_send = '*';
				$this->save();
			}
			if ( $values['ms_add_item_to_shop'] == 13 )
			{
				$this->name = \IPS\Member::loggedIn()->language()->get( 'ms_item_upgrade_usergroup' );
				$this->price = 20000;
				$this->open = 1;
				$this->purchases = 0;
				$this->desc = 'Upgrade your usergroup to XXX for 31 days';
				$this->icon = 'fa-user';
				$this->minRand = 3;
				$this->maxRand = 0;
				$this->extra_field = 31;
				$this->instock = -1;
				$this->cat_id = \IPS\Request::i()->parent;
				$this->item_id = 13;
				$this->groups_view = '*';
				$this->groups_buy = '*';
				$this->groups_send = '*';
				$this->save();
			}
			if ( $values['ms_add_item_to_shop'] == 18 )
			{
				$this->name = \IPS\Member::loggedIn()->language()->get( 'ms_item_trophy' );
				$this->price = 10000;
				$this->open = 1;
				$this->purchases = 0;
				$this->desc = 'Try your luck to win a random trophy';
				$this->icon = 'fa-trophy';
				$this->instock = -1;
				$this->cat_id = \IPS\Request::i()->parent;
				$this->item_id = 18;
				$this->groups_view = '*';
				$this->groups_buy = '*';
				$this->groups_send = '*';
				$this->save();
			}
			if ( $values['ms_add_item_to_shop'] == 19 )
			{
				$this->name = \IPS\Member::loggedIn()->language()->get( 'ms_item_avatar' );
				$this->price = 1250;
				$this->open = 1;
				$this->purchases = 0;
				$this->desc = 'Upload a avatar to your profile';
				$this->icon = 'fa-image';
				$this->instock = -1;
				$this->cat_id = \IPS\Request::i()->parent;
				$this->item_id = 19;
				$this->groups_view = '*';
				$this->groups_buy = '*';
				$this->groups_send = '*';
				$this->save();
			}
			if ( $values['ms_add_item_to_shop'] == 20 )
			{
				$this->name = \IPS\Member::loggedIn()->language()->get( 'ms_item_amonymous' );
				$this->price = 2500;
				$this->open = 1;
				$this->purchases = 0;
				$this->desc = 'Browse the site while appearing offline';
				$this->icon = 'fa-user-secret';
				$this->instock = -1;
				$this->cat_id = \IPS\Request::i()->parent;
				$this->item_id = 20;
				$this->groups_view = '*';
				$this->groups_buy = '*';
				$this->groups_send = '*';
				$this->save();
			}
			if ( $values['ms_add_item_to_shop'] == 21 )
			{
				$this->name = \IPS\Member::loggedIn()->language()->get( 'ms_item_guess_number' );
				$this->price = 500;
				$this->open = 1;
				$this->purchases = 0;
				$this->desc = 'Win 5x times your stake if you guess the number between 1 and 500';
				$this->icon = 'fa-question-circle';
				$this->minRand = 5;
				$this->maxRand = 500;
				$this->instock = -1;
				$this->cat_id = \IPS\Request::i()->parent;
				$this->item_id = 21;
				$this->groups_view = '*';
				$this->groups_buy = '*';
				$this->groups_send = '*';
				$this->save();
			}
			if ( $values['ms_add_item_to_shop'] == 22 )
			{
				$this->name = \IPS\Member::loggedIn()->language()->get( 'ms_item_profile_video' );
				$this->price = 500;
				$this->open = 1;
				$this->purchases = 0;
				$this->desc = 'Embed a video to add to your profile from any of the top video streaming sites';
				$this->icon = 'fa-video-camera';
				$this->instock = -1;
				$this->cat_id = \IPS\Request::i()->parent;
				$this->item_id = 22;
				$this->groups_view = '*';
				$this->groups_buy = '*';
				$this->groups_send = '*';
				$this->save();
			}
			if ( $values['ms_add_item_to_shop'] == 23 )
			{
				$this->name = \IPS\Member::loggedIn()->language()->get( 'ms_item_profilebackground' );
				$this->price = 500;
				$this->open = 1;
				$this->purchases = 0;
				$this->desc = 'Upload a image to use as a background on your profile';
				$this->icon = 'fa-image';
				$this->instock = -1;
				$this->cat_id = \IPS\Request::i()->parent;
				$this->item_id = 23;
				$this->groups_view = '*';
				$this->groups_buy = '*';
				$this->groups_send = '*';
				$this->save();
			}
			if ( $values['ms_add_item_to_shop'] == 24 )
			{
				$this->name = \IPS\Member::loggedIn()->language()->get( 'ms_item_sticky_note' );
				$this->price = 100;
				$this->open = 1;
				$this->purchases = 0;
				$this->desc = 'Send a personal Sticky Note to another member';
				$this->icon = 'fa-sticky-note-o';
				$this->instock = -1;
				$this->cat_id = \IPS\Request::i()->parent;
				$this->item_id = 24;
				$this->groups_view = '*';
				$this->groups_buy = '*';
				$this->groups_send = '*';
				$this->save();
			}
			if ( $values['ms_add_item_to_shop'] == 25 )
			{
				$this->name = \IPS\Member::loggedIn()->language()->get( 'ms_item_members_social_info' );
				$this->price = 100;
				$this->open = 1;
				$this->purchases = 0;
				$this->desc = 'Add / Edit your social info links to add to your profile / hovercard & topic posts';
				$this->icon = 'fa-users';
				$this->instock = -1;
				$this->cat_id = \IPS\Request::i()->parent;
				$this->item_id = 25;
				$this->groups_view = '*';
				$this->groups_buy = '*';
				$this->groups_send = '*';
				$this->save();
			}
			if ( $values['ms_add_item_to_shop'] == 26 )
			{
				$this->name = \IPS\Member::loggedIn()->language()->get( 'ms_item_game_code_coupon' );
				$this->price = 10000;
				$this->open = 1;
				$this->purchases = 0;
				$this->desc = 'Redeem a game code or a coupon code';
				$this->icon = 'fa-ticket';
				$this->instock = 1;
				$this->cat_id = \IPS\Request::i()->parent;
				$this->item_id = 26;
				$this->groups_view = '*';
				$this->groups_buy = '*';
				$this->groups_send = '*';
				$this->save();
			}
			if ( $values['ms_add_item_to_shop'] == 27 )
			{
				$this->name = \IPS\Member::loggedIn()->language()->get( 'ms_item_buy_trophy' );
				$this->price = 10000;
				$this->open = 1;
				$this->purchases = 0;
				$this->desc = 'Buy A Trophy';
				$this->icon = 'fa-trophy';
				$this->instock = -1;
				$this->cat_id = \IPS\Request::i()->parent;
				$this->item_id = 27;
				$this->groups_view = '*';
				$this->groups_buy = '*';
				$this->groups_send = '*';
				$this->save();
			}
			if ( $values['ms_add_item_to_shop'] == 28 )
			{
				$this->name = \IPS\Member::loggedIn()->language()->get( 'ms_item_buy_medal' );
				$this->price = 5000;
				$this->open = 1;
				$this->purchases = 0;
				$this->desc = 'Buy A Medal';
				$this->icon = 'fa-certificate';
				$this->instock = -1;
				$this->cat_id = \IPS\Request::i()->parent;
				$this->item_id = 28;
				$this->groups_view = '*';
				$this->groups_buy = '*';
				$this->groups_send = '*';
				$this->save();
			}
			if ( $values['ms_add_item_to_shop'] == 29 )
			{
				$this->name = \IPS\Member::loggedIn()->language()->get( 'ms_item_reset_rep' );
				$this->price = 50000;
				$this->open = 1;
				$this->purchases = 0;
				$this->desc = 'Reset Your Reputation';
				$this->icon = 'fa-remove';
				$this->instock = -1;
				$this->cat_id = \IPS\Request::i()->parent;
				$this->item_id = 29;
				$this->groups_view = '*';
				$this->groups_buy = '*';
				$this->groups_send = '*';
				$this->save();
			}
			if ( $values['ms_add_item_to_shop'] == 30 )
			{
				$this->name = \IPS\Member::loggedIn()->language()->get( 'ms_item_mystery_item_box' );
				$this->price = 25000;
				$this->open = 1;
				$this->purchases = 0;
				$this->desc = 'Gamble for a chance to win a mystery item';
				$this->icon = 'fa-gift';
				$this->instock = -1;
				$this->cat_id = \IPS\Request::i()->parent;
				$this->item_id = 30;
				$this->extra_field = 78;
				$this->groups_view = '*';
				$this->groups_buy = '*';
				$this->groups_send = '*';
				$this->save();
			}
			if ( $values['ms_add_item_to_shop'] == 100 )
			{
				$this->name = \IPS\Member::loggedIn()->language()->get( 'ms_item_custom' );
				$this->price = 100000;
				$this->open = 1;
				$this->purchases = 0;
				$this->desc = 'Enter your description here';
				$this->icon = 'fa-bell';
				$this->instock = -1;
				$this->cat_id = \IPS\Request::i()->parent;
				$this->item_id = 100;
				$this->groups_view = '*';
				$this->groups_buy = '*';
				$this->groups_send = '*';
				$this->save();
			}
			
			$extensions = \IPS\Application::allExtensions( 'membersshop', 'Items', FALSE );
			foreach ( $extensions as $k => $class )
			{
				if ( method_exists( $class, 'itemFormSubmitted' ) )
				{
					$item = $this;
					$class->itemFormSubmitted( $values, $item  );
				}
			}
			
			\IPS\Output::i()->redirect( \IPS\Http\Url::internal( 'app=membersshop&module=shop&controller=items&subnode=1&do=form&id='.$this->id.'' ), 'saved' );
		}
		else
		{
			/* Save standard values */
		    $this->icon = $values['ms_dFawesome'];
			$this->name = $values['ms_dName'];
			$this->desc = $values['ms_dDescription'];
			$this->price = $values['ms_dPrice'];
			$this->purchase_delay = $values['ms_purchase_delay'];
			$this->instock = $values['ms_stock'];
			
			if ( $values['ms_images'] )
			{
				\IPS\Db::i()->delete( 'membersshop_item_images', array( 'item_id=?', $this->id ) );
			
				foreach ( $values['ms_images'] as $val )
				{
					$images = new \IPS\membersshop\Items\Images;
					$images->location = $val;
					$images->item_id = $this->id;
					$images->save();
				}
			}
			
			if( isset( $values['ms_groups_view'] ) )
			{
				$values['ms_groups_view'] = $values['ms_groups_view'] === '*' ? '*' : implode( ',', $values['ms_groups_view'] );
			}
			if( isset( $values['ms_groups_buy'] ) )
			{
				$values['ms_groups_buy'] = $values['ms_groups_buy'] === '*' ? '*' : implode( ',', $values['ms_groups_buy'] );
			}
			if( isset( $values['ms_groups_send'] ) )
			{
				$values['ms_groups_send'] = $values['ms_groups_send'] === '*' ? '*' : implode( ',', $values['ms_groups_send'] );
			}
			$this->groups_view = $values['ms_groups_view'];
			$this->groups_buy = $values['ms_groups_buy'];
			$this->groups_send = $values['ms_groups_send'];
			
			/* Save random items values */
			if ( $this->item_id == 1 or $this->item_id == 2 or $this->item_id == 3 or $this->item_id == 5 )
			{
				$this->minRand = $values['ms_dMinRand'];
				$this->maxRand = $values['ms_dMaxRand'];
			}
			
			if ( $this->item_id == 1 or $this->item_id == 2 or $this->item_id == 5 or $this->item_id == 30 )
			{
				$this->extra_field = $values['ms_win_percent'];
			}
			
			/* Save rpsls increase values */
			if ( $this->item_id == 10 )
			{
				$this->extra_field = $values['ms_enable_increase'];
				$this->maxRand = $values['ms_increase_amount'];
			}
			
			/* Save pinned values */
			if ( $this->item_id == 11 )
			{
				$this->extra_field = $values['ms_pin_time_days'];
			}
			
			/* Save featured values */
			if ( $this->item_id == 12 )
			{
				$this->extra_field = $values['ms_feature_time_days'];
			}
			
			/* Save member upgrade values */
			if ( $this->item_id == 13 or $this->item_id == 14 or $this->item_id == 15  )
			{
				if ( $values['ms_select_reward_dmy'] == 0 )
				{
					$this->extra_field = $values['ms_upgrade_time_days'];
					$this->minRand = $values['ms_reward_user_group'];
					$this->maxRand = $values['ms_select_reward_dmy'];
				}
				elseif ( $values['ms_select_reward_dmy'] == 1 )
				{
					$this->extra_field = $values['ms_upgrade_time_months'];
					$this->minRand = $values['ms_reward_user_group'];
					$this->maxRand =  $values['ms_select_reward_dmy'];
				}
				elseif ( $values['ms_select_reward_dmy'] == 2 ) 
				{
					$this->extra_field = $values['ms_upgrade_time_years'];
					$this->minRand = $values['ms_reward_user_group'];
					$this->maxRand = $values['ms_select_reward_dmy'];
				}
				elseif ( $values['ms_select_reward_dmy'] == 3 ) 
				{
					$this->extra_field = NULL;
					$this->minRand = $values['ms_reward_user_group'];
					$this->maxRand = $values['ms_select_reward_dmy'];
				}
			}
				
			if ( $this->item_id == 21 )
			{
				$this->minRand = $values['ms_multipleguess'];
				$this->maxRand = $values['ms_guessnumbermaxamount'];
			}
			
			if ( $this->item_id == 26 )
			{
				$this->extra_field_text = $values['ms_custom_code'];
			}
			
			if ( $this->item_id == 27 )
			{
				$this->minRand = $values['ms_pick_trophy']->id;
			}
			
			if ( $this->item_id == 28 )
			{
				$this->minRand = $values['ms_pick_medal']->id;
			}
			
			if ( $this->item_id == 30 )
			{
				foreach ( array( 'ms_select_items' ) as $k )
				{
					if( isset( $values[ $k ] ) )
					{
						$values[ $k ] = \is_array( $values[ $k ] ) ? implode(',', array_keys( $values[ $k ] ) ) : '*';
					}
				}
				$this->extra_field_text = $values[ $k ];
			}
			
			$extensions = \IPS\Application::allExtensions( 'membersshop', 'Items', FALSE );
			foreach ( $extensions as $k => $class )
			{
				if ( method_exists( $class, 'processEditForm' ) )
				{
					$item = $this;
					$class->processEditForm( $item, $values );
				}
			}
			
			$this->save();
			
			\IPS\File::claimAttachments( $this->id, $member->member_id, 'membersshop' );
			\IPS\Output::i()->redirect( \IPS\Http\Url::internal( 'app=membersshop&module=shop&controller=items' ), 'saved' );
        }
	}
	
	public function getButtons( $url, $subnode = false )
    {
        $buttons = parent::getButtons( $url, $subnode );
		$buttons[ 'generate' ] = [
           	'icon'  => 'ravelry',
           	'title' => \IPS\Member::loggedIn()->language()->addToStack('ms_custom_purchase'),
           	'link'  => \IPS\Http\Url::internal( "app=membersshop&module=shop&controller=items&do=doCustom&id={$this->id}" ),
           	'data'  => [ ]
        ];
		/*
		$buttons[ 'stats' ] = [
           	'icon'  => 'eye',
           	'title' => \IPS\Member::loggedIn()->language()->addToStack('ms_purchasers'),
           	'link'  => \IPS\Http\Url::internal( "app=membersshop&module=shop&controller=items&do=purchasers&id={$this->_id}" ),
           	'data'  => [ 'ipsDialog' => '', 'ipsDialog-title' => \IPS\Member::loggedIn()->language()->addToStack( 'ms_purchasers' ) ]
        ];
		*/
        
        return $buttons;
    }
	
	public static function search( $column, $query, $order=NULL, $where=array() )
	{
		if ( $column === '_title' )
		{
			$column	= 'name';
		}

		if( $order == '_title' )
		{
			$order	= 'name';
		}

		return parent::search( $column, $query, $order, $where );
	} 
}