<?php
/**
 * @brief		addDailyPoints Task
 * @author		<a href='https://www.invisioncommunity.com'>Invision Power Services, Inc.</a>
 * @copyright	(c) Invision Power Services, Inc.
 * @license		https://www.invisioncommunity.com/legal/standards/
 * @package		Invision Community
 * @subpackage	membersshop
 * @since		29 Nov 2017
 */

namespace IPS\membersshop\tasks;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * addDailyPoints Task
 */
class _addDailyPoints extends \IPS\Task
{
	/**
	 * Execute
	 *
	 * If ran successfully, should return anything worth logging. Only log something
	 * worth mentioning (don't log "task ran successfully"). Return NULL (actual NULL, not '' or 0) to not log (which will be most cases).
	 * If an error occurs which means the task could not finish running, throw an \IPS\Task\Exception - do not log an error as a normal log.
	 * Tasks should execute within the time of a normal HTTP request.
	 *
	 * @return	mixed	Message to log or NULL
	 * @throws	\IPS\Task\Exception
	 */
	public function execute()
	{
		try
		{
			$pointsName = \IPS\Settings::i()->ms_points_name;
			$task = \IPS\Settings::i()->ms_task_points;
			$time = time();
			$oneDayAgo = \IPS\DateTime::create()->sub( new \DateInterval( 'P1D' ) )->getTimestamp();
			foreach ( \IPS\Db::i()->select( '*', 'membersshop_group_permissions' ) as $groups )
			{
				$add = $groups['daily_points'];
				if ( $groups['daily_points'] )
				{
					foreach ( \IPS\Db::i()->select( '*', 'core_members', array( ''.$task.' BETWEEN ? AND ? AND member_group_id=?', $oneDayAgo, $time, $groups['group_id'] ) ) as $members )
					{
						$logsACP = sprintf( \IPS\Member::loggedIn()->language()->get('ms_acp_daily_points'), $pointsName );
						$member = \IPS\Member::load( $members['member_id'] );
						$member->addPoints( $add, $logsACP );
					}
				}
			}
			return;
		}
		catch( \Exception $e )
		{
			return 'Task Failed';
		}
	}
	
	/**
	 * Cleanup
	 *
	 * If your task takes longer than 15 minutes to run, this method
	 * will be called before execute(). Use it to clean up anything which
	 * may not have been done
	 *
	 * @return	void
	 */
	public function cleanup()
	{
		
	}
}