<?php
/**
 * @brief		bankInterest Task
 * @author		<a href='https://www.invisioncommunity.com'>Invision Power Services, Inc.</a>
 * @copyright	(c) Invision Power Services, Inc.
 * @license		https://www.invisioncommunity.com/legal/standards/
 * @package		Invision Community
 * @subpackage	membersshop
 * @since		17 Mar 2018
 */

namespace IPS\membersshop\tasks;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * bankInterest Task
 */
class _bankInterest extends \IPS\Task
{
	/**
	 * Execute
	 *
	 * If ran successfully, should return anything worth logging. Only log something
	 * worth mentioning (don't log "task ran successfully"). Return NULL (actual NULL, not '' or 0) to not log (which will be most cases).
	 * If an error occurs which means the task could not finish running, throw an \IPS\Task\Exception - do not log an error as a normal log.
	 * Tasks should execute within the time of a normal HTTP request.
	 *
	 * @return	mixed	Message to log or NULL
	 * @throws	\IPS\Task\Exception
	 */
	public function execute()
	{
		
		try
		{
			foreach ( \IPS\Db::i()->select( '*', 'membersshop_bank', array ( 'member_id IS NOT NULL' ) ) as $bank )
			{
				$member = \IPS\Member::load( $bank['member_id'] );
				$group = \IPS\Member\Group::load( $member->member_group_id );
				$banking = \IPS\membersshop\Bank::load( $member->member_id, 'member_id' );
				
				$myNumber = $banking->points;
				$percentToGet = $group->bankInterest();
				$percentInDecimal = $percentToGet / 100;
				$amount = $percentInDecimal * $myNumber;
				$banking->points = $banking->points + $amount;
				$banking->save();
			}
			return;
		}
		catch( \Exception $e )
		{
			return 'Nothing Done';
		}
	}
	
	/**
	 * Cleanup
	 *
	 * If your task takes longer than 15 minutes to run, this method
	 * will be called before execute(). Use it to clean up anything which
	 * may not have been done
	 *
	 * @return	void
	 */
	public function cleanup()
	{
		
	}
}