<?php
/**
 * @brief		mostpoints Widget
 * @author		<a href='https://www.invisioncommunity.com'>Invision Power Services, Inc.</a>
 * @copyright	(c) Invision Power Services, Inc.
 * @license		https://www.invisioncommunity.com/legal/standards/
 * @package		Invision Community
 * @subpackage	membersshop
 * @since		11 Feb 2018
 */

namespace IPS\membersshop\widgets;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * mostpoints Widget
 */
class _mostpoints extends \IPS\Widget
{
	/**
	 * @brief	Widget Key
	 */
	public $key = 'mostpoints';
	
	/**
	 * @brief	App
	 */
	public $app = 'membersshop';
		
	/**
	 * @brief	Plugin
	 */
	public $plugin = '';
	
	/**
	 * Initialise this widget
	 *
	 * @return void
	 */ 
	public function init()
	{
		$this->template( array( \IPS\Theme::i()->getTemplate( 'widgets', $this->app, 'front' ), $this->key ) );
		parent::init();
	}
	
	/**
	 * Specify widget configuration
	 *
	 * @param	null|\IPS\Helpers\Form	$form	Form object
	 * @return	null|\IPS\Helpers\Form
	 */
	public function configuration( &$form=null )
	{
 		if ( $form === null )
		{
	 		$form = new \IPS\Helpers\Form;
 		}

 		$form->add( new \IPS\Helpers\Form\Number( 'ms_widget_show_amount', isset( $this->configuration['ms_widget_show_amount'] ) ? $this->configuration['ms_widget_show_amount'] : 5, TRUE ) );
 		return $form;
 	} 
 	
 	 /**
 	 * Ran before saving widget configuration
 	 *
 	 * @param	array	$values	Values from form
 	 * @return	array
 	 */
 	public function preConfig( $values )
 	{
 		return $values;
 	}

	/**
	 * Render a widget
	 *
	 * @return	string
	 */
	public function render()
	{
		$limit = isset( $this->configuration['ms_widget_show_amount'] ) ? $this->configuration['ms_widget_show_amount'] : 5;
		
		$sort = 'membersshop_points.points DESC';
		
		$select = \IPS\Db::i()->select(
			'*',
			'membersshop_points',
			NULL,
			$sort,
			$limit
		);
		
		return $this->output( $select, $limit );
	}
}